package jip;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.io.*;
import java.util.zip.*;

import jip.*;
import jiputil.*;

/**<P><FONT COLOR="red">
*<B>Descripcin:</B><BR>
*<FONT COLOR="blue">
* Clase que calcula y muestra los resultados del histograma para la secuencia o 
* el frame seleccionado. Tambien puede generar el resultado como imagen o cono 
* fichero ASCII. 
*/

public class HistogramWindow extends JFrame implements ActionListener {
  
    /** Barra de Men */
    JMenuBar menuBar; 
    /** Imagen de entrada*/
    JIPImage img;
    /** Imagen de salida */
    JIPImage res;
    /** Etiqueta de Intensidad */
    JLabel intensidad;
    /** Numero de Ocurrencias */
    JLabel ocurrencias;
    /** Etiqueta para valores de Intensidad*/
    JLabel in;
    /** Etiqueta para Valores de Ocurrencias */
    JLabel oc;
    /** Cota Mxima */
    JLabel cota_maxima;
    /** Panel */
    JPanel pane;    
    /** Mouse Listener */
    MyMouseListener mml;
    /** Variable Auxiliar */
    int max;
    /** Nmero de Items */
    int nItems;
    /** Variable Auxiliar. Indica si existe algun histograma calculado */
    int his;
    /** Variable Auxiliar. Indica si existe histograma de luminosidad  */
    int hLum;
    /** Variable Auxiliar. Indica si existe histograma de rojo */
    int hRed;
    /** Variable Auxiliar. Indica si existe histograma de verde */
    int hGreen;
    /** Variable Auxiliar. Indica si existe histograma de azul */
    int hBlue;
    /** Variable Auxiliar. Indica si existe histograma de Byte */
    int hByte;
    /** Variable Auxiliar. Indica si existe histograma de Word */
    int hWord;
    /** Variable Auxiliar. Indica si existe histograma de Real */
    int hReal;
    /** Variable Auxiliar. Indica el numero de intensidades de tipo real */
    int nIntR;
    /** Variable Auxiliar */
    int maximo;
    /** Variable Auxiliar */
    int div;
    /** Array de puntos */
		int   [] ptos;
		/** Array de puntos (enteros)*/
		int   [] ptosW;
		/** Array de puntos (flotantes)*/
		float [] ptosF;
		/** Array de ocurrencias e intensidades */
		RealDataHistogram [] ptosR;
    
    /**<P><FONT COLOR="red">
		*<B>Descripcin:</B><BR>
		*<FONT COLOR="blue">
    * Contructor de la clase. En el constructor se inicializan las estructuras de 
    * datos necesarias para la realizacion de los calculos. Algunas estructuras son
    * dependientes del tipo de la imagen a tratar. Otras son comunes para cualquier
    * tipo de imagen. Los tipos tratados son: [tCOLOR, tBYTE, tBIT, tWORD y tREAL]
    * Segun el tipo:
    * [tCOLOR]: Este tipo genera un menu con cuatro histogramas distintos. Estos son, 
    * Histograma de luminosidad, resultado de la media de los histogramas RGB.
    * Histograma RED, Histograma GREEN, Histograma BLUE, para las tres bandas correspondientes.
    * [tBYTE, tBIT, tWORD, tREAL]: Para estos tipos solo se genera un histograma completo.
    * Dentro de cada menu existen dos opciones, Guardar ASCII y Guardar JIP, estas opciones
    * son accesibles una vez hayamos calculado algun histograma y permiten salvar los datos del
    * mismo como fichero ASCII o como imagen respectivamente.
    * Estructuras comunes:
    * Las estructuras comunes son aquellas que nos indican datos numericos sobre el histograma.
    * Estas son comunes a todos los tipos y su descripcion es la siguiente:
    * Intensidad:  Muestra la intensidad selecionada.
    * Ocurrencias: Muestra el numero total de ocurrencias en % para la intensidad selecionada.
    * Maximo:      Muestra el numero maximo de ocurrencias dado en la imagen.
    * @param image Objeto JIPImage a tratar perteneciente a la secuencia
    */
    public HistogramWindow(JIPImage image){
      
       super("Histogramas de: "+image.getName());
       
       img=image;
       nIntR=0;
       max=1;
       nItems=0;
       div=256;
       his=0;
       hLum=0;
       hRed=0;
       hGreen=0;
       hBlue=0;
       hByte=0;
       hWord=0;
       hReal=0;
       maximo=0;

       ptos=new int [256];
       ptosW=new int [65536];
       int t=img.getType();
       res=null;
        
			 pane = new JPanel();	 
       menuBar=new JMenuBar();
       setJMenuBar(menuBar);
       
       ImageIcon icon1 = new ImageIcon("icons/colors.jpg");
			 ImageIcon icon2 = new ImageIcon("icons/ocurrencias.jpg");
			 ImageIcon icon3 = new ImageIcon("icons/guardar_ascii.jpg");
			 ImageIcon icon4 = new ImageIcon("icons/guardar_frame.jpg");
 	  
              
       if (t==JIP.tCOLOR){
         JMenu menu = new JMenu("Histogramas (RGB)");
         menuBar.add(menu);
         JMenuItem menuItem; 

				 menuItem=new JMenuItem("Luminosidad");
       	 menuItem.addActionListener(this);
       	 menu.add(menuItem); 
	       menuItem=new JMenuItem("Histograma RED");
  	     menuItem.addActionListener(this);
    	   menu.add(menuItem);
      	 menuItem=new JMenuItem("Histograma GREEN");
	       menuItem.addActionListener(this);
  	     menu.add(menuItem);       
    	   menuItem=new JMenuItem("Histograma BLUE");
	       menuItem.addActionListener(this);
  	     menu.add(menuItem);
  	     
  	     menuItem=new JMenuItem("Guardar ASCII",icon3);
         menuItem.addActionListener(this);
       	 menu.add(menuItem);
			 
			   menuItem=new JMenuItem("Guardar JIP",icon4);
       	 menuItem.addActionListener(this);
       	 menu.add(menuItem);

       }
       else
       	 if (t==JIP.tBYTE){
	         JMenu menu = new JMenu("Histograma (0..256)");
	         menuBar.add(menu);
	         JMenuItem menuItem; 
	
	       	 menuItem=new JMenuItem("Histograma Completo");
	       	 menuItem.addActionListener(this);
	       	 menu.add(menuItem);
	       	 
	       	 menuItem=new JMenuItem("Guardar ASCII",icon3);
			   	 menuItem.addActionListener(this);
			     menu.add(menuItem);
			 
					 menuItem=new JMenuItem("Guardar JIP",icon4);
			     menuItem.addActionListener(this);
			     menu.add(menuItem);

       	 }
       	 else
       	 	if (t==JIP.tWORD){
      	 		JMenu menu = new JMenu("Histograma (0..65536)");
		        menuBar.add(menu);
		        JMenuItem menuItem; 
		
		       	menuItem=new JMenuItem("Histograma Completo");
		       	menuItem.addActionListener(this);
		       	menu.add(menuItem);
		       	
		        menuItem=new JMenuItem("Guardar ASCII",icon3);
		       	menuItem.addActionListener(this);
		       	menu.add(menuItem);
			 
					  menuItem=new JMenuItem("Guardar JIP",icon4);
		       	menuItem.addActionListener(this);
		       	menu.add(menuItem);

       	 	}    
       	 	else
	       	 	if (t==JIP.tREAL){
		     	 		JMenu menu = new JMenu("Histograma (0..1)");
			        menuBar.add(menu);
			        JMenuItem menuItem; 
			
			       	menuItem=new JMenuItem("Histograma Completo");
			       	menuItem.addActionListener(this);
			       	menu.add(menuItem);

  	       	  menuItem=new JMenuItem("Guardar ASCII",icon3);
			       	menuItem.addActionListener(this);
			       	menu.add(menuItem);
			 
						  menuItem=new JMenuItem("Guardar JIP",icon4);
			       	menuItem.addActionListener(this);
			       	menu.add(menuItem);

	       	 	}
	     
	     pane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

       ScrollablePicture pic = new ScrollablePicture(new ImageIcon(),5);
  		 JScrollPane pictureScrollPane = new JScrollPane(pic);
						
			 pane.add(pictureScrollPane);
    	 setContentPane(pane);
			 
			 cota_maxima = new JLabel("Maximo       :");
			 intensidad  = new JLabel("Intensidad   : ",icon1,JLabel.LEFT);
       ocurrencias = new JLabel("Ocurrencias %: ",icon2,JLabel.LEFT);
       in = new JLabel("?");
       oc = new JLabel("?");

       pane.add(intensidad);
       pane.add(in);
       pane.add(ocurrencias);
       pane.add(oc);
       pane.add(cota_maxima);
       
       pane.setLayout(new BoxLayout(pane,BoxLayout.Y_AXIS));

       nItems=6;
       
       setContentPane(pane);
       
       mml=new MyMouseListener();       
    }
    
    /**<P><FONT COLOR="red">
		*<B>Descripcin:</B><BR>
		*<FONT COLOR="blue">
    * Metodo para la relizacion de los calculos segun el evento dado. 
    * @param e , evento tratado, segun dicho evento se realizaran los
    * siguientes acciones:
    * Guardar ASCII: el histograma mostrado por pantalla se guardara en modo 
    * texto en el fichero creado para tal efecto y cuyo nombre dependera del tipo 
    * de imagen y del histograma calculado para ella. Cuando se seleciona esta 
    * opcion del menu no se calcula el histograma, para que esta opcion funcione 
    * debe haber un  histograma ya calculado. En el caso de que no haya ningun 
    * histograma la opcion no hace nada.
    * Guardar JIP: Esta opcion, como la anterior, graba el histograma mostrado
    * en forma de secuencia JIP con un solo frame, cuyo tipo es el mismo que el de
    * la imagen de entrada. El nombre del fichero viene dado segun el nombre de la imagen
    * y el tipo de histograma selecionado.
    */
    
    public void actionPerformed(ActionEvent e)  {
  
			Image img2=null;

		 	if(img!=null) {
				int w=img.getWidth();
				int h=img.getHeight();
				int b=img.getNumBands();
				int t=img.getType();
				int x=0,y=0,nb=0;
				float fp;
				JFileChooser abreFich = new JFileChooser();
        String path,nfile=null;
				
				if (e.getActionCommand().equals("Guardar ASCII") && his==1){
					
					String ascii;
					FileWriter fOut;
					
					if (img!=null){
						
						ascii=null;
										
						if (t==JIP.tCOLOR){
							if (hLum==1){
								ascii=(img.getName())+"_lum.jhi";
							}else
								if (hRed==1){
									ascii=(img.getName())+"_red.jhi";
								}else
									if (hGreen==1){
										ascii=(img.getName())+"_green.jhi";
									}else
										if (hBlue==1){
											ascii=(img.getName())+"_blue.jhi";
										}	
										
						if (ascii!=null){						
							path=".\\";
							nfile=ascii;
							abreFich.setCurrentDirectory(new File(path));
	            abreFich.setSelectedFile(new File(nfile));
						  int returnVal = abreFich.showSaveDialog(this);
        			if (returnVal == abreFich.APPROVE_OPTION){
       					File fich = abreFich.getSelectedFile();
	              nfile=fich.getAbsolutePath();
	              
	              if (nfile!=null){
	              	if (!nfile.toLowerCase().endsWith(".jhi"))
						      	nfile=nfile+".jhi";
									try{
										System.out.println("Guardando "+nfile+"...");
										fOut=new FileWriter(nfile);
										for (x=0;x<256;x++)
											fOut.write(x+"       "+ptos[x]+"\n");	
										fOut.close();
										System.out.println("Fichero "+nfile+" guardado OK");
									}catch(Exception err) {}
	              }
        			}        			
						}

						}else
							if (t==JIP.tBYTE){
								if (hByte==1){
									ascii=(img.getName())+"_byte.jhi";
									path=".\\";
									nfile=ascii;
									abreFich.setCurrentDirectory(new File(path));
			            abreFich.setSelectedFile(new File(nfile));
								  int returnVal = abreFich.showSaveDialog(this);
		        			if (returnVal == abreFich.APPROVE_OPTION){
		       					File fich = abreFich.getSelectedFile();
			              nfile=fich.getAbsolutePath();
			              			              
			              if (nfile!=null){
			              	if (!nfile.toLowerCase().endsWith(".jhi"))
								      	nfile=nfile+".jhi";
											try{
												System.out.println("Guardando "+nfile+"...");
												fOut=new FileWriter(nfile);
												for (x=0;x<256;x++)
													fOut.write(x+"       "+ptos[x]+"\n");	
												fOut.close();
												System.out.println("Fichero "+nfile+" guardado OK");
											}catch(Exception err) {}
			              }
		        			}		        			
								}
							}else
								if (t==JIP.tWORD){
									if (hWord==1){
										ascii=(img.getName())+"_word.jhi";
										path=".\\";
										nfile=ascii;
										abreFich.setCurrentDirectory(new File(path));
				            abreFich.setSelectedFile(new File(nfile));
									  int returnVal = abreFich.showSaveDialog(this);
			        			if (returnVal == abreFich.APPROVE_OPTION){
			       					File fich = abreFich.getSelectedFile();
				              nfile=fich.getAbsolutePath();
				              
				              if (nfile!=null){
				              	if (!nfile.toLowerCase().endsWith(".jhi"))
									      	nfile=nfile+".jhi";
												try{
													System.out.println("Guardando "+nfile+"...");
													fOut=new FileWriter(nfile);
													for (x=0;x<65536;x++)
														fOut.write(x+"       "+ptosW[x]+"\n");
													fOut.close();
													System.out.println("Fichero "+nfile+" guardado OK");
												}catch(Exception err) {}
				              }
			        			}			        
									}
								}else
									if (t==JIP.tREAL){
										if (hReal==1){
											ascii=(img.getName())+"_real.jhi";
											path=".\\";
											nfile=ascii;
											abreFich.setCurrentDirectory(new File(path));
					            abreFich.setSelectedFile(new File(nfile));
										  int returnVal = abreFich.showSaveDialog(this);
				        			if (returnVal == abreFich.APPROVE_OPTION){
				       					File fich = abreFich.getSelectedFile();
					              nfile=fich.getAbsolutePath();
					              
					              if (nfile!=null){
					              	if (!nfile.toLowerCase().endsWith(".jhi"))
										      	nfile=nfile+".jhi";
													try{
														System.out.println("Guardando "+nfile+"...");
														fOut=new FileWriter(nfile);
														for (x=0;x<nIntR;x++)
															fOut.write(ptosR[x].getIValue()+"       "+ptosR[x].getOValue()+"\n");
														fOut.close();
														System.out.println("Fichero "+nfile+" guardado OK");
													}catch(Exception err) {}
					              }
				        			}				        		
										}
									}
									repaint();					
					}
				}
				
				if (e.getActionCommand().equals("Guardar JIP") && his==1){
					
					JIPToolkit jt=new JIPToolkit();

					if (t==JIP.tCOLOR){
						if (hLum==1){
							nfile=(img.getName())+"_H_lum.jip";
							path=".\\";							
							abreFich.setCurrentDirectory(new File(path));
					    abreFich.setSelectedFile(new File(nfile));
							int returnVal = abreFich.showSaveDialog(this);
				      if (returnVal == abreFich.APPROVE_OPTION){
					      File fich = abreFich.getSelectedFile();
						    nfile=fich.getAbsolutePath();
					      if (!nfile.toLowerCase().endsWith(".jip"))
					      	nfile=nfile+".jip";
								jt.saveImageIntoFile(res,nfile);
								System.out.println("Guardando : "+nfile);
				      }
						}else
							if (hRed==1){
								nfile=(img.getName())+"_H_red.jip";
								path=".\\";							
								abreFich.setCurrentDirectory(new File(path));
						    abreFich.setSelectedFile(new File(nfile));
								int returnVal = abreFich.showSaveDialog(this);
					      if (returnVal == abreFich.APPROVE_OPTION){
						      File fich = abreFich.getSelectedFile();
							    nfile=fich.getAbsolutePath();
						      if (!nfile.toLowerCase().endsWith(".jip"))
						      	nfile=nfile+".jip";
									jt.saveImageIntoFile(res,nfile);
									System.out.println("Guardando : "+nfile);
					      }								
							}else
								if (hGreen==1){
									nfile=(img.getName())+"_H_green.jip";
									path=".\\";							
									abreFich.setCurrentDirectory(new File(path));
							    abreFich.setSelectedFile(new File(nfile));
									int returnVal = abreFich.showSaveDialog(this);
						      if (returnVal == abreFich.APPROVE_OPTION){
							      File fich = abreFich.getSelectedFile();
								    nfile=fich.getAbsolutePath();
							      if (!nfile.toLowerCase().endsWith(".jip"))
							      	nfile=nfile+".jip";
										jt.saveImageIntoFile(res,nfile);
										System.out.println("Guardando : "+nfile);
						      }								
								}else
									if (hBlue==1){
										nfile=(img.getName())+"_H_blue.jip";
										path=".\\";							
										abreFich.setCurrentDirectory(new File(path));
								    abreFich.setSelectedFile(new File(nfile));
										int returnVal = abreFich.showSaveDialog(this);
							      if (returnVal == abreFich.APPROVE_OPTION){
								      File fich = abreFich.getSelectedFile();
									    nfile=fich.getAbsolutePath();
								      if (!nfile.toLowerCase().endsWith(".jip"))
								      	nfile=nfile+".jip";
											jt.saveImageIntoFile(res,nfile);
											System.out.println("Guardando : "+nfile);
							      }										
									}
						
					}else
						if (t==JIP.tBYTE){
							if (hByte==1){
								nfile=(img.getName())+"_H_byte.jip";
								path=".\\";							
								abreFich.setCurrentDirectory(new File(path));
								abreFich.setSelectedFile(new File(nfile));
								int returnVal = abreFich.showSaveDialog(this);
							  if (returnVal == abreFich.APPROVE_OPTION){
								  File fich = abreFich.getSelectedFile();
									nfile=fich.getAbsolutePath();
									if (!nfile.toLowerCase().endsWith(".jip"))
									 	nfile=nfile+".jip";
									jt.saveImageIntoFile(res,nfile);
									System.out.println("Guardando : "+nfile);
							  }								
							}
						}else
							if (t==JIP.tWORD){
								if (hWord==1){
									nfile=(img.getName())+"_H_word.jip";
									path=".\\";							
									abreFich.setCurrentDirectory(new File(path));
									abreFich.setSelectedFile(new File(nfile));
									int returnVal = abreFich.showSaveDialog(this);
								  if (returnVal == abreFich.APPROVE_OPTION){
									  File fich = abreFich.getSelectedFile();
										nfile=fich.getAbsolutePath();
										if (!nfile.toLowerCase().endsWith(".jip"))
										 	nfile=nfile+".jip";
										jt.saveImageIntoFile(res,nfile);
										System.out.println("Guardando : "+nfile);
								  }
								}
							}else
								if (t==JIP.tREAL){
									if (hReal==1){
										nfile=(img.getName())+"_H_real.jip";
										path=".\\";							
										abreFich.setCurrentDirectory(new File(path));
										abreFich.setSelectedFile(new File(nfile));
										int returnVal = abreFich.showSaveDialog(this);
									  if (returnVal == abreFich.APPROVE_OPTION){
										  File fich = abreFich.getSelectedFile();
											nfile=fich.getAbsolutePath();
											if (!nfile.toLowerCase().endsWith(".jip"))
											 	nfile=nfile+".jip";
											jt.saveImageIntoFile(res,nfile);
											System.out.println("Guardando : "+nfile);
									  }
									}
								}
								repaint();
				}
			
				if (t==JIP.tCOLOR){
	
					if (e.getActionCommand().equals("Luminosidad")){
						for(y=0;y<256;y++)
						ptos[y]=0;

	      		for(y=0;y<h;y++)
							for(x=0;x<w;x++) {
								ptos[(img.getPixelRed(x,y)+img.getPixelGreen(x,y)+img.getPixelBlue(x,y))/3]++;
							}
				
						max=1;

   			 		for(y=0;y<256;y++){
    					if (ptos[y]>max) max=ptos[y];
    				}

    				res=new JIPImage(b,256,256,t);
    				
    				for(x=0;x<256;x++) 
							for(y=0;y<256;y++){
							res.setPixelRed(x,y,128);		
							res.setPixelGreen(x,y,128);		
							res.setPixelBlue(x,y,128);		
						}
			
						for(x=0;x<256;x++) {
							int n=0;
							y=ptos[x];
							n=(255*y)/max; // ocurrencias en funcion del porcentaje
							for(y=255;n>=0;n--,y--){
								res.setPixelRed(x,y,x);		
								res.setPixelGreen(x,y,x);		
								res.setPixelBlue(x,y,x);
							}
						}

						img2=JIPToolkit.getAWTImage(res); 
						ScrollablePicture pic = new ScrollablePicture(new ImageIcon(img2),5);
						JScrollPane pictureScrollPane = new JScrollPane(pic);
						
						if (pane.getComponentCount()==nItems){
							pane.remove(0);
							setContentPane(pane);
						}
						
    		  	pane.add(pictureScrollPane,0);
    		  	setContentPane(pane);
    		  	pane.setLayout(new BoxLayout(pane,BoxLayout.Y_AXIS));
    		  	
    		  	pane.addMouseListener(mml);
			    	pane.addMouseMotionListener(mml);
						setContentPane(pane);
						his=1;
						hLum=1;
			      hRed=0;
			      hGreen=0;
			      hBlue=0;
			      hByte=0;
			      hWord=0;
			      hReal=0;
					}
					else       
       		if (e.getActionCommand().equals("Histograma RED")){
        		for(y=0;y<256;y++)
						ptos[y]=0;

        		for(y=0;y<h;y++)
						for(x=0;x<w;x++) {
							ptos[img.getPixelRed(x,y)]++;
						}

						max=1;	
						
   			 		for(y=0;y<256;y++){
    					if (ptos[y]>max) max=ptos[y];
	    			}
    				
    				res=new JIPImage(b,256,256,t);
    				
    				for(x=0;x<256;x++) 
							for(y=0;y<256;y++){
							res.setPixelRed(x,y,128);		
							res.setPixelGreen(x,y,128);		
							res.setPixelBlue(x,y,128);		
						}
						
						for(x=0;x<256;x++) {
							int n=0;
							y=ptos[x];
							n=(255*y)/max; // ocurrencias en funcion del porcentaje
							for(y=255;n>=0;n--,y--){
								res.setPixelRed(x,y,x);		
								res.setPixelGreen(x,y,0);		
								res.setPixelBlue(x,y,0);
							}
						}
								
						img2=JIPToolkit.getAWTImage(res); 
						ScrollablePicture pic = new ScrollablePicture(new ImageIcon(img2),5);
						JScrollPane pictureScrollPane = new JScrollPane(pic);

						if (pane.getComponentCount()==nItems){
							pane.remove(0);
							setContentPane(pane);
						}
						
   		  	 	pane.add(pictureScrollPane,0);
    		  	setContentPane(pane);
    		  	pane.setLayout(new BoxLayout(pane,BoxLayout.Y_AXIS));
    		  	
    		  	pane.addMouseListener(mml);
			    	pane.addMouseMotionListener(mml);
						setContentPane(pane);
						his=1;
						hLum=0;
					  hRed=1;
					  hGreen=0;
					  hBlue=0;
					  hByte=0;
					  hWord=0;
					  hReal=0;
        	}
        	else        
        	if (e.getActionCommand().equals("Histograma GREEN")){
        		for(y=0;y<256;y++)
						ptos[y]=0;

        		for(y=0;y<h;y++)
							for(x=0;x<w;x++) {
								ptos[img.getPixelGreen(x,y)]++;
							}
				
						max=1;	
    
   			 		for(y=0;y<256;y++){
    					if (ptos[y]>max) max=ptos[y];
    				}
    		
    				res=new JIPImage(b,256,256,t);
    				
    				for(x=0;x<256;x++) 
							for(y=0;y<256;y++){
							res.setPixelRed(x,y,128);		
							res.setPixelGreen(x,y,128);		
							res.setPixelBlue(x,y,128);		
						}
			
						for(x=0;x<256;x++) {
							int n=0;
							y=ptos[x];
							n=(255*y)/max; // ocurrencias en funcion del porcentaje
							for(y=255;n>=0;n--,y--){
								res.setPixelRed(x,y,0);		
								res.setPixelGreen(x,y,x);		
								res.setPixelBlue(x,y,0);
							}
						}

						img2=JIPToolkit.getAWTImage(res); 
						ScrollablePicture pic = new ScrollablePicture(new ImageIcon(img2),5);
  					JScrollPane pictureScrollPane = new JScrollPane(pic);
	  
						if (pane.getComponentCount()==nItems){
							pane.remove(0);
							setContentPane(pane);
						}

						pane.add(pictureScrollPane,0);
    		  	setContentPane(pane);
    		  	pane.setLayout(new BoxLayout(pane,BoxLayout.Y_AXIS));
    		  	
    		  	pane.addMouseListener(mml);
			    	pane.addMouseMotionListener(mml);
						setContentPane(pane);
						his=1;
						hLum=0;
			      hRed=0;
			      hGreen=1;
			      hBlue=0;
			      hByte=0;
			      hWord=0;
			      hReal=0;
        	}
        	else        
        	if (e.getActionCommand().equals("Histograma BLUE")){
        		for(y=0;y<256;y++)
						ptos[y]=0;

        		for(y=0;y<h;y++)
							for(x=0;x<w;x++) {
								ptos[img.getPixelBlue(x,y)]++;
							}

						max=1;				
						
   			 		for(y=0;y<256;y++){
    					if (ptos[y]>max) max=ptos[y];
    				}
    				
    				res=new JIPImage(b,256,256,t);
    				
    				for(x=0;x<256;x++) 
							for(y=0;y<256;y++){
							res.setPixelRed(x,y,128);		
							res.setPixelGreen(x,y,128);		
							res.setPixelBlue(x,y,128);		
						}
			
						for(x=0;x<256;x++) {
							int n=0;
							y=ptos[x];
							n=(255*y)/max; // ocurrencias en funcion del porcentaje
							for(y=255;n>=0;n--,y--){
								res.setPixelRed(x,y,0);		
								res.setPixelGreen(x,y,0);		
								res.setPixelBlue(x,y,x);
							}
						}

						img2=JIPToolkit.getAWTImage(res); 
						ScrollablePicture pic = new ScrollablePicture(new ImageIcon(img2),5);
						JScrollPane pictureScrollPane = new JScrollPane(pic);

						if (pane.getComponentCount()==nItems){
							pane.remove(0);
							setContentPane(pane);
						}

   		  	 	pane.add(pictureScrollPane,0);
    		  	setContentPane(pane);
    		  	pane.setLayout(new BoxLayout(pane,BoxLayout.Y_AXIS));
    		  	
    		  	pane.addMouseListener(mml);
			    	pane.addMouseMotionListener(mml);
						setContentPane(pane);
						his=1;
						hLum=0;
						hRed=0;
						hGreen=0;
						hBlue=1;
						hByte=0;
						hWord=0;
						hReal=0;
        	}
				}
				else{
		 			if (t==JIP.tBYTE){

						if (e.getActionCommand().equals("Histograma Completo")){
							for(y=0;y<256;y++)
								ptos[y]=0;

	      			for(y=0;y<h;y++)
								for(x=0;x<w;x++) {
									ptos[img.getPixel(0,x,y)]++;
								}
				
							max=1;
    
   				 		for(y=0;y<256;y++){
    						if (ptos[y]>max) max=ptos[y];
    					}

    					res=new JIPImage(1,256,256,JIP.tBYTE);
    				
    					for(x=0;x<256;x++) 
								for(y=0;y<256;y++){
									res.setPixel(0,x,y,128);									
							}
							
							for(x=0;x<256;x++) {
								int n=0;
								y=ptos[x];
								n=(255*y)/max; // ocurrencias en funcion del porcentaje
								for(y=255;n>=0;n--,y--)
									res.setPixel(x,y,x);		
							}
			
							for(x=0;x<256;x++) 
								for(y=ptos[x]-1;y>=0;y--){
									res.setPixel(0,x,255-(y*256)/max,x);		
								}					
									
							img2=JIPToolkit.getAWTImage(res); 
							ScrollablePicture pic = new ScrollablePicture(new ImageIcon(img2),5);
							JScrollPane pictureScrollPane = new JScrollPane(pic);
							
							if (pane.getComponentCount()==nItems){
								pane.remove(0);
								setContentPane(pane);
							}
							
	    		  	pane.add(pictureScrollPane,0);
	    		  	setContentPane(pane);
	    		  	pane.setLayout(new BoxLayout(pane,BoxLayout.Y_AXIS));
	    		  	
	    		  	pane.addMouseListener(mml);
				    	pane.addMouseMotionListener(mml);
							setContentPane(pane);
							his=1;
							hLum=0;
				      hRed=0;
				      hGreen=0;
				      hBlue=0;
				      hByte=1;
				      hWord=0;
				      hReal=0;
						}
			 		}
			 		else{
			 			if (t==JIP.tWORD){
		 				
							if (e.getActionCommand().equals("Histograma Completo")){
								for(y=0;y<65536;y++)
									ptosW[y]=0;
		      			
		      			for(y=0;y<h;y++)
									for(x=0;x<w;x++) {
										ptosW[img.getPixel(0,x,y)]++;
									}
									
								int acum=0;
								for(x=0,y=0;y<65536;y++){
									acum+=ptosW[y];
									if (y%256==0) {
										ptos[x]=acum;
										x++;
										acum=0;
									}
								}
					
								max=1;
	    
	   				 		for(y=0;y<256;y++){
	    						if (ptos[y]>max) max=ptos[y];
	    					}
	    					
	    					res=new JIPImage(1,256,256,JIP.tWORD);
	    				
	    					for(x=0;x<256;x++) 
									for(y=0;y<256;y++){
										res.setPixel(0,x,y,32768);									
									}

								for(x=0;x<256;x++) {
									int n=0;
									y=ptos[x];
									n=(255*y)/max; // ocurrencias en funcion del porcentaje
									for(y=255;n>=0;n--,y--){
										res.setPixel(0,x,y,((x+1)*256)-128);		
										
									}
								}
					
								img2=JIPToolkit.getAWTImage(res); 
								ScrollablePicture pic = new ScrollablePicture(new ImageIcon(img2),5);
								JScrollPane pictureScrollPane = new JScrollPane(pic);
								
								if (pane.getComponentCount()==nItems){
									pane.remove(0);
									setContentPane(pane);
								}
								
		    		  	pane.add(pictureScrollPane,0);
		    		  	setContentPane(pane);
		    		  	pane.setLayout(new BoxLayout(pane,BoxLayout.Y_AXIS));
		    		  	
		    		  	pane.addMouseListener(mml);
					    	pane.addMouseMotionListener(mml);
								setContentPane(pane);
								his=1;
								hLum=0;
				        hRed=0;
				        hGreen=0;
				        hBlue=0;
				        hByte=0;
				        hWord=1;
				        hReal=0;
							}
						}
						else{
				 			if (t==JIP.tREAL){
				 				
								if (e.getActionCommand().equals("Histograma Completo")){
									
									System.out.println("Calculando Histograma...");
			      			
			      			int nOcu  =-1;
			      			float iAct=-1;
			      			int iter=0,i=0;
			      			float iAux=-1;
			      							      			
			      			{
				      			RealDataHistogram [] nData= new RealDataHistogram [w*h];
				      			
				      			for (x=0;x<w*h;x++)
				      				nData[x]=new RealDataHistogram();
				      			
					      		for(x=0;x<w;x++){
					      			for(y=0;y<h;y++){
					      				int fin=0;
					      				iAct=img.getPixelFlo(0,x,y);
					      				for (i=0; i<iter && fin==0; i++){
					      					iAux=nData[i].getIValue();
					      					if (iAux==iAct){	// si ya existe la acualizo.
					      						nData[i].setOValue(nData[i].getOValue()+1);
					      						fin=1;
					      					}//if 				      					
					      				}//for iter
					      				
					      				if (fin==0){ // significa que no existe la intensidad -> la aado
					      					nData[iter].setIValue(iAct);
					      					nData[iter].setOValue(1);
					      					iter++;
					      				}// if
					      			}// for y
					      		}// for x
					     
					     			nIntR=iter; 		

					      		RealDataHistogram [] auxR=new RealDataHistogram [nIntR];
					      		for (x=0;x<nIntR;x++){
				      				auxR[x]=new RealDataHistogram();
				      				auxR[x].setIValue(nData[x].getIValue());
				      				auxR[x].setOValue(nData[x].getOValue());
					      		}
					      		
					      		// ahora vamos a ordenar todo
									
										float inf=10000; // infinito

										ptosR=new RealDataHistogram [nIntR];
										
						      	for (x=0;x<nIntR;x++){
					      			ptosR[x]=new RealDataHistogram();
					      			ptosR[x].setIValue(inf);
					      			ptosR[x].setOValue(-1);
						      	}
						      	
						      	for (x=0;x<nIntR;x++){
						      		float iPtos=ptosR[x].getIValue();
					      			int   oPtos=ptosR[x].getOValue();
					      			float inAux;
					      			int   oAux;
				      				int uPos=0;
						      		
						      		for (y=0;y<nIntR;y++){
						      			inAux=auxR[y].getIValue();
						      			oAux=auxR[y].getOValue();
						      								      			
						      			if (inAux<iPtos){
						      				iPtos=inAux;
					      					oPtos=oAux;						      				
					      					uPos=y;
						      			}
						      		}						      		
						      		ptosR[x].setIValue(auxR[uPos].getIValue());
						      		ptosR[x].setOValue(auxR[uPos].getOValue());
						      		auxR[uPos].setIValue(inf);
						      		auxR[uPos].setOValue(-1);
						      	}				      	
			      			}
			      			
			      			// vamos a obtener el maximo
									
									max=1;
	    
		   				 		for(y=0;y<nIntR;y++){
		    						if ((ptosR[y].getOValue())>max) max=ptosR[y].getOValue();
		    					}
		    					
		    					// declaramos la nueva imagen resultado
								
									res=new JIPImage(1,nIntR,256,JIP.tREAL);
									
									for(x=0;x<nIntR;x++) 
										for(y=0;y<256;y++){
											res.setPixel(0,x,y,(float)(0.5));									
										}
	
									for(x=0;x<nIntR;x++) {
										int n=0;
										y=ptosR[x].getOValue();
										n=(255*y)/max; // ocurrencias en funcion del porcentaje
										for(y=255;n>=0;n--,y--)
											res.setPixel(0,x,y,(ptosR[x].getIValue()));		
									}
									
									img2=JIPToolkit.getAWTImage(res); 
									ScrollablePicture pic = new ScrollablePicture(new ImageIcon(img2),5);
									JScrollPane pictureScrollPane = new JScrollPane(pic);
			
									if (pane.getComponentCount()==nItems){
										pane.remove(0);
										setContentPane(pane);
									}
									
			   		  	 	pane.add(pictureScrollPane,0);
			    		  	setContentPane(pane);
			    		  	pane.setLayout(new BoxLayout(pane,BoxLayout.Y_AXIS));
			    		  	
			    		  	pane.addMouseListener(mml);
						    	pane.addMouseMotionListener(mml);
									setContentPane(pane);
									his=1;
									hLum=0;
								  hRed=0;
								  hGreen=0;
								  hBlue=0;
								  hByte=0;
								  hWord=0;
								  hReal=1;
								}
							}
						}
			 		}
				}
		 	}
		}
    
    /**<P><FONT COLOR="red">
		*<B>Descripcin:</B><BR>
		*<FONT COLOR="blue">
    * Clase para escuchar y capturar eventos de raton.
    * Esta clase se compone de dos metodos:
    * public void mouseMoved(MouseEvent e) y public void mouseDragged(MouseEvent e).
    * Las dos reciben com parametro de entrada un evento. Dicho evento es gestionado 
    * segun la naturaleza de la imagen a tratar. El objeto de esta clase es traducir 
    * y mostrar la informacion recogida por el usuario a traves del raton en informacion
    * acerca del histograma.
    */
     
  public class MyMouseListener extends MouseAdapter implements MouseMotionListener {
    
		public void mouseMoved(MouseEvent e) {
			int x=e.getX()-6;
     	int y=e.getY()-6;
     	int t=img.getType();
			int w=img.getWidth();
			int h=img.getHeight();
     	
     	if (t==JIP.tBYTE || t==JIP.tCOLOR){

				if (x<256 && x>=0 && y<256 && y>=0){
	     		in.setText(" "+x);
	    		oc.setText(" "+(float)((((ptos[x]*256.0)/max)*100.0)/256.0));
				}
	   		else {
		 			in.setText ("?");
	   			oc.setText("?");
	   		}
     	}
			else
			if (t==JIP.tWORD){
				if (x<256 && x>=0 && y<256 && y>=0){
					in.setText (" "+x*256+"-->"+((x*256)+256));
	    		oc.setText(" "+(float)((((ptos[x]*256.0)/max)*100.0)/256.0));
				}
	   		else {
		 			in.setText ("?");
	   			oc.setText("?");
	   		}
			}
			else
			if (t==JIP.tREAL){
				if (x<nIntR && x>=0 && y<256 && y>=0){
	     		in.setText(" "+ptosR[x].getIValue());
	    		oc.setText(" "+(float)((((ptosR[x].getOValue()*256.0)/max)*100.0)/256.0));
				}
	   		else {
		 			in.setText ("?");
	   			oc.setText("?");
	   		}
			}
			cota_maxima.setText("Mximo: "+max);
   	}
    
    public void mouseDragged(MouseEvent e) {
    	int x=e.getX()-6;
    	int y=e.getY()-6;
    	int t=img.getType();
			int w=img.getWidth();
			int h=img.getHeight();
         
     	if (t==JIP.tBYTE || t==JIP.tCOLOR){
			  if (x<256 && x>=0 && y<256 && y>=0){
	    		in.setText (" "+x);
	    		oc.setText(" "+(float)((((ptos[x]*256.0)/max)*100.0)/256.0));
			  }
	   		else {
		 			in.setText ("?");
	   			oc.setText("?");
	   		}
      }
      else
     	if (t==JIP.tWORD){
      	 if (x<256 && x>=0 && y<256 && y>=0){
      	 	in.setText (" "+x*256+"-->"+((x*256)+256));
	    		oc.setText(" "+(float)((((ptos[x]*256.0)/max)*100.0)/256.0));
			  }
	   		else {
		 			in.setText ("?");
	   			oc.setText("?");
	   		}     		
      }
      else
      if (t==JIP.tREAL){
      	if (x<nIntR && x>=0 && y<256 && y>=0){
	     		in.setText(" "+ptosR[x].getIValue());
	    		oc.setText(" "+(float)((((ptosR[x].getOValue()*256.0)/max)*100.0)/256.0));
				}
	   		else {
		 			in.setText ("?");
	   			oc.setText("?");
	   		}
      }
      cota_maxima.setText("Mximo: "+max);
    }
  }
}
 
 