package jip;
import java.util.Vector;

 /**<P><FONT COLOR="red">
 *<B>Descripcin:</B><BR>
 *<FONT COLOR="blue">
 * Clase que define constantes y mtodos bsicos de conversin para el desarrollo
 * de aplicaciones que utilicen la librera JIP.
 */
public final class JIP {

		/** Constructor Vacio*/

    JIP() {}

    //**************************************************
    // Constantes para identificar los tipos de imagenes
    //**************************************************
		
	/**
	 * Cte que identifica un tipo de imagen o bitmap binario.
	 * Los valores de los pixels son enteros en el rango [0..1]
	 */
	public static final int tBIT=0;

	/**
	 * Cte que identifica un tipo de imagen o bitmap en 256 grises.
	 * Los valores de los pixels son enteros en el rango [0..255]
	 */
	public static final int tBYTE=1;

	/**
	 * Cte que identifica un tipo de imagen o bitmap en 65536 grises.
	 * Los valores de los pixels son enteros en el rango [0..65535]
	 */
	public static final int tWORD=2;

	/**
	 * Cte que identifica un tipo de imagen o bitmap en escala de grises.
	 * Los valores de los pixels son reales en el rango [0..1]
	 */
	public static final int tREAL=3;

	/**
	 * Cte que identifica un tipo de imagen en color (3 bandas RGB).
	 * Los valores de los pixels son un entero en el rango [0..255] para cada banda.
	 */
  public static final int tCOLOR=4;
	/**
	 * Cte que identifica un tipo de imagen Segmento.
	 */

  public static final int tSEGMENT=5;
  /**
	 * Cte que identifica un tipo de imagen Punto.
	 */

  public static final int tPOINT=6;
  /**
	 * Cte que identifica un tipo de imagen Polgono.
	 */
  public static final int tPOLY=7;
  /**
	 * Cte que identifica un tipo de imagen Edges
	 */
  
  public static final int tEDGES=8;
  
    //*******************************************
    // Constantes para identificar las bandas RGB
    //*******************************************
	
	/** Cte que identifica la banda RED de una imagen en color. */
    public static final int bRED=0;
	
	/** Cte que identifica la banda GREEN de una imagen en color. */
    public static final int bGREEN=1;
	
	/** Cte que identifica la banda BLUE de una imagen en color. */
    public static final int bBLUE=2;

    //****************************************************
    // Constantes para identificar los tipos de parametros
    //****************************************************
	
	/** Cte que identifica un tipo de parmetro de funcin booleano. */
    public static final int pBOOL=0;

	/** Cte que identifica un tipo de parmetro de funcin entero. */
    public static final int pINT=1;

	/** Cte que identifica un tipo de parmetro de funcin real. */
    public static final int pREAL=2;

	/** Cte que identifica un tipo de parmetro de funcin cadena. */
    public static final int pSTRING=3;

	/** Cte que identifica un tipo de parmetro de funcin imagen. */
    public static final int pIMAGE=4;

    //******************************************************************
    //Metodos para conversiones Represent. USUARIO --> Represent. VALIDA
    //******************************************************************

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
		 *<FONT COLOR="blue">
     * Convierte un valor de pixel en un valor dentro del rango de tipo BIT.
     * @param pix Valor de la intensidad del pixel.
     * @return Intensidad vlida tipo BIT: entero [0..1]
     */
    public static int getValidBit(int pix) {
    	return (pix>=1)?1:0;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
		 *<FONT COLOR="blue">
     * Convierte valores de pixels en valores dentro del rango de tipo BIT.
     * @param pix Valores de intensidades de los pixels.
     * @return Intensidades vlidas tipo BIT: entero [0..1]
     */
    public static int[] getAllValidBit(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
	    int[] res=new int[npixels];
        for(int i=0;i<npixels;i++)
	    	res[i]=(pix[i]>=1)?1:0;
	    return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un valor de pixel en un valor dentro del rango de tipo BYTE.
     * @param pix Valor de la intensidad del pixel.
     * @return Intensidad vlida tipo BYTE: entero [0..255]
     */
    public static int getValidByte(int pix) {
		int res;
        if(pix>=255) res=255;
        else if(pix<=0) res=0;
        else res=pix;
    	return pix;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte valores de pixels en valores dentro del rango de tipo BYTE.
     * @param pix Valores de intensidades de los pixels.
     * @return Intensidades vlidas tipo BYTE: entero [0..255]
     */
    public static int[] getAllValidByte(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
	    int[] res=new int[npixels];
        for(int i=0;i<npixels;i++) {
	        if(pix[i]>=255) res[i]=255;
	        else if(pix[i]<=0) res[i]=0;
	        else res[i]=pix[i];
	    }
	    return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un valor de pixel en un valor dentro del rango de tipo WORD.
     * @param pix Valor de la intensidad del pixel.
     * @return Intensidad vlida tipo WORD: entero [0..65535]
     */
    public static int getValidWord(int pix) {
		int res;
        if(pix>=65535) res=65535;
        else if(pix<=0) res=0;
        else res=pix;
    	return pix;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte valores de pixels en valores dentro del rango de tipo WORD.
     * @param pix Valores de intensidades de los pixels.
     * @return Intensidades vlidas tipo WORD: entero [0..65535]
     */
    public static int[] getAllValidWord(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
	    int[] res=new int[npixels];
        for(int i=0;i<npixels;i++) {
	        if(pix[i]>=65535) res[i]=65535;
	        else if(pix[i]<=0) res[i]=0;
	        else res[i]=pix[i];
	    }
	    return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un valor de pixel en un valor dentro del rango de tipo REAL.
     * @param pix Valor de la intensidad del pixel.
     * @return Intensidad vlida tipo REAL: float [0..1]
     */
    public static float getValidReal(float pix) {
		float res;
        if(pix>=1) res=1;
        else if(pix<=0) res=0;
        else res=pix;
    	return pix;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte valores de pixels en valores dentro del rango de tipo REAL.
     * @param pix Valores de intensidades de los pixels.
     * @return Intensidades vlidas tipo REAL: float [0..1]
     */
    public static float[] getAllValidReal(float[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
	    float[] res=new float[npixels];
        for(int i=0;i<npixels;i++) {
	         res[i]=pix[i];
	      }
	    return res;
    }

    //********************************************
    //Metodos para conversiones relativas al color
    //********************************************

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Extrae la intensidad RED de un pixel con valor RGB.
     * @param rgb Valor RGB del pixel.
     * @return Intensidad RED del pixel.
     */
    public static int getRedFromRGB(int rgb) { return (rgb>>>16)&0xFF; }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Extrae la intensidad GREEN de un pixel con valor RGB.
     * @param rgb Valor RGB del pixel.
     * @return Intensidad GREEN del pixel.
     */
    public static int getGreenFromRGB(int rgb) { return (rgb>>>8)&0xFF; }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Extrae la intensidad BLUE de un pixel con valor RGB.
     * @param rgb Valor RGB del pixel.
     * @return Intensidad BLUE del pixel.
     */
    public static int getBlueFromRGB(int rgb) { return rgb&0xFF; }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Extrae las intensidades RED de un vector de pixels con valores RGB.
     * @param rgb Vector de pixels con valores RGB.
     * @return Intensidades RED de los pixels.
     */
    public static int[] getAllRedFromRGB(int[] rgb) {
    	if(rgb==null) return null;
    	if(rgb.length==0) return null;
        int npixels=rgb.length;
        int[] res=new int[npixels];
        for(int i=0;i<npixels;i++)
            res[i]=(rgb[i]>>>16)&0xFF;
        return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Extrae las intensidades GREEN de un vector de pixels con valores RGB.
     * @param rgb Vector de pixels con valores RGB.
     * @return Intensidades GREEN de los pixels.
     */
    public static int[] getAllGreenFromRGB(int[] rgb) {
    	if(rgb==null) return null;
    	if(rgb.length==0) return null;
        int npixels=rgb.length;
        int[] res=new int[npixels];
        for(int i=0;i<npixels;i++)
            res[i]=(rgb[i]>>>8)&0xFF;
        return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Extrae las intensidades BLUE de un vector de pixels con valores RGB.
     * @param rgb Vector de pixels con valores RGB.
     * @return Intensidades BLUE de los pixels.
     */
    public static int[] getAllBlueFromRGB(int[] rgb) {
    	if(rgb==null) return null;
    	if(rgb.length==0) return null;
        int npixels=rgb.length;
        int[] res=new int[npixels];
        for(int i=0;i<npixels;i++)
            res[i]=(rgb[i])&0xFF;
        return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Obtiene el valor RGB a partir de las intensidades RED, GREEN y BLUE.
     * @param red Intensidad RED del pixel.
     * @param green Intensidad GREEN del pixel.
     * @param blue Intensidad BLUE del pixel.
     * @return Valor RGB del pixel.
     */
    public static int getRGB(int red,int green,int blue) {
        red=red&0xFF;
        green=green&0xFF;
        blue=blue&0xFF;
        return (255<<24)|(red<<16)|(green<<8)|blue;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Obtiene los valores RGB a partir de los vectores de intensidades
     * RED, GREEN y BLUE.
     * @param red Vector de intensidades RED de los pixels.
     * @param green Vector de intensidades GREEN de los pixels.
     * @param blue Vector de intensidades BLUE de los pixels.
     * @return Vector de valores RGB de los pixels.
     */
    public static int[] getAllRGB(int[] red,int[] green,int[] blue) {
    	if(red==null) return null;
    	if(red.length==0) return null;
        if(red.length==green.length&&red.length==blue.length) {
            int npixels=red.length;
            int[] res=new int[npixels];
            for(int i=0;i<npixels;i++)
                res[i]=(255<<24)|
                       ((red[i]&0xFF)<<16)|
                       ((green[i]&0xFF)<<8)|
                       (blue[i]&0xFF);
            return res;
        }
        else return null;
    }

    //****************************************************
    //Metodos para conversiones Represent. USUARIO --> BIT
    //****************************************************

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo BYTE a un pixel tipo BIT.
     * @param pix Valor de intensidad del pixel tipo BYTE.
     * @return Intensidad tipo BIT: entero [0..1]
     */
    public static int getBitFromByte(int pix) {
    	int val=getValidByte(pix);
    	return (val>>>7)&0x01;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo BYTE a pixels tipo BIT.
     * @param pix Valor de intensidades de los pixels tipo BYTE.
     * @return Intensidades tipo BIT: entero [0..1]
     */
    public static int[] getAllBitFromByte(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
    	int[] val=getAllValidByte(pix);
        for(int i=0;i<npixels;i++)
            res[i]=(val[i]>>>7)&0x01;
        return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo WORD a un pixel tipo BIT.
     * @param pix Valor de intensidad del pixel tipo WORD.
     * @return Intensidad tipo BIT: entero [0..1]
     */
    public static int getBitFromWord(int pix) {
    	int val=getValidWord(pix);
    	return (val>>>15)&0x01;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo WORD a pixels tipo BIT.
     * @param pix Valor de intensidades de los pixels tipo WORD.
     * @return Intensidades tipo BIT: entero [0..1]
     */
    public static int[] getAllBitFromWord(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
    	int[] val=getAllValidWord(pix);
        for(int i=0;i<npixels;i++)
            res[i]=(val[i]>>>15)&0x01;
        return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo REAL a un pixel tipo BIT.
     * @param pix Valor de intensidad del pixel tipo REAL.
     * @return Intensidad tipo BIT: entero [0..1]
     */
    public static int getBitFromReal(float pix) {
    	if(getValidReal(pix)>=0.5) return 1;
    	else return 0;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo REAL a pixels tipo BIT.
     * @param pix Valor de intensidades de los pixels tipo REAL.
     * @return Intensidades tipo BIT: entero [0..1]
     */

    public static int[] getAllBitFromReal(float[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
    	float[] val=getAllValidReal(pix);
        for(int i=0;i<npixels;i++) {
        	if(val[i]>=0.5) res[i]=1;
        	else res[i]=0;
        }
        return res;
    }

    //*****************************************************
    //Metodos para conversiones Represent. USUARIO --> BYTE
    //*****************************************************

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo BIT a un pixel tipo BYTE.
     * @param pix Valor de intensidad del pixel tipo BIT.
     * @return Intensidad tipo BYTE: entero [0..255]
     */
    public static int getByteFromBit(int pix) {
    	return (getValidBit(pix)==1)?255:0;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo BIT a pixels tipo BYTE.
     * @param pix Valor de intensidades de los pixels tipo BIT.
     * @return Intensidades tipo BYTE: entero [0..255]
     */
    public static int[] getAllByteFromBit(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
    	int[] val=getAllValidBit(pix);
        for(int i=0;i<npixels;i++)
            res[i]=(val[i]==1)?255:0;
        return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo WORD a un pixel tipo BYTE.
     * @param pix Valor de intensidad del pixel tipo WORD.
     * @return Intensidad tipo BYTE: entero [0..255]
     */
    public static int getByteFromWord(int pix) {
    	return getValidWord(pix)>>>8;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo WORD a pixels tipo BYTE.
     * @param pix Valor de intensidades de los pixels tipo WORD.
     * @return Intensidades tipo BYTE: entero [0..255]
     */
    public static int[] getAllByteFromWord(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
    	int[] val=getAllValidWord(pix);
        for(int i=0;i<npixels;i++)
            res[i]=val[i]>>>8;
        return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo REAL a un pixel tipo BYTE.
     * @param pix Valor de intensidad del pixel tipo REAL.
     * @return Intensidad tipo BYTE: entero [0..255]
     */
    public static int getByteFromReal(float pix) {
    	return (int)Math.round((double)getValidReal(pix)*255.0);
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo REAL a pixels tipo BYTE.
     * @param pix Valor de intensidades de los pixels tipo REAL.
     * @return Intensidades tipo BYTE: entero [0..255]
     */
    public static int[] getAllByteFromReal(float[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
    	float[] val=getAllValidReal(pix);
        for(int i=0;i<npixels;i++)
	    	res[i]=(int)Math.round((double)val[i]*255.0);
        return res;
    }

    //*****************************************************
    //Metodos para conversiones Represent. USUARIO --> WORD
    //*****************************************************

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo BIT a un pixel tipo WORD.
     * @param pix Valor de intensidad del pixel tipo BIT.
     * @return Intensidad tipo WORD: entero [0..65535]
     */
    public static int getWordFromBit(int pix) {
    	return (getValidBit(pix)==1)?65535:0;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo BIT a pixels tipo WORD.
     * @param pix Valor de intensidades de los pixels tipo BIT.
     * @return Intensidades tipo WORD: entero [0..65535]
     */
    public static int[] getAllWordFromBit(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
    	int[] val=getAllValidBit(pix);
        for(int i=0;i<npixels;i++)
            res[i]=(val[i]==1)?65535:0;
        return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo BYTE a un pixel tipo WORD.
     * @param pix Valor de intensidad del pixel tipo BYTE.
     * @return Intensidad tipo WORD: entero [0..65535]
     */
    public static int getWordFromByte(int pix) {
    	return getValidByte(pix)<<8;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo BYTE a pixels tipo WORD.
     * @param pix Valor de intensidades de los pixels tipo BYTE.
     * @return Intensidades tipo WORD: entero [0..65535]
     */
    public static int[] getAllWordFromByte(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
    	int[] val=getAllValidByte(pix);
        for(int i=0;i<npixels;i++)
            res[i]=val[i]<<8;
        return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo REAL a un pixel tipo WORD.
     * @param pix Valor de intensidad del pixel tipo REAL.
     * @return Intensidad tipo WORD: entero [0..65535]
     */
    public static int getWordFromReal(float pix) {
    	return (int)Math.round((double)getValidReal(pix)*65535.0);
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo REAL a pixels tipo WORD.
     * @param pix Valor de intensidades de los pixels tipo REAL.
     * @return Intensidades tipo WORD: entero [0..65535]
     */
    public static int[] getAllWordFromReal(float[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
    	float[] val=getAllValidReal(pix);
        for(int i=0;i<npixels;i++)
	    	res[i]=(int)Math.round((double)val[i]*65535.0);
        return res;
    }

    //*****************************************************
    //Metodos para conversiones Represent. USUARIO --> REAL
    //*****************************************************

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo BIT a un pixel tipo REAL.
     * @param pix Valor de intensidad del pixel tipo BIT.
     * @return Intensidad tipo REAL: float [0..1]
     */
    public static float getRealFromBit(int pix) {
    	return (getValidBit(pix)==1)?1.0f:0.0f;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo BIT a pixels tipo REAL.
     * @param pix Valor de intensidades de los pixels tipo BIT.
     * @return Intensidades tipo REAL: float [0..1]
     */
    public static float[] getAllRealFromBit(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        float[] res=new float[npixels];
    	int[] val=getAllValidBit(pix);
        for(int i=0;i<npixels;i++)
            res[i]=(val[i]==1)?1.0f:0.0f;
        return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo BYTE a un pixel tipo REAL.
     * @param pix Valor de intensidad del pixel tipo BYTE.
     * @return Intensidad tipo REAL: float [0..1]
     */
    public static float getRealFromByte(int pix) {
    	return (float)((double)getValidByte(pix)/255.0);
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo BYTE a pixels tipo REAL.
     * @param pix Valor de intensidades de los pixels tipo BYTE.
     * @return Intensidades tipo REAL: float [0..1]
     */
    public static float[] getAllRealFromByte(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        float[] res=new float[npixels];
    	int[] val=getAllValidByte(pix);
        for(int i=0;i<npixels;i++)
	    	res[i]=(float)((double)val[i]/255.0);
        return res;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo WORD a un pixel tipo REAL.
     * @param pix Valor de intensidad del pixel tipo WORD.
     * @return Intensidad tipo REAL: float [0..1]
     */
    public static float getRealFromWord(int pix) {
    	return (float)((double)getValidWord(pix)/65535.0);
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo WORD a pixels tipo REAL.
     * @param pix Valor de intensidades de los pixels tipo WORD.
     * @return Intensidades tipo REAL: float [0..1]
     */
    public static float[] getAllRealFromWord(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        float[] res=new float[npixels];
    	int[] val=getAllValidWord(pix);
        for(int i=0;i<npixels;i++)
	    	res[i]=(float)((double)val[i]/65535.0);
        return res;
    }

    //*******************************************************************
    //Metodos para conversiones Represent. USUARIO --> Represent. INTERNA
    //*******************************************************************

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo Entero a pixel tipo BIT.
     * @param pix Valor de intensidad del pixel tipo Entero.
     * @return Intensidad tipo BIT
     */
    
    static boolean getBitFromInt(int pix) {
        return (getValidBit(pix)==1)?true:false;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo Entero a pixels tipo BIT
     * @param pix Valor de intensidades de los pixels tipo Entero
     * @return Intensidades tipo BIT
     */

    static boolean[] getAllBitFromInt(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        boolean[] res=new boolean[npixels];
        int[] val=getAllValidBit(pix);
        for(int i=0;i<npixels;i++)
            res[i]=(val[i]==1)?true:false;
        return res;
    }

		 /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo Entero a pixel tipo BYTE.
     * @param pix Valor de intensidad del pixel tipo Entero.
     * @return Intensidad tipo BYTE
     */

    static byte getByteFromInt(int pix) {
        return (byte)getValidByte(pix);
    }
    
    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo Entero a pixels tipo BYTE.
     * @param pix Valor de intensidades de los pixels tipo Entero.
     * @return Intensidades tipo BYTE: entero [0..255]
     */

    static byte[] getAllByteFromInt(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        byte[] res=new byte[npixels];
        int[] val=getAllValidByte(pix);
        for(int i=0;i<npixels;i++)
        	res[i]=(byte)val[i];
        return res;
    }

		 /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo Entero a pixel tipo WORD.
     * @param pix Valor de intensidad del pixel tipo Entero.
     * @return Intensidad tipo WORD: entero [0..65535]
     */

    static short getWordFromInt(int pix) {
        return (short)getValidWord(pix);
    }

		/**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo Entero a pixels tipo WORD.
     * @param pix Valor de intensidades de los pixels tipo Entero.
     * @return Intensidades tipo WORD: entero [0..65535]
     */

    static short[] getAllWordFromInt(int[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        short[] res=new short[npixels];
        int[] val=getAllValidWord(pix);
        for(int i=0;i<npixels;i++)
            res[i]=(short)val[i];
        return res;
    }
	
		/**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel de tipo Flotante a un pixel de tipo REAL.
     * @param pix Valor de intensidad del pixel de tipo Flotante.
     * @return Intensidad de tipo REAL: [0..1]
     */


    static float getRealFromFloat(float pix) {
        return getValidReal(pix);
    }

		 /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo Flotante a pixels tipo REAL.
     * @param pix Valor de intensidades de los pixels tipo Flotante.
     * @return Intensidades tipo REAL: [0..1]
     */

    static float[] getAllRealFromFloat(float[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        return getAllValidReal(pix);
    }

    //*******************************************************************
    //Metodos para conversiones Represent. INTERNA --> Represent. USUARIO
    //*******************************************************************

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo BIT  a pixel tipo Entero.
     * @param pix Valor de intensidad del pixel tipo BIT.
     * @return Intensidad tipo Entero
     */

    static int getIntFromBit(boolean pix) {
        return (pix)?1:0;
    }
		
		 /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo BIT a pixels tipo Entero.
     * @param pix Valor de intensidades de los pixels tipo BIT.
     * @return Intensidades tipo Entero
     */

    static int[] getAllIntFromBit(boolean[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
        for(int i=0;i<npixels;i++)
            res[i]=(pix[i])?1:0;
        return res;
    }

		 /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo BYTE a pixel tipo Entero.
     * @param pix Valor de intensidad del pixel tipo BYTE.
     * @return Intensidad tipo Entero
     */
    static int getIntFromByte(byte pix) {
        return (pix<0)?(int)pix+256:(int)pix;
    }


		/**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo BYTE a pixels tipo Entero.
     * @param pix Valor de intensidades de los pixels tipo BYTE.
     * @return Intensidades tipo Entero
     */
    static int[] getAllIntFromByte(byte[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
        for(int i=0;i<npixels;i++)
            res[i]=(pix[i]<0)?(int)pix[i]+256:(int)pix[i];
        return res;
    }
		
		 /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un pixel tipo WORD a pixel tipo Entero.
     * @param pix Valor de intensidad del pixel tipo WORD.
     * @return Intensidad tipo Entero
     */

    static int getIntFromWord(short pix) {
        return (pix<0)?(int)pix+65536:(int)pix;
    }

		/**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo WORD a pixels tipo Entero.
     * @param pix Valor de intensidades de los pixels tipo WORD.
     * @return Intensidades tipo Entero
     */

    static int[] getAllIntFromWord(short[] pix) {
    	if(pix==null) return null;
    	if(pix.length==0) return null;
        int npixels=pix.length;
        int[] res=new int[npixels];
        for(int i=0;i<npixels;i++)
            res[i]=(pix[i]<0)?(int)pix[i]+65536:(int)pix[i];
        return res;
    }

    //************************************
    //Conversiones INTERNA --> BRUTA (RAW)
    //************************************
    
     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo BIT a pixels en formato RAW (Array de Bytes)
     * @param pix Valor de intensidades de los pixels tipo BIT.
     * @return Intensidades en formato RAW
     */

    static byte[] getRawFromBit(boolean[] bmp) {
        int bmpRAWlg=(int)Math.ceil((double)bmp.length/8.0);
        byte[] bmpRAW=new byte[bmpRAWlg];
		
        for(int j=0,i=0;j<bmpRAWlg;j++) {
            byte mask=0; 
            for(int bit=7;bit>=0;bit--) {
                if(i<bmp.length && bmp[i]) mask|=1<<bit;
                i++;
            }
            bmpRAW[j]=mask;
        }
        return bmpRAW;		
    }
    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo BYTE a pixels en formato RAW (Array de Bytes)
     * @param pix Valor de intensidades de los pixels tipo BYTE.
     * @return Intensidades en formato RAW
     */
    static byte[] getRawFromByte(byte[] bmp) {
        int bmpRAWlg=bmp.length;
        byte[] bmpRAW=new byte[bmpRAWlg];
        for(int i=0;i<bmp.length;i++) bmpRAW[i]=bmp[i];
        return bmpRAW;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un Vector tipo SEGMENT a formato RAW (Array de Bytes)
     * @param pix Vector de tipo SEGMENT
     * @return Vector en formato RAW
     */
    static byte[] getRawFromSegment(Vector bmp){
        int bmpRAWlg=bmp.size()*2;
        byte[] bmpRAW=new byte[bmpRAWlg];
        for(int i=0,j=0;i<bmp.size();i++,j+=2) {
          Integer dd=new Integer(0);
          dd=(Integer)bmp.elementAt(i);
          bmpRAW[j]=(byte)(dd.intValue() >>> 8);
          bmpRAW[j+1]=(byte)(dd.intValue() & 0x00ff);    
        }
        return bmpRAW;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un Vector tipo POINTT a formato RAW (Array de Bytes)
     * @param pix Vector de tipo POINT
     * @return Vector en formato RAW
     */
    static byte[] getRawFromPoint(Vector bmp){
      int bmpRAWlg=bmp.size()*2;
        byte[] bmpRAW=new byte[bmpRAWlg];
        for(int i=0,j=0;i<bmp.size();i++,j+=2) {
          Integer dd=new Integer(0);
          dd=(Integer)bmp.elementAt(i);
          bmpRAW[j]=(byte)(dd.intValue() >>> 8);
          bmpRAW[j+1]=(byte)(dd.intValue() & 0x00ff);    
        }
        return bmpRAW;
    }
     
     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un Vector tipo POLY a formato RAW (Array de Bytes)
     * @param pix Vector de tipo POLY
     * @return Vector en formato RAW
     */

    static byte[] getRawFromPoly(Vector bmp){
        int sizet=0;
        for (int npol=0;npol<bmp.size();npol++){
          sizet+=((Vector)bmp.elementAt(npol)).size();        
        }
        sizet*=2;
        int bmpRAWlg=sizet;
        byte[] bmpRAW=new byte[bmpRAWlg];
        Vector vaux;
        for (int kj=0,j=0;kj<bmp.size();kj++){
          vaux=new Vector((Vector)bmp.elementAt(kj));
          for(int i=0;i<vaux.size();i++,j+=2) {
            Integer dd=new Integer(0);
            dd=(Integer)vaux.elementAt(i);
            bmpRAW[j]=(byte)(dd.intValue() >>> 8);
            bmpRAW[j+1]=(byte)(dd.intValue() & 0x00ff);    
          }
        }
        return bmpRAW;
    }           
    
    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un Vector tipo EDGES a formato RAW (Array de Bytes)
     * @param pix Vector de tipo EDGES
     * @return Vector en formato RAW
     */

    static byte[] getRawFromEdges(Vector bmp){
      int sizet=0;
        for (int npol=0;npol<bmp.size();npol++){
          sizet+=((Vector)bmp.elementAt(npol)).size();        
        }
        sizet*=2;
        int bmpRAWlg=sizet;
        byte[] bmpRAW=new byte[bmpRAWlg];
        Vector vaux;
        for (int kj=0,j=0;kj<bmp.size();kj++){
          vaux=new Vector((Vector)bmp.elementAt(kj));
          for(int i=0;i<vaux.size();i++,j+=2) {
            Integer dd=new Integer(0);
            dd=(Integer)vaux.elementAt(i);
            bmpRAW[j]=(byte)(dd.intValue() >>> 8);
            bmpRAW[j+1]=(byte)(dd.intValue() & 0x00ff);    
          }
        }
        return bmpRAW;

    }           
    
    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo WORD a pixels en formato RAW (Array de Bytes)
     * @param pix Valor de intensidades de los pixels tipo WORD.
     * @return Intensidades en formato RAW
     */
    static byte[] getRawFromWord(short[] bmp) {
        int bmpRAWlg=bmp.length*2;
        byte[] bmpRAW=new byte[bmpRAWlg];
        for(int i=0,j=0;i<bmp.length;i++,j+=2) {
          bmpRAW[j]=(byte)(bmp[i] >>> 8);
          bmpRAW[j+1]=(byte)(bmp[i] & 0x00ff);    
        }
        return bmpRAW;
    }
    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels tipo REAL a pixels en formato RAW (Array de Bytes)
     * @param pix Valor de intensidades de los pixels tipo REAL.
     * @return Intensidades en formato RAW
     */
    static byte[] getRawFromReal(float[] bmp) {
        int bmpRAWlg=bmp.length*4;
        byte[] bmpRAW=new byte[bmpRAWlg];
        for(int i=0,j=0;i<bmp.length;i++,j+=4) {
            int val=Float.floatToIntBits(bmp[i]);
            bmpRAW[j]=(byte)(val >>> 24);
            bmpRAW[j+1]=(byte)((val >>> 16) & 0xff);
            bmpRAW[j+2]=(byte)((val >>> 8) & 0xff);
            bmpRAW[j+3]=(byte)(val & 0xff);
        }
        return bmpRAW;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte un vector de WORD a formato RAW (Array de Bytes)
     * @param pix Vector de tipo WORD.
     * @return Intensidades en formato RAW
     */

		static byte[] getRawFromVectorWord(Vector bmp) {
				short [] aux = new short[bmp.size()];
				for (int k=0;k<bmp.size();k++) {
					aux[k]=(short) ((Integer)(bmp.elementAt(k))).intValue();	
				}
        return getRawFromWord(aux);
    }

    //************************************
    //Conversiones BRUTA (RAW) --> INTERNA
    //************************************

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels en formato RAW  a pixels de tipo BIT
     * @param pix Valor de intensidades en formato RAW
     * @return Intensidades de tipo BIT
     */

    static boolean[] getBitFromRaw(byte[] bmpRAW,int npixels) {
        boolean[] bmp=new boolean[npixels];
        for(int i=0,j=0;i<bmpRAW.length;i++) {
            for(int bit=7;bit>=0;bit--) {
                if(j<npixels) {
                    if((bmpRAW[i] & (1<<bit))!=0) bmp[j]=true;
                    else bmp[j]=false;
                }
                j++;
            }
        }
        return bmp;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels en formato RAW  a pixels de tipo BYTE
     * @param pix Valor de intensidades en formato RAW
     * @return Intensidades de tipo BYTE
     */

    static byte[] getByteFromRaw(byte[] bmpRAW) {
        int npixels=bmpRAW.length;
        byte[] bmp=new byte[npixels];
        for(int i=0;i<npixels;i++) bmp[i]=bmpRAW[i];
        return bmp;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels en formato RAW  a pixels de tipo WORD
     * @param pix Valor de intensidades en formato RAW
     * @return Intensidades de tipo WORD
     */

    static short[] getWordFromRaw(byte[] bmpRAW) {
        int npixels=bmpRAW.length/2;
        short[] bmp=new short[npixels];
        for(int i=0,j=0;i<npixels;i++) {
            bmp[i]=(short)((bmpRAW[j]<<8) | (bmpRAW[j+1])& 0x00ff);
            j+=2;
         }	
        return bmp;
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Convierte pixels en formato RAW  a pixels de tipo REAL
     * @param pix Valor de intensidades en formato RAW
     * @return Intensidades de tipo REAL
     */

    static float[] getRealFromRaw(byte[] bmpRAW) {
        int npixels=bmpRAW.length/4;
        float[] bmp=new float[npixels];
        for(int i=0,j=0;i<npixels;i++) {
        	int b0=(bmpRAW[j]<0)?(int)bmpRAW[j]+256:(int)bmpRAW[j];	
        	int b1=(bmpRAW[j+1]<0)?(int)bmpRAW[j+1]+256:(int)bmpRAW[j+1];	
        	int b2=(bmpRAW[j+2]<0)?(int)bmpRAW[j+2]+256:(int)bmpRAW[j+2];	
        	int b3=(bmpRAW[j+3]<0)?(int)bmpRAW[j+3]+256:(int)bmpRAW[j+3];	
            int val=(b0<<24)|(b1<<16)|(b2<<8)|(b3);
            bmp[i]=Float.intBitsToFloat(val);
            j+=4;
        }	
        return bmp;
    }
}