package jip;
import java.util.Vector;

 /**<P><FONT COLOR="red">
 *<B>Descripcin:</B><BR>
 *<FONT COLOR="blue">
 * Clase que define el objeto bitmap. Un bitmap representa una matriz de pixels
 * de anchura y altura constantes, o bien tambin puede contener vectores
 * de tipos geomtricos. Si es un conjunto de pixels, cada componente de la matriz 
 * contendr un valor de intensidad correspondiente al pixel que represente. 
 * Estos valores de intensidad se codifican de distinta forma segn el tipo de bitmap. Los 4 tipos
 * de bitmaps de pixels con sus rangos de valores son:
 * tBIT -> enteros [0..1] , tBYTE -> enteros [0..255] , tWORD -> enteros [0..65535]
 * y tREAL -> flotantes [0..1].
 * Tambien podemos crear un bitmap que contenga elementos geomtricos, como Puntos, Segmentos
 * Polgonos o Edges, cuyos tipos asociados son: tPOINT, tSEGMENT, tPOLY y tEDGES.
 * Podremos construir bitmaps 'vacos' (sin valores asignados pero s con anchura,
 * altura y tipo) o 'inicializados'. Al construir un bitmap, aunque est vaco,
 * hay que especificar su anchura, altura y tipo. Estas propiedades no podrn
 * variarse durante la vida del objeto.
 * Sin embargo si el tipo es geomtrico podremos crearlo a partir de un vector vacio, y mas
 * tarde aadir valores, con lo que cambiar el tamao del vector que contiene los
 * elementos geomtricos.
 * Podremos tambin asignar y obtener los valores de los pixels en conjunto
 * (como un array) o individualmente. El acceso individual se realiza especificando
 * las coordenadas x e y del pixel. Estas coordenadas se corresponden con: (0,0) ->
 * pixel de la esquina superior izquierda y (anchura-1,altura-1) -> pixel de la
 * esquina inferior derecha.
 * Igualmente podremos obtener los valores geomtricos, obteniendo un tipo vector que
 * contiene todas las coordenadas geomtricas.
 * Un bitmap no puede representar color, slo contiene intensidades de pixels.
 * Un bitmap codificar una banda de una imagen y sern las propias imgenes las que
 * puedan ser en color (3 bandas RGB representadas con sus correspondientes bitmaps).
 * @see JIP#tBIT
 * @see JIP#tBYTE
 * @see JIP#tWORD
 * @see JIP#tREAL
 * @see JIP#tPOINT
 * @see JIP#tSEGMENT
 * @see JIP#tPOLY
 * @see JIP#tEDGES
 */
public class JIPBitmap {

		/** Ancho del Bitmap */
    int width=-1;
    
    /** Alto del Bitmap */
    int height=-1;
    
    /** Tipo del Bitmap */
    int type=-1;
    
    /** Vector de Segmentos (si es de tipo SEGMENT)
    * Cada segmento ocupa cuatro posiciones del vector consecutivas. Las dos primeras
    * se corresponden al punto de inicio, y las dos ltimas al punto final del vector.
    */
    Vector Segmentos;
    
    /** Vector de Puntos (si es de tipo POINT)
    * Cada punto ocupa dos posiciones del vector consecutivas. La primera
    * posicion es la x y la segunda la y. 
    */
    Vector Puntos;
    
    /** Vector de Poligonos (si es de tipo POLY)
    * Cada posicion del vector es a su vez otro vector en la que cada dos posiciones
    * se indica un punto del vector. Habr tantos polgonos como posiciones del vector inicial.
    */
    Vector Poligonos;
    
    /** Vector de Edges (si es de tipo EDGES)
    * Cada posicion del vector es a su vez otro vector en la que cada dos posiciones
    * se indica un punto del vector. Habr tantos edges como posiciones del vector inicial. 
    */
    Vector Edges;
    
    /** Nmero de Segmentos */
    int lSegmentos=-1;
    
    /** Nmero de Puntos */
    int lPuntos=-1;
    
    /** Nmero de Polgonos */
    int lPoligonos=-1;
    
    /** Nmero de Edges */
    int lEdges=-1;
    
		/** Array de punots (si es de tipo BIT)*/
		boolean[] bmpBit=null;
		
		/** Array de puntos (si es de tipo BYTE)*/
    byte[] bmpByte=null;
    
    /** Array de puntos (si es de tipo WORD)*/
    short[] bmpWord=null;
    
    /** Array de puntos (si es de tipo REAL)*/
    float[] bmpReal=null;

    //******************//
    //  Constructores   //
    //******************//
	
     /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
 		 * Constructor de un bitmap como copia de otro.
     * @param bmp Bitmap de referencia para construir la copia.
     */
	public JIPBitmap(JIPBitmap bmp) {
		if(bmp!=null) {
			width=bmp.width;
			height=bmp.height;
			type=bmp.type;
	    switch(type) {
	        case 0:  bmpBit=new boolean[width*height];        break;
	        case JIP.tBYTE: bmpByte=new byte[width*height];   break;
	        case JIP.tWORD: bmpWord=new short[width*height];  break;
          case JIP.tREAL: bmpReal=new float[width*height];  break;
          case JIP.tPOINT:Puntos=new Vector();              break;
          case JIP.tPOLY: Poligonos=new Vector();           break;
          case JIP.tEDGES:Edges=new Vector();               break;                        
	        
	    }
    	if (type==JIP.tBIT || type==JIP.tBYTE || type==JIP.tWORD || type==JIP.tREAL) {
    	    
    	    for(int i=0;i<width*height;i++) {
		         switch(type) {
		                 case JIP.tBIT:  bmpBit[i]=bmp.bmpBit[i]; 	break;
		                 case JIP.tBYTE: bmpByte[i]=bmp.bmpByte[i];  break;
		                 case JIP.tWORD: bmpWord[i]=bmp.bmpWord[i];  break;
                     case JIP.tREAL: bmpReal[i]=bmp.bmpReal[i];  break;
		         }
    	    } 
    	}
    	else 
    	   	switch (type) {
    	    		case JIP.tPOINT:
    	    	  	Puntos=(Vector)bmp.Puntos.clone();
    	    			lPuntos=bmp.lPuntos;
    	    			break;
    	    		case JIP.tSEGMENT:
    	    			Segmentos=(Vector)bmp.Segmentos.clone();
    	    			lSegmentos=bmp.lSegmentos;
    	    			break;
    	    		case JIP.tPOLY:
    	    			Poligonos=(Vector)bmp.Poligonos.clone();
    	    			lPoligonos=bmp.lPoligonos;
    	    			break;
    	    		case JIP.tEDGES:
    	    			Edges=(Vector)bmp.Edges.clone();
    	    			lEdges=bmp.lEdges;
    	    			break;  
    	    		default: System.out.println("Error, tipo de imagen desconocido.");  	
    	    			break;
    	   	}
		}
	}
 
     /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Constructor de un bitmap vaco (sin valores de pixels).
     * @param w Anchura (columnas) del bitmap (w>0)
     * @param h Altura (filas) del bitmap (h>0)
     * @param t Tipo de bitmap.
     */
    public JIPBitmap(int w,int h,int t) {
    	if(w>0&&h>0) {
	        width=w;
	        height=h;
	        type=t;
	        switch(t) {
	        case JIP.tBIT:  bmpBit=new boolean[w*h]; break;
	        case JIP.tBYTE: bmpByte=new byte[w*h];   break;
	        case JIP.tWORD: bmpWord=new short[w*h];  break;
          case JIP.tREAL: bmpReal=new float[w*h];    	break;
	        case JIP.tSEGMENT: Segmentos=new Vector();	break;
	        case JIP.tPOINT: Puntos=new Vector(); 			break;
	        case JIP.tPOLY: Poligonos=new Vector();			break;
	        case JIP.tEDGES: Edges=new Vector();				break;
	        
	        
	        }
	    }
    }

     /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Constructor de un bitmap con valores iniciales enteros de los pixels.
     * @param w Anchura (columnas) del bitmap (w>0)
     * @param h Altura (filas) del bitmap (h>0)
     * @param t Tipo de bitmap (no se permite tREAL pues los valores iniciales
     * son enteros)
     * @param bmpInt Vector que contiene los valores iniciales de los pixels.
     * bmpInt.length debe coincidir con el nmero de pixels (w*h).
     * El rango al que se ajustarn los valores depende del tipo del bitmap:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535]
     * El orden de los pixels dentro del vector es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */
    public JIPBitmap(int w,int h,int t,int[] bmpInt) {
        if(w>0&&h>0&&w*h==bmpInt.length) {
	        width=w;
	        height=h;
	        type=t;
	        switch(t) {
	        case JIP.tBIT:  bmpBit=JIP.getAllBitFromInt(bmpInt); break;
	        case JIP.tBYTE: bmpByte=JIP.getAllByteFromInt(bmpInt); break;
	        case JIP.tWORD: bmpWord=JIP.getAllWordFromInt(bmpInt); break;
	        }
	    }
    }

     /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Constructor de un bitmap con valores iniciales reales de los pixels.
     * El tipo no se especifica pues ser tREAL.
     * @param w Anchura (columnas) del bitmap (w>0)
     * @param h Altura (filas) del bitmap (h>0)
     * @param bmpFloat Vector que contiene los valores iniciales de los pixels.
     * bmpFloat.length debe coincidir con el nmero de pixels (w*h).
     * El rango al que se ajustarn los valores es [0..1].
     * El orden de los pixels dentro del vector es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */
    public JIPBitmap(int w,int h,float[] bmpFloat) {
        if(w>0&&h>0&&w*h==bmpFloat.length) {
    	    width=w;
	        height=h;
	        type=JIP.tREAL;
	        bmpReal=JIP.getAllRealFromFloat(bmpFloat);
	    }
    }
		 /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Constructor de un bitmap con valores iniciales booleanos de los pixels.
     * El tipo no se especifica pues ser tBIT.
     * @param w Anchura (columnas) del bitmap (w>0)
     * @param h Altura (filas) del bitmap (h>0)
     * @param bmp Vector que contiene los valores iniciales de los pixels.
     * bmp.length debe coincidir con el nmero de pixels (w*h).
     * El rango al que se ajustarn los valores es [0,1].
     * El orden de los pixels dentro del vector es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */

    public JIPBitmap(int w,int h,boolean[] bmp) {
        if(w>0&&h>0&&w*h==bmp.length) {
	        width=w;
	        height=h;
	        type=JIP.tBIT;
	        bmpBit=bmp;
	    }
    }
		 
		 /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Constructor de un bitmap con valores iniciales de tipo byte de los pixels.
     * El tipo no se especifica pues ser tBYTE.
     * @param w Anchura (columnas) del bitmap (w>0)
     * @param h Altura (filas) del bitmap (h>0)
     * @param bmp Vector que contiene los valores iniciales de los pixels.
     * bmp.length debe coincidir con el nmero de pixels (w*h).
     * El rango al que se ajustarn los valores es [0..255].
     * El orden de los pixels dentro del vector es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */


    public JIPBitmap(int w,int h,byte[] bmp) {
        if(w>0&&h>0&&w*h==bmp.length) {
	        width=w;
	        height=h;
	        type=JIP.tBYTE;
	        bmpByte=bmp;
	    }
    }
		 /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Constructor de un bitmap con valores iniciales de tipo WORD de los pixels.
     * El tipo no se especifica pues ser tWORD.
     * @param w Anchura (columnas) del bitmap (w>0)
     * @param h Altura (filas) del bitmap (h>0)
     * @param bmp Vector que contiene los valores iniciales de los pixels.
     * bmp.length debe coincidir con el nmero de pixels (w*h).
     * El rango al que se ajustarn los valores es [0..65535].
     * El orden de los pixels dentro del vector es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */


    public JIPBitmap(int w,int h,short[] bmp) {
        if(w>0&&h>0&&w*h==bmp.length) {
	        width=w;
	        height=h;
	        type=JIP.tWORD;
	        bmpWord=bmp;
	    }
    }
 		 
 		 /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Constructor de un bitmap de tipo geomtrico con el vector de valores.
     * El tipo se especifica para saber si ser de tipo POINT, SEGMENT, POLY o EDGES
     * @param w Anchura (columnas) del bitmap (w>0)
     * @param h Altura (filas) del bitmap (h>0)
     * @param v Vector que contiene los valores geomtricos
     * El orden de los elementos depende del tipo:<BR>
     * - POINT: Cada punto ocupa dos posiciones del vector consecutivas. La primera
     * posicion es la x y la segunda la y. <BR>
     * - SEGMENT: Cada segmento ocupa cuatro posiciones del vector consecutivas. Las dos primeras
     * se corresponden al punto de inicio, y las dos ltimas al punto final del vector.<BR>
     * - POLY: Cada posicion del vector es a su vez otro vector en la que cada dos posiciones
     * se indica un punto del vector. Habr tantos polgonos como posiciones del vector inicial.<BR>
     * - EDGES: Cada posicion del vector es a su vez otro vector en la que cada dos posiciones
     * se indica un punto del vector. Habr tantos edges como posiciones del vector inicial. <BR>
     */
	
		public JIPBitmap(int w, int h, int t, Vector v) {
			type=t;
			switch (type) {
					case JIP.tPOINT:    Puntos=new Vector();
					                    Puntos=(Vector)v.clone();
					                    lPuntos=Puntos.size();
					                    break;					                 
					case JIP.tSEGMENT:  Segmentos=new Vector();
															Segmentos=(Vector)v.clone();
															lSegmentos=Segmentos.size();
															break;
					case JIP.tPOLY:			Poligonos=new Vector();
															Poligonos=(Vector)v.clone();
															lPoligonos=Poligonos.size();
															break;
					case JIP.tEDGES:    Edges=new Vector();
															Edges=(Vector)v.clone();
															lEdges=Edges.size();
															break;
			}
		}
		


    //*******************//
    //     METODOS       //
    //*******************//

    /******************************************/
    /* Obtencion de caracteristicas generales */
    /******************************************/

    /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el nmero de pixels del bitmap.
     * @return Nmero de pixels (filas*columnas)
     */
    public int getNumPixels() { return width*height; }

    /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene la anchura del bitmap.
     * @return Nmero de columnas.
     */
    public int getWidth()  { return width; }

    /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene la altura del bitmap.
     * @return Nmero de filas.
     */
    public int getHeight() { return height; }

    /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el tipo de bitmap.
     * @return Cte que identifica el tipo.
     * @see JIP#tBIT
     * @see JIP#tBYTE
     * @see JIP#tWORD
     * @see JIP#tREAL
     * @see JIP#tPOINT
     * @see JIP#tSEGMENT
		 * @see JIP#tPOLY
     * @see JIP#tEDGES
     */
    public int getType()   { return type; }

    /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el vector de puntos del bitmap
     * @return Vector de puntos
     */
   public Vector getVectorPoint() { return Puntos; }
  	 /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el vector de segmentos del bitmap
     * @return Vector de segmentos
     */
	  public Vector getVectorSegment() { return Segmentos; }
		/**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el vector de poligonos del bitmap
     * @return Vector de poligonos
     */
		public Vector getVectorPoly() { return Poligonos; }
		/**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el vector de edges del bitmap
     * @return Vector de edges
     */
	  public Vector getVectorEdges() { return Edges; }
		
		/**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene nmero de puntos del Vector de puntos
     * @return Nmero de puntos
     */

		public int getLVectorPoint() { return lPuntos; }
		/**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene nmero de Segmentos del Vector de segmentos
     * @return Nmero de segmentos
     */
    			
		public int getLVectorSegment() { return lSegmentos; }
		
		/**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene nmero de poligonos del Vector de poligonos
     * @return Nmero de poligonos
     */

		public int getLVectorPoly() { return lPoligonos; }

		/**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene nmero de edges del Vector de edges
     * @return Nmero de edges
     */

		public int getLVectorEdges() { return lEdges; }


    			

    /**************************************/
    /* Asignacion de valores a los pixels */
    /**************************************/

     /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna el valor entero de un pixel a un bitmap de tipo tBIT, tBYTE o tWORD.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @param pix Valor asignado al pixel.
     * El rango depende del tipo de bitmap:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535]
     */
 
    public void setPixel(int x,int y,int pix) {
        if(x<width&&y<height&&x>=0&&y>=0) {
            switch(type) {
            case JIP.tBIT:  bmpBit[x+y*width]=JIP.getBitFromInt(pix);
                            break;
            case JIP.tBYTE: bmpByte[x+y*width]=JIP.getByteFromInt(pix);
                            break;
            case JIP.tWORD: bmpWord[x+y*width]=JIP.getWordFromInt(pix);
                            break;
            }
        }
    }

    /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna el valor real de un pixel a un bitmap de tipo tREAL.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @param pix Valor asignado al pixel.
     * El rango de valores es [0..1]
     */
    public void setPixel(int x,int y,float pix) {
        if(x<width&&y<height&&x>=0&&y>=0&&type==JIP.tREAL)
        	bmpReal[x+y*width]=JIP.getRealFromFloat(pix);
    }

    /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna el valor de todos los pixels a un bitmap de tipo tBIT, tBYTE o tWORD.
     * @param bmpInt Vector que contiene los valores de los pixels.
     * bmpInt.length debe coincidir con el nmero de pixels (anchura*altura)
     * El rango al que se ajustarn los valores depende del tipo del bitmap:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535]
     * El orden de los pixels dentro del vector es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */
    public void setAllPixel(int[] bmpInt) {
        if(width*height==bmpInt.length) {
            switch(type) {
            case JIP.tBIT:  bmpBit=JIP.getAllBitFromInt(bmpInt); break;
            case JIP.tBYTE: bmpByte=JIP.getAllByteFromInt(bmpInt); break;
            case JIP.tWORD: bmpWord=JIP.getAllWordFromInt(bmpInt); break;
            }
        }
    }

    /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna el valor de los pixels a un Bitmap de tipo tREAL.
     * @param bmpFloat Vector que contiene los valores de los pixels.
     * bmpFloat.length debe coincidir con el numero de pixels (anchura*altura)
     * El rango al que se ajustarn los valores es [0..1].
     * El orden de los pixels dentro del vector es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */
    public void setAllPixel(float[] bmpFloat) {
        if(width*height==bmpFloat.length&&type==JIP.tREAL) {
			bmpReal=JIP.getAllRealFromFloat(bmpFloat);
        }
    }

	 	 /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna el valor de un vector de Geometria a un Bitmap de tipo
     * tPOINT, tSEGMENT, tPOLY o tEDGES
     * @param t: Tipo del bitmap
     * @param v: Vector que contiene la geometra.
     */
		 
		 public void setVector(int t, Vector v) {
 		 		switch (t) {
		 			case JIP.tPOINT: 		
             
             Puntos=(Vector)v.clone();	
		 													break;
		 			case JIP.tSEGMENT: 	Segmentos=(Vector)v.clone();	
		 													break;
		 			case JIP.tPOLY: 		Poligonos=(Vector)v.clone();	
		 													break;
		 			case JIP.tEDGES: 		Edges=(Vector)v.clone();	
		 													break;		 															 															 												
		 		}
		 	
		 }

		/**************************************/
    /* Aadir valores sueltos a vectores  */
    /**************************************/
		
		 /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Aade un segmento cuyas coordenadas se espeficican en los parmetros de entrada,
     * a un bitmap de tipo tSEGMENT.
     * @param x0: Coordenada X del punto inicial.
     * @param y0: Coordenada Y del punto inicial.
     * @param x1: Coordenada X del punto final.
     * @param y1: Coordenada Y del punto final.
     */
		
		public void addSegment(int x0,int y0,int x1,int y1) {
				if (type!=JIP.tSEGMENT) return;
			 Integer aux=new Integer (x0);
       Segmentos.addElement(aux);
       aux=new Integer(y0);
       Segmentos.addElement(aux);
				aux=new Integer(x1);
       Segmentos.addElement(aux);
				aux=new Integer(y1);
       Segmentos.addElement(aux);
       lSegmentos=Segmentos.size();
 		}

		 /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Aade un punto cuyas coordenadas se espeficican en los parmetros de entrada,
     * a un bitmap de tipo tPOINT.
     * @param x0: Coordenada X del punto.
     * @param y0: Coordenada Y del punto.
     */


		public void addPoint(int x0, int y0) {
				if (type!=JIP.tPOINT) return;
  			Integer aux=new Integer (x0);
       Puntos.addElement(aux);
       aux=new Integer(y0);
       Puntos.addElement(aux);
		}

    /**************************************/
    /* Obtencion de valores de los pixels */
    /**************************************/

     /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
 		 * Obtiene el valor de un pixel de un bitmap de tipo tBIT, tBYTE o tWORD.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @return Valor entero del pixel.
     * El rango del valor obtenido depende del tipo de bitmap:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535]
     */
    
    public int getPixel(int x,int y) {
        int res=-1;
        if(x<width&&y<height&&x>=0&&y>=0) {
            switch(type) {
            case JIP.tBIT:  res=JIP.getIntFromBit(bmpBit[x+y*width]);
                            break;
            case JIP.tBYTE: res=JIP.getIntFromByte(bmpByte[x+y*width]);
                            break;
            case JIP.tWORD: res=JIP.getIntFromWord(bmpWord[x+y*width]);
                            break;
           }
        }
        return res;
    }

     /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
 		 * Obtiene el valor de un pixel de un bitmap de tipo tREAL.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @return Valor real del pixel.
     * El rango del valor obtenido es [0..1]
     */
    public float getPixelFlo(int x,int y) {
        float res=-1;
        if(x<width&&y<height&&x>=0&&y>=0&&type==JIP.tREAL)
            res=bmpReal[x+y*width];
        return res;
    }

     /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
 		 * Obtiene el valor de los pixels de un bitmap de tipo tBIT, tBYTE o tWORD.
     * @return Vector que contiene los valores enteros de los pixels.
     * La longitud del vector devuelto es anchura*altura.
     * El rango de los valores obtenidos depende del tipo del bitmap:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535]
     * El orden de los pixels dentro del vector es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */
    public int[] getAllPixel() {
        int[] res=null;
        switch(type) {
        case JIP.tBIT:  res=JIP.getAllIntFromBit(bmpBit);
                        break;
        case JIP.tBYTE: res=JIP.getAllIntFromByte(bmpByte);
                        break;
        case JIP.tWORD: res=JIP.getAllIntFromWord(bmpWord);
                        break;
        }
        return res;
    }

     /**<P><FONT COLOR="red">
 		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
 		 * Obtiene el valor de los pixels de un bitmap de tipo tREAL.
     * @return Vector que contiene los valores reales de los pixels.
     * La longitud del vector devuelto es anchura*altura.
     * El rango de los valores obtenidos es [0..1]
     * El orden de los pixels dentro del vector es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */
    public float[] getAllPixelFlo() {
        float[] res=null;
        if(type==JIP.tREAL) res=JIP.getAllRealFromFloat(bmpReal);
        return res;
    }
}