package jip;

 /**<P><FONT COLOR="red">
 *<B>Descripcin:</B><BR>
 *<FONT COLOR="blue">
 *    Clase base para definir objetos comando. Un comando es un objeto que tiene
 * un objeto funcin asociado y que nos permite crear de forma simple una
 * aplicacin que procese dicha funcin sobre una secuencia contenida en un
 * fichero, devolviendo en otro fichero la secuencia de salida.
 * No podremos crear directamente objetos de la clase comando, tendremos que
 * crear subclases de sta que al heredar implementen como mnimo el constructor
 * (que debe asignar la funcin asociada al comando) y el mtodo main() que
 * recoja los argumentos desde la lnea de comandos y lance la ejecucin del
 * comando en s. Internamente el objeto comando al ejecutar chequear los
 * argumentos introducidos y dar valores a los parmetros de la funcin,
 * cargar la secuencia de entrada y obtendr el nombre del fichero en el que
 * almacenar la secuencia de salida (por defecto "out.jip"). En caso de que
 * algn parmetro requerido no tome valor, o se especifique un parmetro
 * desconocido o no se de el nombre del fichero
 * que contiene la secuencia de entrada, se presentar por la salida estndar
 * una ayuda construda con las descripciones obtenidas de la funcin asociada
 * y sus parmetros. Sea cual sea la funcin asociada al comando y tenga o no
 * parmetros, siempre se admitir un argumento -help que provocar la presentacin
 * de las lineas de ayuda. Si la funcin asociada al comando devuelve resultados
 * adicionales, podremos acceder a ellos en main() tras la ejecucin y mostrarlos
 * por pantalla o generar ficheros con ellos.
 */
public abstract class JIPCommand {

     /**
     * Variable que contendr el nombre del fichero en el que salvar la secuencia
     * resultado tras procesar el comando. Esta variable toma valor tras invocar
     * con xito el mtodo parseArgs().
     * @see JIPCommand#parseArgs
     */
	protected String fichdest;

    /**
     * Variable que contendr la secuencia de entrada a procesar.
     * Esta variable toma valor tras invocar con xito el mtodo parseArgs(), el
     * cual la cargar desde el fichero de entrada especificado en los argumentos.
     * @see JIPCommand#parseArgs
     */
    protected JIPSequence source;

    /**
     * Variable que contendr el objeto funcin asociado al comando.
     * Slo ser accesible desde las clases derivadas que debern asignar esta
     * variable en su constructor.
     * @see jip.JIPFunction
     */
    protected JIPFunction function;

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Constructor de un comando. La clase derivada debe implementarlo asignando
     * la variable function.
     */
    public JIPCommand() {}

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Invoca la ejecucin del comando a partir de un vector de argumentos.
     * Los argumentos pueden ser:
     * -help -> Provoca la muestra de unas lineas de ayuda basadas en las
     * descripciones de la funcin y sus parmetros.
     * -nombre_parametro -> Provoca la asignacin de un valor a un
     * parmetro de la funcin asociada al comando. El valor a asignar ser
     * el siguiente argumento al actual (se comprobar que el valor
     * corresponde al tipo de parmetro y que este parmetro existe).
     * nombre_fichero -> La primera aparicin es obligatoria y se referir
     * a un fichero que contenga la secuencia de entrada a procesar
     * (se comprobar que el fichero exista y tenga formato JIP).
     * La segunda aparicin es opcional y especificar un nombre de
     * fichero para la secuencia de salida (por defecto out.jip).
     * En el caso de que algo falle (no se especifique el fichero de entrada,
     * no se de valor a un parmetro requerido, ...) la ejecucin mostrar un
     * error y las lineas de ayuda del comando. Este mtodo invoca internamente
     * al mtodo parseArgs() con los mismos argumentos que recibe.
     * @param args Vector de argumentos obtenidos de la linea de comando de la
     * aplicacin. Normalmente ser directamente el vector recibido en el mtodo
     * main() de la aplicacin.
     */
    
    public void execute(String[] args) {
		source=null;	
		fichdest=null;
		if(function!=null&&parseArgs(args)) {
		    // Aplica la funcion y almacena la salida
            JIPSequence dest=function.processSeq(source);
            JIPToolkit.saveSeqIntoFile(dest,fichdest);
		}
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Muestra un error por la salida de error estndar y las lineas de ayuda del
     * comando por la salida estndar.
     * @param str Error a mostrar.
     */
    public void error(String str) {
		System.err.println("*** ERROR: "+str+" ***");
		help();
    }
    
     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Muestra las lineas de ayuda del comando por la salida estndar.
     * Esta ayuda se construye a partir de los nombres y descripciones de la
     * funcin asociada al comando y de sus parmetros.
     */
    public void help() {
		String[] params=function.getParamNames();
		int i;
		String aux1;

		System.out.println("");
		System.out.println("FUNCION:");
		System.out.println("  "+function.getDescription());
		System.out.println("");
		System.out.println("Forma de uso: java "+
			function.getName().substring(1)+
			" <parametros> <infile> [<outfile>]");
		System.out.println("  <infile>:  Fichero fuente a procesar [REQUERIDO]");
		System.out.println("  <outfile>: Fichero destino [Por Defecto: out.jip]");
		System.out.println("");
		System.out.println("Parametros:");
		System.out.println("  -help");
		System.out.println("    Muestra forma de uso");
		if(params!=null) {
			for(i=0;i<params.length;i++) {
				aux1="  -"+params[i]+" ";
				
				switch(function.getParamType(params[i])) {
				case JIP.pBOOL: aux1=aux1+"<boolean>"; break;
				case JIP.pINT:  aux1=aux1+"<integer>"; break;
				case JIP.pREAL: aux1=aux1+"<real>"; break;
				case JIP.pSTRING: aux1=aux1+"<string>"; break;
				case JIP.pIMAGE: aux1=aux1+"<imagen>"; break;
				}
				
				if(function.isParamRequired(params[i]))
					aux1=aux1+" [REQUERIDO]";
				else {
					aux1=aux1+" [Por Defecto: ";
					switch(function.getParamType(params[i])) {
					case JIP.pBOOL:
					    aux1=aux1+function.getParamValueBool(params[i]);
					    break;
					case JIP.pINT:
					    aux1=aux1+function.getParamValueInt(params[i]);
					    break;
					case JIP.pREAL:
					    aux1=aux1+function.getParamValueReal(params[i]);
					    break;
					case JIP.pSTRING:
					    aux1=aux1+function.getParamValueStr(params[i]);
					    break;
					case JIP.pIMAGE:
					    aux1=aux1+function.getParamValueImg(params[i]).getName();
					    break;
					}
					aux1=aux1+"]";
			    }
			    System.out.println(aux1);
			    System.out.println("    "+function.getParamDescr(params[i]));
			}
		}
		System.out.println("");
    }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Este mtodo se encarga de chequear los argumentos que le pasamos y preparar el
     * comando para su ejecucin. Comprobar que los parmetros existen en la funcin
     * del comando, asignar los valores comprobando que son del tipo adecuado y cargar
     * la secuencia de entrada asignndola a la variable source del comando. Para parmetros
     * del tipo pIMAGE se asignar como imagen el primer frame del fichero especificado
     * como valor del parmetro, comprobndose que existe y tiene formato JIP. Tambin
     * asignar el nombre del fichero de salida a la variable fichdest (por defecto out.jip si
     * no se especifica en los argumentos) y comprobar que todos los parmetros
     * requeridos de la funcin han tomado valor. Si algo no va bien, el mtodo mostrar
     * un error y devolver FALSE. Si estamos utilizando el mtodo execute(),
     * no tiene sentido utilizar antes ste mtodo pues ya lo hace execute(). Este mtodo es til
     * cuando queramos controlar nosotros la ejecucin del comando y por tanto no utilicemos
     * execute(). Por ejemplo, si deseamos ir ejecutando el comando sobre la secuencia de entrada
     * frame a frame, para ir accediendo a ciertos resultados propios del procesamiento de cada frame,
     * podemos utilizar este mtodo al principio  ara chequear los argumentos y cargar la secuencia
     * de entrada en la variable source, luego meternos en un bucle que procese cada frame de la
     * secuencia con el mtodo function.processImg(frame), obtener los resultados obtenidos del
     * proceso y mostrarlos, e ir creando la secuencia de salida para almacenarla al final en
     * el fichero con nombre fichdest.
     * @param args Vector de argumentos de la linea de comando.
     * @return TRUE si todo ha ido correctamente.
     */
	public boolean parseArgs(String[] args) {
		String pname,pval,fichsrc;
		int ptipo;
	
		fichsrc=null;
		pname=null;
		pval=null;
	
		if(function==null) {
	   	    error("El comando no tiene funcion asignada. Revise el constructor.");
		    return false;
		}
		
		for(int i=0;i<args.length;i++) {

			// EL ARGUMENTO ES UN PARAMETRO
			if(args[i].trim().startsWith("-")) {

				//Comprobar si es -help
				if(args[i].equals("-help")) {
					help();
					return false;
				}

				//Comprobar que el parametro existe
				pname=args[i].trim().substring(1);
				if(!function.isParam(pname)) {
					error("Parametro -"+pname+" desconocido");
					return false;
				}

				//Comprobar si el valor es de tipo valido
				if(i+1==args.length) {
					error("Parametro -"+pname+" sin valor asociado");
					return false;
				}
	
				pval=args[i+1].trim();
				ptipo=function.getParamType(pname);
				switch(ptipo) {
				case JIP.pBOOL:
					if(pval.toUpperCase().equals("TRUE"))
						function.setParamValue(pname,true);
					else if(pval.toUpperCase().equals("FALSE"))
						function.setParamValue(pname,false);
					else {
						error("Parametro -"+pname+" espera un valor BOOLEANO");
						return false;
					}
				break;
				case JIP.pINT:
					try { function.setParamValue(pname,
						Integer.valueOf(pval).intValue()); }
					catch(NumberFormatException e) {
						error("Parametro -"+pname+" espera un valor ENTERO");
						return false;
					}
				break;
				case JIP.pREAL:
					try { function.setParamValue(pname,
						Float.valueOf(pval).floatValue()); }
					catch(NumberFormatException e) {
						error("Parametro -"+pname+" espera un valor REAL");
						return false;
					}
				break;
				case JIP.pSTRING:
					function.setParamValue(pname,pval);
				break;
				case JIP.pIMAGE:
	                JIPSequence auxseq=JIPToolkit.getSeqFromFile(pval);
	                if(auxseq!=null) {
						function.setParamValue(pname,auxseq.getFrame(0));
					}
					else {
						error("Parametro -"+pname+" espera un fichero jip");
						return false;
					}
				break;
				}	    	
				
				//Salta el valor del parmetro
				i++;
			}
			else if(fichsrc==null) fichsrc=args[i].trim();
			else if(fichdest==null) fichdest=args[i].trim();
			else {
				error("Valor "+args[i]+" sin parametro asociado");
				return false;
			}	
		}

		//Comprobar fichero fuente
		if(fichsrc==null) {
	   	    error("Se requiere el fichero fuente a procesar");
		    return false;
		}
		if(!(new java.io.File(fichsrc)).isFile()) {
    	    error("El fichero '"+fichsrc+"' NO EXISTE");
		    return false;
		}
        source=JIPToolkit.getSeqFromFile(fichsrc);
        if(source==null) {
    	    error("El fichero '"+fichsrc+"' NO TIENE FORMATO JIP");
    	    return false;
    	}

		//Comprobar fichero destino
		if(fichdest==null) fichdest="out.jip";

		//Comprobar parametros requeridos
		if(!function.paramsOK()) {
    	    error("Falta especificar algun parametro requerido");
		    return false;
		}
	
		return true;
    }

}