package jip;
import java.util.Vector;



 /**<P><FONT COLOR="red">
 *<B>Descripcin:</B><BR>
 *<FONT COLOR="blue">
 *     Clase que define el objeto imagen. Una imagen representa un conjunto de bandas,
 * cada una de las cuales consiste en un bitmap. La anchura, altura y tipo de cada
 * banda son las mismas para todas y sern la anchura, altura y tipo de la imagen.
 * A cada una de las bandas se acceder con un entero desde 0 a numbandas-1.
 * Una imagen puede ser de cualquiera de los tipos posibles de bitmap siendo los
 * rangos de valores de los pixels los mismos:
 * tBIT -> enteros [0..1] , tBYTE -> enteros [0..255] , tWORD -> enteros [0..65535]
 * y tREAL -> flotantes [0..1]. Adems una imagen podr ser del tipo tCOLOR. Este
 * tipo se corresponde con una imagen de 3 bandas cuyos bitmaps son de tipo tBYTE.
 * Las bandas de una imagen tCOLOR se corresponden con las intensidades RGB y
 * podremos acceder a ellas con las constantes definidas en la clase JIP: bRED,
 * bGREEN y bBLUE. El acceso a los pixels de una imagen tCOLOR podr realizarse
 * con valores de intensidad RGB empaquetados en un entero o con valores de
 * intensidades Red, Green o Blue de forma separada. De esta forma podremos
 * trabajar con imgenes en color tal y como si el concepto de banda no existiese.
 * Para imgenes en gris de una sola banda podremos utilizar tambin constructores
 * y mtodos de acceso a los pixels sin tener que especificar siempre la banda.
 * De esta forma tambin podremos trabajar con imgenes en gris tal y como si
 * el concepto de banda no existiese. Por ltimo, a una imagen podremos darle un
 * nombre que por defecto ser "[Unnamed]".
 */
public class JIPImage {

		/** Nmero de bandas de la Imagen */
    int nbands;
    /** Ancho de la imagen */
    int width;
    /** Alto de la imagen */
    int height;
    /** Tipo de la imagen*/ 
    int type;
    /** Nombre de la imagen*/
    String name="[Unnamed]";
    /** Array de Bitmaps que contienen los valores de la imagen*/
    JIPBitmap[] bmp;

    //******************//
    //  Constructores   //
    //******************//

		/**<P><FONT COLOR="red">
 		*<B>Descripcin:</B><BR>
 		*<FONT COLOR="blue">
 		*Constructor Vacio de la imagen 
 		*/
		
    JIPImage() {}

    /**<P><FONT COLOR="red">
 		*<B>Descripcin:</B><BR>
 		*<FONT COLOR="blue">
    * Constructor de una imagen como copia de otra.
    * @param img Imagen de referencia para construir la copia.
    */
	public JIPImage(JIPImage img) {
		if(img!=null) {
			nbands=img.nbands;
			width=img.width;
			height=img.height;
			type=img.type;
			name=new String(img.name);
	        bmp=new JIPBitmap[nbands];
        	for(int i=0;i<nbands;i++)
        		bmp[i]=new JIPBitmap(img.bmp[i]);
		}
	}

    /**<P><FONT COLOR="red">
 		*<B>Descripcin:</B><BR>
 		*<FONT COLOR="blue">
    * Constructor de una imagen vaca (sin valores de pixels).
    * @param w Anchura (columnas) de la imagen (w>0)
    * @param h Altura (filas) de la imagen (h>0)
    * @param t Tipo de imagen. Si el tipo es tCOLOR la imagen tendr internamente
    * 3 bandas RGB. En otro caso tendr una nica banda.
    */
    public JIPImage(int w,int h,int t) {
    	if(w>0&&h>0) {
	        nbands=(t==JIP.tCOLOR)?3:1;
	        bmp=new JIPBitmap[nbands];
	        width=w;
	        height=h;
	        type=t;
	        if(t==JIP.tCOLOR) {
	            bmp[JIP.bRED]=new JIPBitmap(w,h,JIP.tBYTE);
	            bmp[JIP.bGREEN]=new JIPBitmap(w,h,JIP.tBYTE);
	            bmp[JIP.bBLUE]=new JIPBitmap(w,h,JIP.tBYTE);
	        }
	        else bmp[0]=new JIPBitmap(w,h,t);
	    }
    }

     /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Constructor de una imagen vaca con b bandas (sin valores de pixels).
     * @param b Nmero de bandas de la imagen (b>0)
     * @param w Anchura (columnas) de la imagen (w>0)
     * @param h Altura (filas) de la imagen (h>0)
     * @param t Tipo de imagen. Si el tipo es tCOLOR la imagen tendr internamente
     * 3 bandas RGB independientemente del b especificado.
     * En otro caso tendr b bandas.
     */
    public JIPImage(int b,int w,int h,int t) {
    	if(b>0&&w>0&&h>0) {
	        nbands=(t==JIP.tCOLOR)?3:b;
	        bmp=new JIPBitmap[nbands];
	        width=w;
	        height=h;
	        type=t;
	        if(t==JIP.tCOLOR) {
	            bmp[JIP.bRED]=new JIPBitmap(w,h,JIP.tBYTE);
	            bmp[JIP.bGREEN]=new JIPBitmap(w,h,JIP.tBYTE);
	            bmp[JIP.bBLUE]=new JIPBitmap(w,h,JIP.tBYTE);
	        }
	        else for(int i=0;i<b;i++) bmp[i]=new JIPBitmap(w,h,t);
	    }
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Constructor de una imagen con valores iniciales enteros de los pixels.
     * @param w Anchura (columnas) de la imagen (w>0)
     * @param h Altura (filas) de la imagen (h>0)
     * @param t Tipo de la imagen (no se permite tREAL pues los valores iniciales
     * son enteros). Si el tipo es tCOLOR la imagen tendr internamente
     * 3 bandas RGB y los enteros del vector bmpInt se considerarn como valores
     * RGB empaquetados. En otro caso tendr una nica banda.
     * @param bmpInt Vector que contiene los valores iniciales de los pixels.
     * bmpInt.length debe coincidir con el nmero de pixels (w*h).
     * El rango al que se ajustarn los valores depende del tipo de imagen:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535] , tCOLOR -> RGB
     * empaquetado.
     * El orden de los pixels dentro del vector es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */
    public JIPImage(int w,int h,int t,int[] bmpInt) {
        if(w>0&&h>0&&w*h==bmpInt.length) {
	        nbands=(t==JIP.tCOLOR)?3:1;
	        bmp=new JIPBitmap[nbands];
	        width=w;
	        height=h;
	        type=t;
	        if(t==JIP.tCOLOR) {
	            bmp[JIP.bRED]=
	                new JIPBitmap(w,h,JIP.tBYTE,JIP.getAllRedFromRGB(bmpInt));
	            bmp[JIP.bGREEN]=
	                new JIPBitmap(w,h,JIP.tBYTE,JIP.getAllGreenFromRGB(bmpInt));
	            bmp[JIP.bBLUE]=
	                new JIPBitmap(w,h,JIP.tBYTE,JIP.getAllBlueFromRGB(bmpInt));
	        }
	        else bmp[0]=new JIPBitmap(w,h,t,bmpInt);
	    }
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Constructor de una imagen en color con valores iniciales de los pixels
     * RGB por separado.
     * @param w Anchura (columnas) de la imagen (w>0)
     * @param h Altura (filas) de la imagen (h>0)
     * @param red Vector que contiene los valores RED iniciales de los pixels.
     * @param green Vector que contiene los valores GREEN iniciales de los pixels.
     * @param blue Vector que contiene los valores BLUE iniciales de los pixels.
     * Las longitudes de los 3 vectores deben coincidir con el nmero de
     * pixels (w*h).
     * El rango al que se ajustarn los valores de los vectores ser [0..255]
     * El orden de los pixels dentro de los vectores es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */
    public JIPImage(int w,int h,int[] red,int[] green,int[] blue) {
        if(w>0&&h>0&&w*h==red.length&&w*h==green.length&&w*h==blue.length) {
	        nbands=3;
	        bmp=new JIPBitmap[3];
	        width=w;
	        height=h;
	        type=JIP.tCOLOR;
	        bmp[JIP.bRED]=new JIPBitmap(w,h,JIP.tBYTE,red);
	        bmp[JIP.bGREEN]=new JIPBitmap(w,h,JIP.tBYTE,green);
	        bmp[JIP.bBLUE]=new JIPBitmap(w,h,JIP.tBYTE,blue);
	    }
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Constructor de una imagen de una sola banda de tipo tREAL con valores
     * iniciales reales de los pixels.
     * @param w Anchura (columnas) de la imagen (w>0)
     * @param h Altura (filas) de la imagen (h>0)
     * @param bmpFloat Vector que contiene los valores iniciales de los pixels.
     * bmpFloat.length debe coincidir con el nmero de pixels (w*h).
     * El rango al que se ajustarn los valores es [0..1]
     * El orden de los pixels dentro del vector es por filas:
     * ( valores fila 0, valores fila 1, ... , valores fila h-1 ) siendo
     * la fila 0 la superior.
     */
    public JIPImage(int w,int h,float[] bmpFloat) {
    	if(w>0&&h>0&&w*h==bmpFloat.length) {
	        nbands=1;
	        bmp=new JIPBitmap[1];
	        width=w;
	        height=h;
	        type=JIP.tREAL;
	        bmp[0]=new JIPBitmap(w,h,bmpFloat);
	    }
    }
    
    
    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Constructor de una imagen de una sola banda de tipo Geometrico, especificado
     * mediante argumento, y a partir del vector de entrada.
     * @param w Anchura (columnas) de la imagen (w>0)
     * @param h Altura (filas) de la imagen (h>0)
     * @param t Tipo de la Imagen (tPOINT, tSEGMENT, tPOLY, tEDGES)
     * @param v Vector que contiene los datos geometricos
     */
    
    public JIPImage(int w, int h, int t, Vector v) {
	        nbands=1;
	        bmp=new JIPBitmap[nbands];
	        width=w;
	        height=h;
	        type=t;
	        bmp[0]=new JIPBitmap(w,h,t,v);
    }

    //*******************//
    //     METODOS       //
    //*******************//

    /*******************************************************/
    /* Obtencion y asignacion de caracteristicas generales */
    /*******************************************************/

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el nmero de bandas de la imagen.
     * @return Nmero de bandas.
     */
    public int getNumBands() { return nbands; }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el nmero de pixels de la imagen.
     * @return Nmero de pixels (anchura*altura)
     */
    public int getNumPixels() { return width*height; }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene la anchura de la imagen.
     * @return Nmero de columnas.
     */
    public int getWidth()  { return width; }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene la altura de la imagen.
     * @return Nmero de filas.
     */
    public int getHeight() { return height; }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el tipo de la imagen.
     * @return Cte que identifica el tipo.
     * @see JIP#tBIT
     * @see JIP#tBYTE
     * @see JIP#tWORD
     * @see JIP#tREAL
     * @see JIP#tCOLOR
     * @see JIP#tPOINT
		 * @see JIP#tSEGMENT
 		 * @see JIP#tPOLY
 		 * @see JIP#tEDGES
     */
    public int getType()   { return type; }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el nombre de la imagen.
     * @return Nombre de la imagen.
     */
    public String getName() { return name; }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna el nombre de la imagen.
     * @param nom Nombre de la imagen.
     */
    public void setName(String nom) {
        if(nom==null||nom.length()==0) name="[Unnamed]";
        else name=new String(nom);
    }

    /**************************************/
    /* Asignacion de valores a los pixels */
    /**************************************/
 
     /**<P><FONT COLOR="red">
   	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna un valor entero a un pixel de una banda (b,x,y) de una imagen de tipo
     * tBIT, tBYTE, tWORD o tCOLOR.
     * @param b Banda a la que pertenece el pixel (0 <= b <= numbands-1)
     * Si la imagen es de tipo tCOLOR, b puede referirse con las constantes
     * bRED, bGREEN o bBLUE.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @param pix Valor asignado al pixel.
     * El rango depende del tipo de imagen:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535] , tCOLOR -> [0..255]
     */
    public void setPixel(int b,int x,int y,int pix) {
        if(b>=0&&b<nbands) bmp[b].setPixel(x,y,pix);
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna el valor de intensidad RED a un pixel (bRED,x,y) de una imagen
     * en color. Equivale a: setPixel(JIP.bRED,x,y,red) de una imagen en color.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @param red Valor asignado a la intensidad RED del pixel.
     * El rango de intensidad es [0..255]
     */
    public void setPixelRed(int x,int y,int red) {
        if(type==JIP.tCOLOR) setPixel(JIP.bRED,x,y,red);
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna el valor de intensidad GREEN a un pixel (bGREEN,x,y) de una imagen
     * en color. Equivale a: setPixel(JIP.bGREEN,x,y,green) de una imagen en color.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @param green Valor asignado a la intensidad GREEN del pixel.
     * El rango de intensidad es [0..255]
     */
    public void setPixelGreen(int x,int y,int green) {
        if(type==JIP.tCOLOR) setPixel(JIP.bGREEN,x,y,green);
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna el valor de intensidad BLUE a un pixel (bBLUE,x,y) de una imagen
     * en color. Equivale a: setPixel(JIP.bBLUE,x,y,blue) de una imagen en color.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @param blue Valor asignado a la intensidad BLUE del pixel.
     * El rango de intensidad es [0..255]
     */
    public void setPixelBlue(int x,int y,int blue) {
        if(type==JIP.tCOLOR) setPixel(JIP.bBLUE,x,y,blue);
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna un valor entero a un pixel de la banda 0, (0,x,y) de una imagen
     * de tipo tBIT, tBYTE o tWORD o un valor RGB empaquetado a un pixel (x,y)
     * de una imagen de tipo tCOLOR.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @param pix Valor asignado al pixel.
     * El rango depende del tipo de imagen:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535] , tCOLOR ->
     * RGB empaquetado.
     */
    public void setPixel(int x,int y,int pix) {
        if(type!=JIP.tCOLOR) bmp[0].setPixel(x,y,pix);
        else {
            bmp[JIP.bRED].setPixel(x,y,JIP.getRedFromRGB(pix));
            bmp[JIP.bGREEN].setPixel(x,y,JIP.getGreenFromRGB(pix));
            bmp[JIP.bBLUE].setPixel(x,y,JIP.getBlueFromRGB(pix));
        }
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna valores enteros a los pixels de una banda (b,x,y) de una imagen
     * de tipo tBIT, tBYTE, tWORD o tCOLOR.
     * @param b Banda a la que pertenecen los pixels (0 <= b <= numbands-1)
     * Si la imagen es de tipo tCOLOR, b puede referirse con las constantes
     * bRED, bGREEN o bBLUE.
     * @param bmpInt Vector de valores asignados a los pixels.
     * El rango de los valores del vector depende del tipo de imagen:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535] , tCOLOR -> [0..255]
     */
    public void setAllPixel(int b,int[] bmpInt) {
        if(b>=0&&b<nbands) bmp[b].setAllPixel(bmpInt);
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna valores de intensidad RED a los pixels de una imagen
     * en color. Equivale a: setAllPixel(JIP.bRED,red) de una imagen en color.
     * @param red Vector de valores asignados a la intensidad RED de los pixels.
     * El rango de intensidad es [0..255]
     */
    public void setAllPixelRed(int[] red) {
        if(type==JIP.tCOLOR) setAllPixel(JIP.bRED,red);
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna valores de intensidad GREEN a los pixels de una imagen
     * en color. Equivale a: setAllPixel(JIP.bGREEN,green) de una imagen en color.
     * @param green Vector de valores asignados a la intensidad GREEN de los pixels.
     * El rango de intensidad es [0..255]
     */
    public void setAllPixelGreen(int[] green) {
        if(type==JIP.tCOLOR) setAllPixel(JIP.bGREEN,green);
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna valores de intensidad BLUE a los pixels de una imagen
     * en color. Equivale a: setAllPixel(JIP.bBLUE,blue) de una imagen en color.
     * @param blue Vector de valores asignados a la intensidad BLUE de los pixels.
     * El rango de intensidad es [0..255]
     */
    public void setAllPixelBlue(int[] blue) {
        if(type==JIP.tCOLOR) setAllPixel(JIP.bBLUE,blue);
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna valores enteros a los pixels de la banda 0, (0,x,y) de una imagen
     * de tipo tBIT, tBYTE o tWORD o valores RGB empaquetados a los pixels
     * de una imagen de tipo tCOLOR.
     * @param bmpInt Valores asignados a los pixels.
     * El rango de los valores depende del tipo de imagen:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535] , tCOLOR ->
     * RGB empaquetado.
     */
    public void setAllPixel(int[] bmpInt) {
        if(type!=JIP.tCOLOR) bmp[0].setAllPixel(bmpInt);
        else {
            bmp[JIP.bRED].setAllPixel(JIP.getAllRedFromRGB(bmpInt));
            bmp[JIP.bGREEN].setAllPixel(JIP.getAllGreenFromRGB(bmpInt));
            bmp[JIP.bBLUE].setAllPixel(JIP.getAllBlueFromRGB(bmpInt));
        }
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna un valor real a un pixel de una banda (b,x,y) de una imagen de tipo
     * tREAL.
     * @param b Banda a la que pertenece el pixel (0 <= b <= numbands-1)
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @param pix Valor asignado al pixel.
     * El rango del valor es [0..1]
     */
    public void setPixel(int b,int x,int y,float pix) {
        if(b>=0&&b<nbands) bmp[b].setPixel(x,y,pix);
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna un valor real a un pixel de la banda 0, (0,x,y) de una imagen
     * de tipo tREAL.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @param pix Valor asignado al pixel.
     * El rango del valor es [0..1]
     */
    public void setPixel(int x,int y,float pix) {
        if(type!=JIP.tCOLOR) bmp[0].setPixel(x,y,pix);
    }

     /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Aade un segmento, cuyas coordenadas se pasan como parmetro, a la
     * imagen. La imagen debe ser de tip tSEGMENT.
     * @param x0 Coordenada x del punto inicial
     * @param y0 Coordenada y del punto inicial
     * @param x1 Coordenada x del punto final
     * @param y1 Coordenada y del punto final
     */

    public void addSegment(int x0,int y0,int x1,int y1)
     {
      if (type!=JIP.tSEGMENT) return;
      int i=0;
      if (x0<0 || y0<0 || x1<0 || y1<0 ||
          x0>width-1 || y0>height-1 ||
          x0>width-1 || y1>height-1) return;
		      bmp[0].addSegment(x0,y0,x1,y1);
     }

			/**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Aade un punto, cuyas coordenadas se pasan como parmetro, a la
     * imagen. La imagen debe ser de tip tPOINT.
     * @param x0 Coordenada x del punto
     * @param y0 Coordenada y del punto
     */

     public void addPoint(int x0, int y0) {
    	if (type!=JIP.tPOINT) return;
			int i=0;
    	if (x0<0 || y0<0 ||
          x0>width-1 || y0>height-1) return;
		      bmp[0].addPoint(x0,y0);
    	
    }
    
     /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna valores reales a los pixels de una banda (b,x,y) de una imagen
     * de tipo tREAL.
     * @param b Banda a la que pertenecen los pixels (0 <= b <= numbands-1)
     * @param bmpFloat Vector de valores asignados a los pixels.
     * El rango de los valores del vector es [0..1]
     */
    public void setAllPixel(int b,float[] bmpFloat) {
        if(b>=0&&b<nbands) bmp[b].setAllPixel(bmpFloat);
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna valores reales a los pixels de la banda 0, (0,x,y) de una imagen
     * de tipo tREAL.
     * @param bmpFloat Valores asignados a los pixels.
     * El rango de los valores del vector es [0..1]
     */
    public void setAllPixel(float[] bmpFloat) {
        if(type!=JIP.tCOLOR) bmp[0].setAllPixel(bmpFloat);
    }

    /****************************************/
    /* Asignacion de valores a los vectores */
    /****************************************/
    
    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna valores a los vectores geomtricos. A partir del vector de entrada, y dependiendo del
     * tipo de imagen, se sustituyen los valores de los vectores actuales por los nuevos.
     * @param v Vector que contiene los elementos geomtricos.
     */

    public void setVector(Vector v) {
    	
				if (type==JIP.tPOINT || type==JIP.tSEGMENT || 
					  type==JIP.tPOLY || type==JIP.tEDGES) {
							bmp[0].setVector(type,v);
		  }
        
    }

    /**************************************/
    /* Obtencion de valores de los pixels */
    /**************************************/

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el valor entero de un pixel de una banda de una imagen de tipo
     * tBIT, tBYTE, tWORD o tCOLOR.
     * @param b Banda a la que pertenece el pixel (0 <= b <= numbands-1)
     * Si la imagen es de tipo tCOLOR, b puede referirse con las constantes
     * bRED, bGREEN o bBLUE.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @return Valor del pixel (b,x,y)
     * El rango depende del tipo de imagen:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535] , tCOLOR -> [0..255]
     */
    public int getPixel(int b,int x,int y) {
        if(b>=0&&b<nbands) return bmp[b].getPixel(x,y);
        else return -1;
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el valor de intensidad RED de un pixel de una imagen
     * en color. Equivale a: getPixel(JIP.bRED,x,y) de una imagen en color.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @return Valor de la intensidad RED del pixel (x,y).
     * El rango de intensidad es [0..255].
     * Devuelve -1 si la imagen no es en color o las coordenadas no son vlidas.
     */
    public int getPixelRed(int x,int y) {
        if(type==JIP.tCOLOR) return getPixel(JIP.bRED,x,y);
        else return -1;
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el valor de intensidad GREEN de un pixel de una imagen
     * en color. Equivale a: getPixel(JIP.bGREEN,x,y) de una imagen en color.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @return Valor de la intensidad GREEN del pixel (x,y).
     * El rango de intensidad es [0..255].
     * Devuelve -1 si la imagen no es en color o las coordenadas no son vlidas.
     */
    public int getPixelGreen(int x,int y) {
        if(type==JIP.tCOLOR) return getPixel(JIP.bGREEN,x,y);
        else return -1;
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el valor de intensidad BLUE de un pixel de una imagen
     * en color. Equivale a: getPixel(JIP.bBLUE,x,y) de una imagen en color.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @return Valor de la intensidad BLUE del pixel (x,y).
     * El rango de intensidad es [0..255].
     * Devuelve -1 si la imagen no es en color o las coordenadas no son vlidas.
     */
    public int getPixelBlue(int x,int y) {
        if(type==JIP.tCOLOR) return getPixel(JIP.bBLUE,x,y);
        else return -1;
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el valor entero de un pixel de la banda 0, (0,x,y) de una imagen
     * de tipo tBIT, tBYTE o tWORD o el valor RGB empaquetado de un pixel (x,y)
     * de una imagen de tipo tCOLOR.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @return Valor asignado al pixel.
     * El rango depende del tipo de imagen:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535] , tCOLOR ->
     * RGB empaquetado.
     */
    public int getPixel(int x,int y) {
        if(type!=JIP.tCOLOR) return bmp[0].getPixel(x,y);
        else {
            int r=bmp[JIP.bRED].getPixel(x,y);
            int g=bmp[JIP.bGREEN].getPixel(x,y);
            int b=bmp[JIP.bBLUE].getPixel(x,y);
            return JIP.getRGB(r,g,b);
        }
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene los valores enteros de los pixels de una banda de una imagen
     * de tipo tBIT, tBYTE, tWORD o tCOLOR.
     * @param b Banda a la que pertenecen los pixels (0 <= b <= numbands-1)
     * Si la imagen es de tipo tCOLOR, b puede referirse con las constantes
     * bRED, bGREEN o bBLUE.
     * @return Vector de valores asignados a los pixels.
     * El rango de los valores del vector depende del tipo de imagen:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535] , tCOLOR -> [0..255]
     */
    public int[] getAllPixel(int b) {
        if(b>=0&&b<nbands) return bmp[b].getAllPixel();
        else return null;
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene los valores de intensidad RED de los pixels de una imagen
     * en color. Equivale a: getAllPixel(JIP.bRED) de una imagen en color.
     * @return Vector de valores de la intensidad RED de los pixels.
     * El rango de intensidad es [0..255]
     */
    public int[] getAllPixelRed() {
        if(type==JIP.tCOLOR) return getAllPixel(JIP.bRED);
        else return null;
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene los valores de intensidad GREEN de los pixels de una imagen
     * en color. Equivale a: getAllPixel(JIP.bGREEN) de una imagen en color.
     * @return Vector de valores de la intensidad GREEN de los pixels.
     * El rango de intensidad es [0..255]
     */
    public int[] getAllPixelGreen() {
        if(type==JIP.tCOLOR) return getAllPixel(JIP.bGREEN);
        else return null;
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene los valores de intensidad BLUE de los pixels de una imagen
     * en color. Equivale a: getAllPixel(JIP.bBLUE) de una imagen en color.
     * @return Vector de valores de la intensidad BLUE de los pixels.
     * El rango de intensidad es [0..255]
     */
    public int[] getAllPixelBlue() {
        if(type==JIP.tCOLOR) return getAllPixel(JIP.bBLUE);
        else return null;
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene los valores enteros de los pixels de la banda 0 de una imagen
     * de tipo tBIT, tBYTE o tWORD o los valores RGB empaquetados de los pixels
     * de una imagen de tipo tCOLOR.
     * @return Valores de los pixels.
     * El rango de los valores depende del tipo de imagen:
     * tBIT -> [0..1] , tBYTE -> [0..255] , tWORD -> [0..65535] , tCOLOR ->
     * RGB empaquetado.
     */
    public int[] getAllPixel() {
        if(type!=JIP.tCOLOR) return bmp[0].getAllPixel();
        else {
            int[] r=bmp[JIP.bRED].getAllPixel();
            int[] g=bmp[JIP.bGREEN].getAllPixel();
            int[] b=bmp[JIP.bBLUE].getAllPixel();
            return JIP.getAllRGB(r,g,b);
        }
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el valor real de un pixel de una banda (b,x,y) de una imagen de tipo
     * tREAL.
     * @param b Banda a la que pertenece el pixel (0 <= b <= numbands-1)
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @return Valor real del pixel.
     * El rango del valor es [0..1]
     */
    public float getPixelFlo(int b,int x,int y) {
        if(b>=0&&b<nbands) return bmp[b].getPixelFlo(x,y);
        else return -1;
    }
		
    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el valor real de un pixel de la banda 0, (0,x,y) de una imagen
     * de tipo tREAL.
     * @param x Coordenada x del pixel (0 <= x <= anchura-1)
     * @param y Coordenada y del pixel (0 <= y <= altura-1)
     * @return Valor real del pixel.
     * El rango del valor es [0..1]
     */
    public float getPixelFlo(int x,int y) {
        if(type!=JIP.tCOLOR) return bmp[0].getPixelFlo(x,y);
        else return -1;
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene los valores reales de los pixels de una banda de una imagen
     * de tipo tREAL.
     * @param b Banda a la que pertenecen los pixels (0 <= b <= numbands-1)
     * @return Vector de valores reales de los pixels de la banda b.
     * El rango de los valores del vector es [0..1]
     */
    public float[] getAllPixelFlo(int b) {
        if(b>=0&&b<nbands) return bmp[b].getAllPixelFlo();
        else return null;
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene los valores reales de los pixels de la banda 0 de una imagen
     * de tipo tREAL.
     * @return Vector de valores reales de los pixels.
     * El rango de los valores del vector es [0..1]
     */
    public float[] getAllPixelFlo() {
        if(type!=JIP.tCOLOR) return bmp[0].getAllPixelFlo();
        else return null;
    }

		/************************************************/
    /* Obtencion de Valores de Vectores Geometricos */
    /************************************************/

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene los valores geometricos del vector, si la imagen es de
     * tipo geomtrico.
     * @return Vector de valores geometricos de la imagen
     */

		 public Vector getVector() {
	      Vector aux=new Vector();	 	
		 		switch (type) {
		 				case JIP.tPOINT: 	aux=bmp[0].getVectorPoint();
		 				                  break;
		 				case JIP.tSEGMENT:aux=bmp[0].getVectorSegment();
		 													break;
		 				case JIP.tPOLY:   aux=bmp[0].getVectorPoly();
		 													break;
		 				case JIP.tEDGES:  aux=bmp[0].getVectorEdges();
		 													break;
		 		}
		 	
		 	  return aux;
		 }


   	/************************************************/
    /* Obtencion de Valores de la longitud actual de los vectores */
    /************************************************/

		 /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el tamao de elementos geomtricos de la imagen, si la imagen es de
     * tipo geomtrico.
     * @return Numero de elementos geomtricos de la imagen
     */

		 public int getLVector() {
       int aux=-1;
		 		switch (type) {
		 				case JIP.tPOINT: 	aux=bmp[0].getLVectorPoint();
		 				                  break;
		 				case JIP.tSEGMENT:aux=bmp[0].getLVectorSegment();
		 													break;
		 				case JIP.tPOLY:   aux=bmp[0].getLVectorPoly();
		 													break;
		 				case JIP.tEDGES:  aux=bmp[0].getLVectorEdges();
		 													break;
		 		}
		 	
		 	  return aux;
		 }


    /***************************************/
    /* Manipulacin de bandas de la imagen */
    /***************************************/

     /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Obtiene el bitmap que representa una banda de la imagen.
     * @param b Banda a obtener (0 <= b <= numbands-1)
     * Si la imagen es de tipo tCOLOR, b puede referirse con las constantes
     * bRED, bGREEN o bBLUE. El bitmap de una de las 3 bandas de una imagen
     * tipo tCOLOR es siempre de tipo tBYTE.
     * @return Bitmap que corresponde a la banda b.
     */
    public JIPBitmap getBand(int b) {
    	if(b>=0&&b<nbands) return bmp[b];
    	else return null;
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Asigna un nuevo bitmap a una banda de la imagen.
     * @param b Banda a asignar (0 <= b <= numbands-1)
     * Si la imagen es de tipo tCOLOR, b puede referirse con las constantes
     * bRED, bGREEN o bBLUE.
     * @param newbmp Bitmap que asignamos a la banda b de la imagen. Debe tener
     * el mismo tamao que la imagen y ser del mismo tipo que la imagen si sta
     * no es tCOLOR. En caso de que la imagen sea tCOLOR el bitmap debe ser
     * de tipo tBYTE.
     */
    public void setBand(int b,JIPBitmap newbmp) {
    	if(newbmp!=null && width==newbmp.width && height==newbmp.height &&
    	   b>=0 && b<nbands )
    	{
    		if((type==JIP.tCOLOR && newbmp.type==JIP.tBYTE) ||
    		   (type!=JIP.tCOLOR && type==newbmp.type))
    		{
    			bmp[b]=new JIPBitmap(newbmp);
    		}
    	}    	
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Aade una banda a una imagen. La imagen no puede ser de tipo tCOLOR.
     * @param newbmp Bitmap que aadimos como banda a la imagen. Debe tener
     * el mismo tamao que la imagen y ser del mismo tipo.
     */
    public void appendBand(JIPBitmap newbmp) {
    	if(newbmp!=null && type!=JIP.tCOLOR && type==newbmp.type &&
    	   width==newbmp.width && height==newbmp.height) {
    	   	nbands++;
	        JIPBitmap[] auxbmp=new JIPBitmap[nbands];
        	for(int i=0;i<nbands-1;i++) auxbmp[i]=bmp[i];
        	auxbmp[nbands-1]=new JIPBitmap(newbmp);
        	bmp=auxbmp;    	   	
    	}
    }

    /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
 	   *<FONT COLOR="blue">
     * Elimina una banda de la imagen. La imagen no puede ser de tipo tCOLOR
     * y debe tener ms de una banda (pues no puede quedar vaca)
     * @param b Banda a eliminar (0 <= b <= numbands-1) siendo (numbands > 1)
     */
    public void removeBand(int b) {
    	if(nbands>1&&type!=JIP.tCOLOR&&b>=0&&b<nbands) {
    		int i,j;
	        JIPBitmap[] auxbmp=new JIPBitmap[nbands-1];
        	for(i=0,j=0;i<nbands;i++) {
        		if(i!=b) {
        			auxbmp[j]=bmp[i];
        			j++;
        		}
        	}
    	   	nbands--;
        	bmp=auxbmp;    	   	
    	}
    }

}