package jip;

 /**<P><FONT COLOR="red">
 *<B>Descripcin:</B><BR>
 *<FONT COLOR="blue">
 *     Clase que define el objeto parmetro. Un parmetro contiene informacin
 * adicional que ser utilizada por una funcin y que condicionar la forma en
 * que la funcin procesa la imagen de entrada para obtener la imagen de salida.
 * Un parmetro puede ser de 5 tipos: pBOOL, pINT, pREAL, pSTRING y pIMAGE. Estos
 * tipos se definen en la clase JIP. Adems un parmetro puede ser requerido o no.
 * En caso de no ser requerido el parmetro podr tener un valor asignado por
 * defecto. El constructor bsico de un parmetro debe especificar su nombre,
 * tipo y si es o no requerido. Estas caractersticas no variarn durante la
 * vida del objeto. Tambin podremos asignar a un parmetro una descripcin
 * que indicar su funcionamiento. Podremos acceder al valor del parmetro o
 * asignarlo. Si un parmetro no tiene valor asignado y no es requerido, al
 * acceder a su valor obtendremos el valor que tenga asignado por defecto.
 * @see JIP#pBOOL
 * @see JIP#pINT
 * @see JIP#pREAL
 * @see JIP#pSTRING
 * @see JIP#pIMAGE
 */
public class JIPParameter {

    /**Nombre del Parmetro*/
    String name;
    
    /** Tipo del Parmetro */
    int type;
    
    /** Indica si el parmetro es requerido */
    boolean required;
    
    /** Indica si el parmetro est asignado */
    boolean asigned;
    
    /** Descripccin del parmetro */
    String description=null;   

    //Valores del Parametro
		
		/** Indica si el parmetro es Booleano*/
    boolean valbool=false;
    
    /** Indica si el parmetro es Entero */
    int valint=0;
    
    /** Indica si el parmetro es Real */
    float valreal=0;
    
    /** Indica si el parmetro es una Cadena de Texto*/
    String valstr=null;
    
    /** Indica si el parmetro es una imagen */
    JIPImage valimg=null;
    
    //Valores por defecto del parametro
    
    /** Valor por defecto Booleano */
    boolean defvalbool=false;
    
    /** Valor por defecto Entero */
    int defvalint=0;
    
    /** Valor por defecto Real */
    float defvalreal=0;
    
    /** Valor por defecto de Cadena de Texto*/
    String defvalstr=null;
    
    /** Valor por defecto de Imagen */
    JIPImage defvalimg=null;
    
    //******************//
    //  Constructores   //
    //******************//

     /**<P><FONT COLOR="red">
		 * <B>Descripcin:</B><BR>
 		 * <FONT COLOR="blue">
     * Constructor de un parmetro.
     * @param n Nombre del parmetro.
     * @param t Tipo de parmetro.
     * @param req Flag que indica si es requerido (true) o no (false).
     */
    public JIPParameter(String n,int t,boolean req) {
        name=n;
        type=t;
        required=req;
        asigned=false;
    }

    //*******************//
    //     METODOS       //
    //*******************//

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el nombre del parmetro.
     * @return Nombre del parmetro.
     */
    public String getName() { return name; }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el tipo del parmetro.
     * @return Tipo del parmetro.
	 	 * @see JIP#pBOOL
	   * @see JIP#pINT
	   * @see JIP#pREAL
	   * @see JIP#pSTRING
	   * @see JIP#pIMAGE
     */
    
    public int getType()    { return type; }

     /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Nos dice si el parmetro es o no requerido.
     * @return true si el parmetro es requerido, false en caso contrario.
     */
    public boolean isRequired() { return required; }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Nos dice si el valor del parmetro est ya asignado o no.
     * @return true si el valor del parmetro est asignado,
     * false en caso contrario.
     */
    public boolean isAsigned() { return asigned; }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna una descripcin al parmetro.
     * @param d Descripcin a asignar al parmetro.
     */
    public void setDescription(String d) { description=d; }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene la descripcin asignada al parmetro.
     * @return Descripcin asignada al parmetro.
     */
    public String getDescription() { return description; }

    // ********************************************
    // Asignacion y obtencion del valor por defecto
    // ********************************************
    
    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna al parmetro un valor por defecto booleano.
     * El parmetro no debe ser requerido y debe ser de tipo pBOOL.
     * @param d Valor a asignar como valor por defecto.
     */
    public void setDefault(boolean d) {
      if((!required)&&(type==JIP.pBOOL)) defvalbool=d;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna al parmetro un valor por defecto entero.
     * El parmetro no debe ser requerido y debe ser de tipo pINT.
     * @param d Valor a asignar como valor por defecto.
     */
    public void setDefault(int d) {
      if((!required)&&(type==JIP.pINT)) defvalint=d;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna al parmetro un valor por defecto real.
     * El parmetro no debe ser requerido y debe ser de tipo pREAL.
     * @param d Valor a asignar como valor por defecto.
     */
    public void setDefault(float d) {
      if((!required)&&(type==JIP.pREAL)) defvalreal=d;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna al parmetro un valor por defecto string.
     * El parmetro no debe ser requerido y debe ser de tipo pSTRING.
     * @param d Valor a asignar como valor por defecto.
     */
    public void setDefault(String d) {
      if((!required)&&(type==JIP.pSTRING)) defvalstr=d;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna al parmetro un valor por defecto imagen.
     * El parmetro no debe ser requerido y debe ser de tipo pIMAGE.
     * @param img Imagen a asignar como valor por defecto.
     */
    public void setDefault(JIPImage img) {
      if((!required)&&(type==JIP.pIMAGE)) defvalimg=new JIPImage(img);
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor por defecto booleano asignado al parmetro.
     * El parmetro no debe ser requerido y debe ser de tipo pBOOL.
     * @return Valor por defecto asignado al parmetro.
     */
    public boolean getDefaultBool() {
      if((!required)&&(type==JIP.pBOOL)) return defvalbool;
      else return false;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor por defecto entero asignado al parmetro.
     * El parmetro no debe ser requerido y debe ser de tipo pINT.
     * @return Valor por defecto asignado al parmetro.
     */
    public int getDefaultInt() {
      if((!required)&&(type==JIP.pINT)) return defvalint;
      else return 0;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor por defecto real asignado al parmetro.
     * El parmetro no debe ser requerido y debe ser de tipo pREAL.
     * @return Valor por defecto asignado al parmetro.
     */
    public float getDefaultReal() {
      if((!required)&&(type==JIP.pREAL)) return defvalreal;
      else return 0;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor por defecto string asignado al parmetro.
     * El parmetro no debe ser requerido y debe ser de tipo pSTRING.
     * @return Valor por defecto asignado al parmetro.
     */
    public String getDefaultStr() {
      if((!required)&&(type==JIP.pSTRING)) return defvalstr;
      else return null;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor por defecto imagen asignado al parmetro.
     * El parmetro no debe ser requerido y debe ser de tipo pIMAGE.
     * @return Valor por defecto asignado al parmetro.
     */
    public JIPImage getDefaultImg() {
      if((!required)&&(type==JIP.pIMAGE)) return defvalimg;
      else return null;
    }

    // **********************************************
    // Asignacion y obtencion del valor del parametro
    // **********************************************
    
    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna al parmetro un valor booleano.
     * El parmetro debe ser de tipo pBOOL.
     * @param v Valor a asignar. Tras llamar a este mtodo, el mtodo isAsigned()
     * devolver true.
     */
    public void setValue(boolean v) {
      if(type==JIP.pBOOL) { valbool=v; asigned=true; }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna al parmetro un valor entero.
     * El parmetro debe ser de tipo pINT.
     * @param v Valor a asignar. Tras llamar a este mtodo, el mtodo isAsigned()
     * devolver true.
     */
    public void setValue(int v) {
      if(type==JIP.pINT) { valint=v; asigned=true; }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna al parmetro un valor real.
     * El parmetro debe ser de tipo pREAL.
     * @param v Valor a asignar. Tras llamar a este mtodo, el mtodo isAsigned()
     * devolver true.
     */
    public void setValue(float v) {
      if(type==JIP.pREAL) { valreal=v; asigned=true; }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna al parmetro un valor string.
     * El parmetro debe ser de tipo pSTRING.
     * @param v Valor a asignar. Tras llamar a este mtodo, el mtodo isAsigned()
     * devolver true.
     */
    public void setValue(String v) {
      if(type==JIP.pSTRING) { valstr=v; asigned=true; }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna al parmetro un valor imagen.
     * El parmetro debe ser de tipo pIMAGE.
     * @param img Valor a asignar. Tras llamar a este mtodo, el mtodo isAsigned()
     * devolver true.
     */
    public void setValue(JIPImage img) {
      if(type==JIP.pIMAGE) { valimg=new JIPImage(img); asigned=true; }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor booleano asignado al parmetro.
     * El parmetro debe ser de tipo pBOOL.
     * @return Valor asignado al parmetro. Si no est asignado y el parmetro
     * no es requerido devolver el valor asignado por defecto.
     */
    public boolean getValueBool() {
      if(type==JIP.pBOOL) {
          if(asigned) return valbool;
		      else if(!required) return defvalbool;
		      else return false;
      }
      else return false;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor entero asignado al parmetro.
     * El parmetro debe ser de tipo pINT.
     * @return Valor asignado al parmetro. Si no est asignado y el parmetro
     * no es requerido devolver el valor asignado por defecto.
     */
    public int getValueInt() {
      if(type==JIP.pINT) {
          if(asigned) return valint;
    		  else if(!required) return defvalint;
		      else return 0;
      }
      else return 0;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor real asignado al parmetro.
     * El parmetro debe ser de tipo pREAL.
     * @return Valor asignado al parmetro. Si no est asignado y el parmetro
     * no es requerido devolver el valor asignado por defecto.
     */
    public float getValueReal() {
      if(type==JIP.pREAL) {
          if(asigned) return valreal;
     		  else if(!required) return defvalreal;
		      else return 0;
      }
      else return 0;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor string asignado al parmetro.
     * El parmetro debe ser de tipo pSTRING.
     * @return Valor asignado al parmetro. Si no est asignado y el parmetro
     * no es requerido devolver el valor asignado por defecto.
     */
    public String getValueStr() {
      if(type==JIP.pSTRING) {
          if(asigned) return valstr;
     		  else if(!required) return defvalstr;
		      else return null;
      }
      else return null;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el valor imagen asignado al parmetro.
     * El parmetro debe ser de tipo pIMAGE.
     * @return Valor asignado al parmetro. Si no est asignado y el parmetro
     * no es requerido devolver el valor asignado por defecto.
     */
    public JIPImage getValueImg() {
      if(type==JIP.pIMAGE) {
          if(asigned) return valimg;
    		  else if(!required) return defvalimg;
		      else return null;
      }
      else return null;
    }

}