package jip;

import java.util.Vector;

/**<P><FONT COLOR="red">
 *<B>Descripcin:</B><BR>
 *<FONT COLOR="blue">
 *     Clase que define el objeto secuencia. Una secuencia representa una lista
 * ordenada de frames cada uno de los cuales es un objeto imagen. Los tipos de
 * imgenes que corresponden a los frames no tienen por qu coincidir as como
 * tampoco las dimensiones de las imgenes. El nmero de frames de una secuencia
 * puede variar durante la vida del objeto e incluso puede ser 0 (secuencia
 * vaca). El acceso a las imgenes de una secuencia se realiza con un ndice
 * entero en el rango [0..numframes-1]. Podremos aadir, insertar y eliminar
 * frames de una secuencia as como aadir otras secuencias al final. Tambin
 * podremos obtener o reasignar la imagen de un frame de la secuencia.
 * El objeto secuencia es el que siempre se almacena en un fichero con formato
 * JIP (salvo la secuencia vaca que no se almacena).
 * De este modo, para almacenar una nica imagen en fichero tendremos que crear
 * una secuencia con un solo frame que la contenga.
 * Los mtodos para salvar y cargar secuencias se encuentran en la clase
 * JIPToolkit y no en la propia clase secuencia. El motivo de separarlos es que
 * el formato del fichero no es inherente a la estructura de la secuencia y de
 * esta forma podemos cambiarlo sin tener que tocar esta clase. Por ltimo, a una
 * secuencia podemos darle un nombre que por defecto ser "[Unnamed]".
 * @see JIPToolkit#getSeqFromFile
 * @see JIPToolkit#saveSeqIntoFile
 */
public class JIPSequence {

    /** Nmero de Frames que componen la Secuencia */
    int nframes;
    /** Nombre de la secuencia */
    String name;
    /** Vector que contiene los frames de la secuencia */
    Vector frames;
    
		/**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
 		 * Funcion de Representacin
 		 * @return Cadena de texto con la descripcin de la secuencia.
     */
    
    public String toString (){
        String seq=new String();
      
      seq="Secuencia ["+name+"]/nNFrames: "+nframes;
      for (int i=0;i<nframes;i++){
        JIPImage imga=(JIPImage)frames.elementAt(i);
        seq=seq+"\nFrame ["+i+"]\n\tNBandas: "+imga.getNumBands()+
          "\n\tNPixels: "+imga.getNumPixels()+"\n\tWidth: "+imga.getWidth()+
          "\n\tHeight: "+imga.getHeight()+"\n\tType: "+imga.getType();
      }
      return(seq);
    }
    

    //******************//
    //  Constructores   //
    //******************//

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Constructor de una secuencia vaca (sin ningn frame).
     */
     
    public JIPSequence() {
        nframes=0;
        name="[Unnamed]";
        frames=new Vector();
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Constructor de una secuencia con un solo frame.
     * @param img Imagen correspondiente al frame de la secuencia.
     */
    public JIPSequence(JIPImage img) {
        nframes=0;
        name="[Unnamed]";
        frames=new Vector();
        if(img!=null) frames.addElement(img);
        nframes=frames.size();
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Constructor de una secuencia como copia de otra.
     * @param seq Secuencia de referencia para construir la copia.
     */
    public JIPSequence(JIPSequence seq) {
        if(seq!=null) {
        	nframes=seq.nframes;
	        name=new String(seq.name);
	    }
        else {
        	nframes=0;
	        name="[Unnamed]";
	    }
        frames=new Vector();
        if(nframes>0) {
            for(int i=0;i<nframes;i++)
                frames.addElement(new JIPImage(seq.getFrame(i)));
            nframes=frames.size();
        }
    }

    //*******************//
    //     METODOS       //
    //*******************//
    
    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el nmero de frames de la secuencia.
     * @return Nmero de frames.
     */
    public int getNumFrames() { return nframes; }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene el nombre de la secuencia.
     * @return Nombre de la secuencia.
     */
    public String getName() { return name; }


    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna el nombre de la secuencia.
     * @param nom Nombre de la secuencia.
     */
    public void setName(String nom) {
        if(nom==null||nom.length()==0) name="[Unnamed]";
        else name=new String(nom);
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene la imagen correspondiente a un frame de la secuencia.
     * @param n Nmero de frame (0 <= n <= numframes-1)
     * @return Imagen correspondiente al frame n.
     * null si n no corresponde a un frame de la secuencia.
     */
    public JIPImage getFrame(int n) {
        if(n>=0&&n<nframes)
            return (JIPImage)frames.elementAt(n);
        else
            return null;
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Asigna una imagen a un frame de la secuencia.
     * @param img Imagen que ser asignada al frame.
     * @param n Nmero de frame al que se asignar la imagen.
     * n indica el ndice del frame a asignar, por tanto, debe corresponder a
     * un ndice de un frame ya existente siendo: 0 <= n <= numframes-1.
     */
    public void setFrame(JIPImage img,int n) {
    	if(img!=null&&n>=0&&n<nframes) {
	        frames.setElementAt(img,n);
	        nframes=frames.size();
	    }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Aade un frame al final de la secuencia.
     * @param img Imagen correspondiente al frame aadido.
     */
    public void addFrame(JIPImage img) {
    	if(img!=null) {
	        frames.addElement(img);
	        nframes=frames.size();
	    }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Inserta un frame en la secuencia.
     * @param img Imagen correspondiente al frame insertado.
     * @param n Posicin correspondiente al frame insertado.
     * n indica el ndice del frame a insertar, por tanto, debe corresponder a
     * un ndice de un frame ya existente siendo: 0 <= n <= numframes-1.
     * Aquellos frames cuyo ndice era igual o mayor que n vern incrementado su
     * ndice en una unidad.
     */
    public void insertFrame(JIPImage img,int n) {
    	if(img!=null&&n>=0&&n<nframes) {
	        frames.insertElementAt(img,n);
	        nframes=frames.size();
	    }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Elimina un frame de la secuencia.
     * @param n Numero de frame (0 <= n <= numframes-1)
     */
    public void removeFrame(int n) {
        if(n>=0&&n<nframes) {
            frames.removeElementAt(n);
            nframes=frames.size();
        }
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Elimina todos los frames de la secuencia (quedar vaca).
     */

    public void removeAllFrames() {
        frames.removeAllElements();
        nframes=frames.size();
    }

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Aade una secuencia al final de la secuencia.
     * @param seq Secuencia a aadir.
     */
    public void appendSequence(JIPSequence seq) {
    	if(seq!=null) {
	        int nf=seq.nframes;
	        if(nf>0) {
	            for(int i=0;i<nf;i++)
	                frames.addElement(seq.getFrame(i));
	            nframes=frames.size();
	        }
	    }
    }
}