package jip;

import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.util.zip.*;
import java.util.Vector;



/**<P><FONT COLOR="red">
 *<B>Descripcin:</B><BR>
 *<FONT COLOR="blue">
 * Clase que define mtodos bsicos relacionados con la obtencin y almacenamiento
 * de imgenes AWT y JIP. Las imgenes AWT las podremos obtener a partir de imgenes
 * JIP o directamente de ficheros GIF o JPEG. Las imgenes JIP las podremos obtener a
 * partir de imgenes AWT o como secuencias directamente de ficheros JIP. Podremos
 * tambin almacenar imgenes y secuencias JIP tanto en ficheros JIP como en
 * ficheros con formato TGA (Targa).
 */
public final class JIPToolkit {

    static final Toolkit toolkit=Toolkit.getDefaultToolkit();
    
 /**<P><FONT COLOR="red">
  *<B>Descripcin:</B><BR>
  *<FONT COLOR="blue">
  *   Constructor Vacio de la clase 
  */

	JIPToolkit() {}
	
    /***********************************************/
    /* Metodos para la obtencion de una Imagen AWT */
    /***********************************************/

    /**<P><FONT COLOR="red">
		 *<B>Descripcin:</B><BR>
 		 *<FONT COLOR="blue">
     * Obtiene un objeto imagen AWT a partir de una banda de una imagen JIP.
     * @param img Imagen JIP fuente.
     * @param b Banda de img cuyo bitmap ser convertido a imagen AWT
     * (0 <= b <= numbandas-1)
     * @return Imagen AWT resultado. Aunque las imgenes AWT siempre se representan
     * en color, como una banda no contiene color, los colores de la imagen AWT
     * obtenida sern siempre grises.
     */
    public static Image getAWTImage(JIPImage img,int b) {
        if(img==null) return null;
        Image res=null;
        if(b>=0&&b<img.nbands) {
            int npixels=img.width*img.height;
            int[] pix=new int[npixels];
            switch(img.type) {
            case JIP.tBIT:
                pix=JIP.getAllByteFromBit(
                			JIP.getAllIntFromBit(img.bmp[b].bmpBit)
                		);
                break;
            case JIP.tBYTE:
            case JIP.tCOLOR:
                pix=JIP.getAllIntFromByte(img.bmp[b].bmpByte);
                break;
            case JIP.tWORD:
                pix=JIP.getAllByteFromWord(
                			JIP.getAllIntFromWord(img.bmp[b].bmpWord)
                        );
                break;
            case JIP.tREAL:
                pix=JIP.getAllByteFromReal(img.bmp[b].bmpReal);
                break;
                
            case JIP.tPOLY:
              for (int i=0;i<npixels;i++) pix[i]=0;
              break;
            case JIP.tPOINT:
              for (int i=0;i<npixels;i++) pix[i]=0;
              break;
            case JIP.tEDGES:
              for (int i=0;i<npixels;i++) pix[i]=0;
              break;
            case JIP.tSEGMENT:

                for (int i=0;i<npixels;i++) pix[i]=0;
                break;
                
            }
            for(int i=0;i<npixels;i++) {
                int val=pix[i]&0xFF;
                pix[i]=(0xFF<<24)|(val<<16)|(val<<8)|val;
            }
            
            res=toolkit.createImage(
                new MemoryImageSource(img.width,img.height,pix,0,img.width)
                );
        }

        return res;
    }

		 /**<P><FONT COLOR="red">
  	 *<B>Descripcin:</B><BR>
  	 *<FONT COLOR="blue">
     * Obtiene un objeto imagen AWT a partir de una imagen JIP.
     * Si la imagen JIP no es del tipo tCOLOR se obtendr la imagen de la banda 0.
     * Si la imagen JIP es en color la imagen obtenida ser tambin en color.
     * @param img Imagen JIP fuente.
     * @return Imagen AWT resultado.
     * Si la imagen JIP no es en color se obtendr la imagen de la banda 0. Por
     * tanto, aunque la imagen AWT sea en color, contendr una escala de grises.
     * Si la imagen JIP es en color la imagen obtenida se corresponder con ella.
     * @see JIPToolkit#getAWTImage(jip.JIPImage, int)
     */
    public static Image getAWTImage(JIPImage img) {
        if(img.type!=JIP.tCOLOR) 
            return getAWTImage(img,0);
        else {
            int[] pix=img.getAllPixel();
            Image res=toolkit.createImage(
                new MemoryImageSource(img.width,img.height,pix,0,img.width)
                );
            return res;
        }
    }

     /**<P><FONT COLOR="red">
     *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Obtiene un objeto imagen AWT a partir de un fichero GIF o JPEG.
     * Este mtodo es bloqueante y no devuelve el control hasta que la imagen
     * resultado est completamente formada. Conviene utilizarlo frente a los
     * mtodos de carga de imgenes no bloqueantes de AWT como getImage() para
     * evitar la utilizacin de un protocolo asncrono basado en la interfaz
     * ImageObserver de AWT que asegure que accedemos a los valores correctos
     * de los pixels de la imagen. Tambin nos desentendemos de la utilizacin
     * del objeto MediaTracker para esperar la carga de la imagen pues este mtodo
     * ya utiliza MediaTracker internamente.
     * @param file Fichero fuente (debe ser GIF o JPEG)
     * @return Imagen AWT resultado. (null si no puede cargar la imagen)
     */
    public static Image getAWTImage(String file) {
    	Image res=toolkit.getImage(file);
    	if(res==null) return null;
    	MediaTracker mt=new MediaTracker(new Canvas());
    	mt.addImage(res,0);
    	try {
    		mt.waitForID(0);
    	} catch (InterruptedException e) { return null; }
    	if(mt.isErrorID(0)) return null;
		else return res;
    }

    /***********************************************/
    /* Metodos para la obtencion de una Imagen JIP */
    /***********************************************/

    /**<P><FONT COLOR="red">
     *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Obtiene un objeto imagen JIP de tipo tCOLOR a partir de una imagen AWT.
     * @param img Imagen AWT fuente (debe estar cargada completamente lo cual se
     * asegura con getAWTImage(String fich).
     * @see JIPToolkit#getAWTImage(java.lang.String)
     * @return Imagen JIP resultado (tipo tCOLOR).
     */
    public static JIPImage getColorImage(Image img) {
        Canvas canv=new Canvas();
        int w=img.getWidth(canv);
        int h=img.getHeight(canv);
        if(w<=0||h<=0) return null;
        int[] pix=new int[w*h];
        PixelGrabber pg=new PixelGrabber(img,0,0,w,h,pix,0,w);
        try {
            pg.grabPixels();
        }
        catch(InterruptedException e) {return null;}

        return new JIPImage(w,h,JIP.tCOLOR,pix);
    }

    /**************************************/
    /* Metodos para salvar una imagen JIP */
    /**************************************/

    /**<P><FONT COLOR="red">
     *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Salva un objeto imagen JIP en un fichero con formato JIP.
     * La imagen se salvar como una secuencia con un nico frame que la contenga.
     * @param img Imagen a salvar.
     * @param file Fichero en el cual almacenar la imagen (si no existe se crea,
     * si existe se sobreescribe)
     */
    public static void saveImageIntoFile(JIPImage img,String file) {
		saveSeqIntoFile(new JIPSequence(img),file);
    }

    /**<P><FONT COLOR="red">
     *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Salva un objeto imagen JIP en un fichero con formato TGA (Targa).
     * Si la imagen dispone de varias bandas de grises, se generar un
     * fichero para cada una de ellas, aadindose al nombre (antes de la extensin)
     * el postfijo "_b0", "_b1", ...  segn la banda a la que corresponda.
     * @param img Imagen a salvar.
     * @param file Fichero en el cual almacenar la imagen (si no existe se crea,
     * si existe se sobreescribe)
     */
    public static void saveImageIntoTga(JIPImage img,String file) {
    	
        //Streams para escribir en el fichero de salida
        FileOutputStream fs=null;
        BufferedOutputStream bfs=null;
        DataOutputStream dat=null;
    	
    	int numfiles=1;
    	boolean color=(img.type==JIP.tCOLOR);
    	if(!color&&img.nbands>1) numfiles=img.nbands;

    	//Obtiene nombre del fichero y extension
    	String name=null; String ext=null;
    	if(file.lastIndexOf(".")==-1) {
    		name=file;
    		ext=".tga";
    	}
    	else {
    		name=file.substring(0,file.lastIndexOf("."));
    		ext=file.substring(file.lastIndexOf("."));
    	}
    	    	
        int w=img.width;
        int h=img.height;
        int pix=0;

        // Generamos cada uno de los ficheros necesarios
        for(int nf=0;nf<numfiles;nf++) {
	        try {
	        	if(numfiles>1) fs=new FileOutputStream(name+"_b"+nf+ext);
	        	else fs=new FileOutputStream(name+ext);
		        bfs=new BufferedOutputStream(fs,4096);
		        dat=new DataOutputStream(bfs);

				//Escribe la cabecera
				for(int i=0;i<12;i++) dat.writeByte((i==2)?2:0);
				dat.writeByte(w&0xff);
				dat.writeByte((w>>8)&0xff);
				dat.writeByte(h&0xff);
				dat.writeByte((h>>8)&0xff);
				dat.writeByte(24);
				dat.writeByte(0x20);
		
				//Escribe los datos
				for(int i=0;i<h;i++)
					for(int j=0;j<w;j++) {
						if(color) {
							dat.writeByte(img.getPixel(JIP.bBLUE,j,i)&0xff);
							dat.writeByte(img.getPixel(JIP.bGREEN,j,i)&0xff);
							dat.writeByte(img.getPixel(JIP.bRED,j,i)&0xff);
						}
						else {
				            switch(img.type) {
				            case JIP.tBIT:
				                pix=JIP.getByteFromBit(img.getPixel(nf,j,i));
			                break;
				            case JIP.tBYTE:
				                pix=img.getPixel(nf,j,i);
			                break;
				            case JIP.tWORD:
				                pix=JIP.getByteFromWord(img.getPixel(nf,j,i));
			                break;
				            case JIP.tREAL:
				                pix=JIP.getByteFromReal(img.getPixelFlo(nf,j,i));
			                break;
			            	}
							
							pix&=0xff;
							dat.writeByte(pix);
							dat.writeByte(pix);
							dat.writeByte(pix);
						}
					}
			
		       //Vuelca los datos
		       dat.flush();
		       fs.close();
	        } catch(Exception e) {System.err.println(e);}
	
    	}    	    	
    }

    /******************************************/
    /* Metodo para la carga de una Secuencia  */
    /******************************************/

    /**<P><FONT COLOR="red">
     *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Obtiene un objeto secuencia JIP a partir de un fichero JIP.
     * @param file Nombre del fichero.
     * @return Secuencia JIP. (null si no puede cargar la secuencia)
     */
    public static JIPSequence getSeqFromFile(String file) {
        JIPSequence res=new JIPSequence();
        JIPImage[] imgs=null;
        int[] imgsizes=null;
        byte[] raw;

        //Streams para leer del fichero
        FileInputStream fs=null;
        BufferedInputStream bfs=null;
        DataInputStream dat=null;
        BufferedReader cab=null;
        
        String line;
        int index,index1,i,j,nframes,sz;
        int w,h,b,t;
        int auxmio;
        Vector tamPoly=new Vector(),tamEdges=new Vector();
        Vector arbPoli = new Vector();
        Vector arbEdge = new Vector();
        String name,tip;

        try {

            /* LECTURA DE LA CABECERA */
            
            //Abre el stream para leer la cabecera
            cab=new BufferedReader(new FileReader(file));

            cab.readLine(); //Linea 1 "JIP\n"
            cab.readLine(); //Linea 2 "\n"

            //Linea 3 "Sequence:<name>\n"
            line=cab.readLine();
            res.name=line.substring(9).trim();

            //Linea 4 "Frames:<nframes>\n"
            line=cab.readLine();
            nframes=Integer.parseInt(line.substring(7).trim());
            
            line=cab.readLine(); //Linea 5 "\n"

            //Crea el vector de imagenes
            imgs=new JIPImage[nframes];
            imgsizes=new int[nframes];

            //Para cada frame de la cabecera
            for(i=0;i<nframes;i++) {
            
                //Nombre de la imagen
                line=cab.readLine();
                index=line.indexOf(":")+1;
                name=line.substring(index).trim();

                //Anchura y altura de la imagen
                line=cab.readLine();
                index=line.indexOf(":")+1;
                index1=line.lastIndexOf("x");
                w=Integer.parseInt(line.substring(index,index1).trim());
                h=Integer.parseInt(line.substring(index1+1).trim());

                //Tipo de la imagen
                line=cab.readLine();
                index=line.indexOf(":")+1;
                tip=line.substring(index).trim();
                t=JIP.tBIT;
                if(tip.compareTo("BIT")==0) t=JIP.tBIT;
                else if(tip.compareTo("BYTE")==0) t=JIP.tBYTE;
                else if(tip.compareTo("WORD")==0) t=JIP.tWORD;
                else if(tip.compareTo("REAL")==0) t=JIP.tREAL;
                else if(tip.compareTo("COLOR")==0) t=JIP.tCOLOR;
                else if(tip.compareTo("POINT")==0) t=JIP.tPOINT;
                else if(tip.compareTo("SEGMENT")==0) t=JIP.tSEGMENT;
                else if(tip.compareTo("POLY")==0) t=JIP.tPOLY;
                else if(tip.compareTo("EDGES")==0) t=JIP.tEDGES;

                //Bandas de la imagen
                line=cab.readLine();
                index=line.indexOf(":")+1;
                b=Integer.parseInt(line.substring(index).trim());

                //Bytes de la imagen
                if (t!=JIP.tPOLY && t!=JIP.tEDGES) {
                  line=cab.readLine();
                  index=line.indexOf(":")+1;
                  index1=line.lastIndexOf(" ");
                  imgsizes[i]=Integer.parseInt(
                  line.substring(index,index1).trim());           
                  arbPoli.addElement(null);
                  arbEdge.addElement(null);

                }
                else {
                  tamPoly=new Vector();
                  
                  while (true) {
                    line=cab.readLine();                    
                    index=line.indexOf(":")+1;
                    if (line.compareTo("\tend")==0) break;
                    index1=line.lastIndexOf(" ");
                    auxmio=Integer.parseInt(line.substring(index,index1).trim());
                    Integer auxmio2=new Integer(auxmio);
                    tamPoly.addElement(auxmio2);    
                  } 
                  
                  arbPoli.addElement((Vector)(tamPoly));               
                  arbEdge.addElement((Vector)(tamPoly));               

                  imgsizes[i]=0;
                  Integer pp=new Integer(0);
                  for (int kl=0;kl<tamPoly.size();kl++) {
                    imgsizes[i]+=((Integer)tamPoly.elementAt(kl)).intValue();
                  }                  
                }
                
                //Linea "\n"
                cab.readLine();

                //Crea la imagen totalmente vacia
                imgs[i]=new JIPImage();

                //Asigna los datos que se conocen 
                imgs[i].width=w;
                imgs[i].height=h;
                imgs[i].type=t;
                imgs[i].nbands=b;
                imgs[i].name=name;

            }

            //Ultima linea de cabecera "RAW\n"
            cab.readLine();

            cab.close();

            /* LECTURA DE DATOS */
            
            //Abre el fichero de entrada con buffering de 4K
            fs=new FileInputStream(file);
            bfs=new BufferedInputStream(fs,4096);
            dat=new DataInputStream(bfs);

            //Busca la marca de comienzo de datos
            while(dat.readUnsignedByte()!=255);

            //Para cada frame de datos
            for(i=0;i<nframes;i++) {
                w=imgs[i].width;
                h=imgs[i].height;
                t=imgs[i].type;
                b=imgs[i].nbands;
                sz=imgsizes[i]/b;

                imgs[i].bmp=new JIPBitmap[b];
                
                //Para cada banda lee y asigna el bitmap
                for(j=0;j<b;j++) {
                    raw=new byte[sz];
                    dat.readFully(raw);
                    switch(t) {
                    case JIP.tBIT:
                        imgs[i].bmp[j]=
                            new JIPBitmap(w,h,JIP.getBitFromRaw(raw,w*h));
                        break;
                    case JIP.tBYTE:
                    case JIP.tCOLOR:
                        imgs[i].bmp[j]=
                            new JIPBitmap(w,h,JIP.getByteFromRaw(raw));
                        break;
                    case JIP.tWORD:
                        imgs[i].bmp[j]=
                            new JIPBitmap(w,h,JIP.getWordFromRaw(raw));
                        break;
                    case JIP.tREAL:
                        imgs[i].bmp[j]=
                            new JIPBitmap(w,h,JIP.getRealFromRaw(raw));
                        break;
                    case JIP.tPOINT:
                        Vector puntos=new Vector();
                      {
                        int npixels=raw.length/2;                        
                        short[] bmp=new short[npixels];
                        for(int i1=0, j1=0;i1<npixels;i1++) {
                             bmp[i1]=(short)((raw[j1]<<8) | (raw[j1+1])& 0x00ff);
                             j1+=2;
                        }
                        for (int i2=0;i2<npixels;i2++){
                           Integer datap=new Integer((int)bmp[i2]);
                           puntos.addElement(datap);
                        }
                      }
                        imgs[i]=new JIPImage(w,h,t,puntos);                        
                        break;
                    case JIP.tSEGMENT:
                        Vector segmentos=new Vector();
                      {
                        int npixels=raw.length/2;                        
                        short[] bmp=new short[npixels];
                        for(int i1=0, j1=0;i1<npixels;i1++) {
                             bmp[i1]=(short)((raw[j1]<<8) | (raw[j1+1])& 0x00ff);
                             j1+=2;
                        }
                        for (int i2=0;i2<npixels;i2++){
                           Integer datas=new Integer((int)bmp[i2]);
                           segmentos.addElement(datas);
                        }
                      }
                        imgs[i]=new JIPImage(w,h,t,segmentos);
                        break;
                    case JIP.tPOLY:
                        Vector tPoligonos=new Vector();
                        Vector poligonos=new Vector();
                      {                        
                        int npixels=raw.length/2;
                        
                        short[] bmp=new short[npixels];
                        for(int i1=0, j1=0;i1<npixels;i1++) {
                           bmp[i1]=(short)((raw[j1]<<8) | (raw[j1+1])& 0x00ff);
                           j1+=2;
                        }
                        for (int i2=0;i2<npixels;i2++){
                           Integer datas=new Integer((int)bmp[i2]);
                           tPoligonos.addElement(datas);
                        }
                        
                        Vector numPoli=new Vector((Vector)(arbPoli.elementAt(i)));
                        int poligonosPorFrame=numPoli.size();
                                              
                        for (int i2=0,elem=0; i2<poligonosPorFrame; i2++){
                          Vector polaux = new Vector();
                         
                          for (int i3=0; i3<(((Integer)(numPoli.elementAt(i2))).intValue())/2;i3++,elem++){
                            polaux.addElement((Integer)(tPoligonos.elementAt(elem))); 
                          }
                          poligonos.addElement((Vector)(polaux).clone());              
                        }
                      }
                      
                        imgs[i]=new JIPImage(w,h,t,poligonos);
                        
                        break;
                    case JIP.tEDGES:
                        Vector tEdges=new Vector();
                        Vector edges=new Vector();
                      {                        
                        int npixels=raw.length/2;
                        
                        short[] bmp=new short[npixels];
                        for(int i1=0, j1=0;i1<npixels;i1++) {
                           bmp[i1]=(short)((raw[j1]<<8) | (raw[j1+1])& 0x00ff);
                           j1+=2;
                        }
                        for (int i2=0;i2<npixels;i2++){
                           Integer datas=new Integer((int)bmp[i2]);
                           tEdges.addElement(datas);
                        }

                        Vector numEdge=new Vector((Vector)(arbEdge.elementAt(i)));
                        int edgesPorFrame=numEdge.size();
                                              
                        for (int i2=0,elem=0; i2<edgesPorFrame; i2++){
                          Vector polaux = new Vector();
                         
                          for (int i3=0; i3<(((Integer)(numEdge.elementAt(i2))).intValue())/2;i3++,elem++){
                            polaux.addElement((Integer)(tEdges.elementAt(elem))); 
                          }
                          edges.addElement((Vector)(polaux));                          
                        }
                      }
                        imgs[i]=new JIPImage(w,h,t,edges);
                        break;
                    }
                }
                res.addFrame(imgs[i]);
            }
            fs.close();

        } catch(Exception e) { return null; }

        return res;
    }

    /**************************************/
    /* Metodos para salvar una Secuencia  */
    /**************************************/

    /**<P><FONT COLOR="red">
     *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Salva un objeto secuencia JIP en un fichero con formato JIP.
     * @param seq Secuencia a salvar.
     * @param file Fichero en el cual almacenar la secuencia (si no existe se crea,
     * si existe se sobreescribe)
     */
    public static void saveSeqIntoFile(JIPSequence seq,String file) {
        JIPImage img;
        byte[] raw;

        //Streams para escribir en el fichero
        FileOutputStream fs=null;
        BufferedOutputStream bfs=null;
        DataOutputStream dat=null;
        BufferedWriter cab=null;
        
        String line;
        int i,j,nframes,sz;
        int w,h,b,t;
        String name,tip;

        try {

            //Abre el fichero de salida con buffering de 4K
            fs=new FileOutputStream(file);
            bfs=new BufferedOutputStream(fs,4096);
            dat=new DataOutputStream(bfs);
            cab=new BufferedWriter(new OutputStreamWriter(dat));

            /* ESCRITURA DE LA CABECERA */

            //Linea 1 "JIP\n"
            line="JIP";
            cab.write(line,0,line.length());
            cab.newLine();

            //Linea 2 "\n"
            cab.newLine(); 

            //Linea 3 "Sequence:<name>\n"
            line="Sequence:"+seq.name;
            cab.write(line,0,line.length());
            cab.newLine();

            //Linea 4 "Frames:<nframes>\n"
            nframes=seq.nframes;
            line="Frames:"+nframes;
            cab.write(line,0,line.length());
            cab.newLine();

            //Linea 5 "\n"
            cab.newLine();

            //Para cada frame de la secuencia
            for(i=0;i<nframes;i++) {

                //Obtiene datos de la imagen del frame
                img=seq.getFrame(i);
                t=img.type;
                Vector geoData;
                tip="";
                name="";
                w=0;
                h=0;
                b=0;
                sz=0;

                switch(t) {
                case JIP.tBIT:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
                    tip="BIT"; sz=((int)Math.ceil((double)(w*h)/8.0))*b;
                    break;
                case JIP.tBYTE:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
                    tip="BYTE"; sz=w*h*b;
                    break;
                case JIP.tCOLOR:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
                    tip="COLOR"; sz=w*h*b;
                    break;
                case JIP.tWORD:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
                    tip="WORD"; sz=w*h*2*b;
                    break;
                case JIP.tREAL:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
                    tip="REAL"; sz=w*h*4*b;
                    break;
								case JIP.tPOINT:
                    w=img.width;
                    h=img.height;
                    Vector auxx=new Vector();
                    b=img.nbands;
                    name=img.name;
                    auxx=img.getVector();
										tip="POINT"; 
                    sz=img.getVector().size();
                    sz*=2;
                    break;
								case JIP.tSEGMENT:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
										tip="SEGMENT"; 
                    sz=img.getVector().size();
                    sz*=2;
										break;
								case JIP.tPOLY:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
										tip="POLY"; 
                    sz=0;
                    Vector vaux1;
                    geoData=new Vector (img.getVector());
                    for (int a=0; a< geoData.size(); a++){
                      vaux1=new Vector((Vector)geoData.elementAt(a));
                      sz+=vaux1.size();
                    }
                    sz*=2;
                    break;
								case JIP.tEDGES:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
										tip="EDGES";                    
                    sz=0;
                    Vector vaux2;
                    geoData=new Vector (img.getVector());
                    for (int a=0; a< geoData.size(); a++){
                      vaux2=new Vector((Vector)geoData.elementAt(a));
                      sz+=vaux2.size();
                    }
                    sz*=2;
                    break;
                }
                
                //Nombre de la imagen
                line="Frame "+i+":"+name;
                cab.write(line,0,line.length());
                cab.newLine();

                //Anchura y altura de la imagen
                line="\tPixels:"+String.valueOf(w)+"x"+String.valueOf(h);
                cab.write(line,0,line.length());
                cab.newLine();

                //Tipo de la imagen
                line="\tType:"+tip;
                cab.write(line,0,line.length());
                cab.newLine();

                //Bandas de la imagen
                line="\tBands:"+String.valueOf(b);
                cab.write(line,0,line.length());
                cab.newLine();

                //Bytes de la imagen
                if (t!=JIP.tPOLY && t!=JIP.tEDGES) {
                line="\tSize:"+String.valueOf(sz)+" bytes";
                cab.write(line,0,line.length());
                cab.newLine();
                }
                else {
                  sz=0;
                  Vector vaux1;
                  geoData=new Vector (img.getVector());
                  for (int a=0; a< geoData.size(); a++){
                      vaux1=new Vector((Vector)geoData.elementAt(a));
                      line="\tSize:"+String.valueOf(vaux1.size()*2)+" bytes";
                      cab.write(line,0,line.length());
                      cab.newLine();
                  }
                  line="\t"+"end";
                  cab.write(line,0,line.length());
                  cab.newLine();
                  
                }
                //Linea "\n"
                cab.newLine();

            }
            
            //Ultima linea de cabecera "RAW\n"
            line="RAW";
            cab.write(line,0,line.length());
            cab.newLine();

            //Vuelca la cabecera
            cab.flush();
            
            /* ESCRITURA DE DATOS */

            //Marca de comienzo de datos RAW
            dat.writeByte(255);

            //Para cada frame de la secuencia
            for(i=0;i<nframes;i++) {

                //Obtiene datos de la imagen del frame
                img=seq.getFrame(i);
                b=img.nbands;
                t=img.type;

                //Para cada banda del frame
                for(j=0;j<b;j++) {
                    raw=null;
                    switch(t) {
                    case JIP.tBIT:
                        raw=JIP.getRawFromBit(img.bmp[j].bmpBit);
                        break;
                    case JIP.tBYTE:
                    case JIP.tCOLOR:
                        raw=JIP.getRawFromByte(img.bmp[j].bmpByte);
                        break;
                    case JIP.tWORD:
                        raw=JIP.getRawFromWord(img.bmp[j].bmpWord);
                        break;
                    case JIP.tREAL:
                        raw=JIP.getRawFromReal(img.bmp[j].bmpReal);
                        break;
                        
                    case JIP.tSEGMENT:
                      raw=JIP.getRawFromSegment(img.bmp[j].Segmentos);
                      break;
                    case JIP.tPOLY:
                      raw=JIP.getRawFromPoly(img.bmp[j].Poligonos);
                      break;

                    case JIP.tPOINT:
                      raw=JIP.getRawFromPoint(img.bmp[j].Puntos);
                      break;

                    case JIP.tEDGES:
                      raw=JIP.getRawFromEdges(img.bmp[j].Edges);
                      break;

                    }
                    dat.write(raw,0,raw.length);
                }                
            }

            //Vuelca los datos
            dat.flush();

            fs.close();

        } catch(Exception e) {System.err.println(e);}
    }
    
    /**<P><FONT COLOR="red">
     *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Salva un objeto secuencia JIP en un fichero con formato ZIP.
     * @param seq Secuencia a salvar.param file Nombre del fichero .jip.param fliezip Nombre del fichero .zip
     * @param file Fichero en el cual almacenar la secuencia (si no existe se crea,
     * si existe se sobreescribe)
     */
     

     public static void saveSeqIntoFileZip(JIPSequence seq, String dir, String namefile) {
				JIPImage img;
        byte[] raw;

        //Streams para escribir en el fichero
        FileOutputStream fs=null;
        BufferedOutputStream bfs=null;
        DataOutputStream dat=null;
        BufferedWriter cab=null;
        
        String line;
        int i,j,nframes,sz;
        int w,h,b,t;
        String name,tip;

        try {

            //Abre el fichero de salida con buffering de 4K
            fs=new FileOutputStream(dir+"temp.~~~");
            bfs=new BufferedOutputStream(fs,4096);
            dat=new DataOutputStream(bfs);
            cab=new BufferedWriter(new OutputStreamWriter(dat));

            /* ESCRITURA DE LA CABECERA */

            //Linea 1 "JIP\n"
            line="JIP";
            cab.write(line,0,line.length());
            cab.newLine();

            //Linea 2 "\n"
            cab.newLine(); 

            //Linea 3 "Sequence:<name>\n"
            line="Sequence:"+seq.name;
            cab.write(line,0,line.length());
            cab.newLine();

            //Linea 4 "Frames:<nframes>\n"
            nframes=seq.nframes;
            line="Frames:"+nframes;
            cab.write(line,0,line.length());
            cab.newLine();

            //Linea 5 "\n"
            cab.newLine();

            //Para cada frame de la secuencia
            for(i=0;i<nframes;i++) {

                //Obtiene datos de la imagen del frame
                img=seq.getFrame(i);
                t=img.type;
                Vector geoData;
                tip="";
                name="";
                w=0;
                h=0;
                b=0;
                sz=0;

                switch(t) {
                case JIP.tBIT:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
                    tip="BIT"; sz=((int)Math.ceil((double)(w*h)/8.0))*b;
                    break;
                case JIP.tBYTE:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
                    tip="BYTE"; sz=w*h*b;
                    break;
                case JIP.tCOLOR:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
                    tip="COLOR"; sz=w*h*b;
                    break;
                case JIP.tWORD:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
                    tip="WORD"; sz=w*h*2*b;
                    break;
                case JIP.tREAL:
                    w=img.width;
                    h=img.height;
                    b=img.nbands;
                    name=img.name;
                    tip="REAL"; sz=w*h*4*b;
                    break;
								case JIP.tPOINT:
                    Vector auxx=new Vector();
                    b=img.nbands;
                    name=img.name;
                    auxx=img.getVector();
										tip="POINT"; 
                    sz=img.getVector().size();
                    sz*=2;
                    break;
								case JIP.tSEGMENT:
                    b=img.nbands;
                    name=img.name;
										tip="SEGMENT"; 
                    sz=img.getVector().size();
                    sz*=2;
										break;
								case JIP.tPOLY:
                    b=img.nbands;
                    name=img.name;
										tip="POLY"; 
                    sz=0;
                    Vector vaux1;
                    geoData=new Vector (img.getVector());
                    for (int a=0; a< geoData.size(); a++){
                      vaux1=new Vector((Vector)geoData.elementAt(a));
                      sz+=vaux1.size();
                    }
                    sz*=2;
                    break;
								case JIP.tEDGES:
                    b=img.nbands;
                    name=img.name;
										tip="EDGES";                    
                    sz=0;
                    Vector vaux2;
                    geoData=new Vector (img.getVector());
                    for (int a=0; a< geoData.size(); a++){
                      vaux2=new Vector((Vector)geoData.elementAt(a));
                      sz+=vaux2.size();
                    }
                    sz*=2;
                    break;
                }
                
                //Nombre de la imagen
                line="Frame "+i+":"+name;
                cab.write(line,0,line.length());
                cab.newLine();

                //Anchura y altura de la imagen
                line="\tPixels:"+String.valueOf(w)+"x"+String.valueOf(h);
                cab.write(line,0,line.length());
                cab.newLine();

                //Tipo de la imagen
                line="\tType:"+tip;
                cab.write(line,0,line.length());
                cab.newLine();

                //Bandas de la imagen
                line="\tBands:"+String.valueOf(b);
                cab.write(line,0,line.length());
                cab.newLine();

                //Bytes de la imagen
                if (t!=JIP.tPOLY && t!=JIP.tEDGES) {
                line="\tSize:"+String.valueOf(sz)+" bytes";
                cab.write(line,0,line.length());
                cab.newLine();
                }
                else {
                  sz=0;
                  Vector vaux1;
                  geoData=new Vector (img.getVector());
                  for (int a=0; a< geoData.size(); a++){
                      vaux1=new Vector((Vector)geoData.elementAt(a));
                      line="\tSize:"+String.valueOf(vaux1.size()*2)+" bytes";
                      cab.write(line,0,line.length());
                      cab.newLine();
                  }
                  line="\t"+"end";
                  cab.write(line,0,line.length());
                  cab.newLine();
                  
                }
                //Linea "\n"
                cab.newLine();

            }
            
            //Ultima linea de cabecera "RAW\n"
            line="RAW";
            cab.write(line,0,line.length());
            cab.newLine();

            //Vuelca la cabecera
            cab.flush();
            
            /* ESCRITURA DE DATOS */

            //Marca de comienzo de datos RAW
            dat.writeByte(255);

            //Para cada frame de la secuencia
            for(i=0;i<nframes;i++) {

                //Obtiene datos de la imagen del frame
                img=seq.getFrame(i);
                b=img.nbands;
                t=img.type;

                //Para cada banda del frame
                for(j=0;j<b;j++) {
                    raw=null;
                    switch(t) {
                    case JIP.tBIT:
                        raw=JIP.getRawFromBit(img.bmp[j].bmpBit);
                        break;
                    case JIP.tBYTE:
                    case JIP.tCOLOR:
                        raw=JIP.getRawFromByte(img.bmp[j].bmpByte);
                        break;
                    case JIP.tWORD:
                        raw=JIP.getRawFromWord(img.bmp[j].bmpWord);
                        break;
                    case JIP.tREAL:
                        raw=JIP.getRawFromReal(img.bmp[j].bmpReal);
                        break;
                        
                    case JIP.tSEGMENT:
                      raw=JIP.getRawFromSegment(img.bmp[j].Segmentos);
                      break;
                    case JIP.tPOLY:
                      raw=JIP.getRawFromPoly(img.bmp[j].Poligonos);
                      break;

                    case JIP.tPOINT:
                      raw=JIP.getRawFromPoint(img.bmp[j].Puntos);
                      break;

                    case JIP.tEDGES:
                      raw=JIP.getRawFromEdges(img.bmp[j].Edges);
                      break;

                    }
                    dat.write(raw,0,raw.length);
                }                
            }

            //Vuelca los datos
            dat.flush();

            fs.close();

            // fichero .zip
           // System.out.println("LO GUARDO");
            FileOutputStream f = new FileOutputStream(dir+namefile+".jipz");
            CheckedOutputStream csum = new CheckedOutputStream(f, new CRC32());
            ZipOutputStream out = new ZipOutputStream (new BufferedOutputStream(csum));
            
            ZipEntry ze = new ZipEntry(namefile+".jip");

            BufferedInputStream in = new BufferedInputStream (new FileInputStream (dir+"temp.~~~"));
                       
            System.out.println("Comprimiendo...");
            long t0,t1;
            
            File aux= new File (dir+"temp.~~~");  // longitud en bytes del fichero
            long longi = aux.length();
            
            out.putNextEntry(ze);
            
            out.setLevel(9);
            out.setComment("Imagen JIP (Sistemas Informaticos)");
            
            t0 = System.currentTimeMillis();  // tiempo inicial
                       
            byte b1[]=new byte[(int)longi];  
            in.read(b1,0,(int)longi);        // leo el fichero
            out.write(b1);                   // grabo los datos comprimidos
            
            out.closeEntry();
            
            in.close();
            out.close(); 
            aux.delete();
            
            t1 = System.currentTimeMillis();  // tiempo final
                                   
            float tsc=(float)ze.getSize();;
            float tco=(float)ze.getCompressedSize();
            float ratio=(float)(100.0-((tco*100.0)/tsc));
            
            System.out.println("Proceso de compresion Terminando en: "+(double)(t1-t0)/1000.0+" seg.");
            System.out.println("Original = "+(long)tsc+" bytes. Comprimido = "+(long)tco+" bytes. Ratio = "+ratio+" %");
            
        } catch(Exception e) {System.err.println(e);}

    }



    /**<P><FONT COLOR="red">
     *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     * Salva un objeto secuencia JIP en ficheros con formato TGA (Targa).
     * Si la secuencia dispone de varios frames, se generar un fichero para
     * cada uno de ellos, (o ms si los frames tienen varias bandas de grises),
     * aadindose al nombre (antes de la extensin) el postfijo "_f0", "_f1", ...
     * segn el frame al que corresponda y el postfijo "_b0", "_b1", ... en el caso
     * de varias bandas de grises para un mismo frame.
     * @param seq Secuencia a salvar.
     * @param file Fichero en el cual almacenar la secuencia (si no existe se crea,
     * si existe se sobreescribe)
     */
    public static void saveSeqIntoTga(JIPSequence seq,String file) {
    	
    	//Obtiene nombre del fichero y extension
    	String name=null; String ext=null;
    	if(file.lastIndexOf(".")==-1) {
    		name=file;
    		ext=".tga";
    	}
    	else {
    		name=file.substring(0,file.lastIndexOf("."));
    		ext=file.substring(file.lastIndexOf("."));
    	}
    	
    	int nf=seq.getNumFrames();
    	for(int i=0;i<nf;i++) {
    		if(nf>1) saveImageIntoTga(seq.getFrame(i),name+"_f"+i+ext);
    		else saveImageIntoTga(seq.getFrame(i),name+ext);
    	}

    }

		/**<P><FONT COLOR="red">
     *<B>Descripcin:</B><BR>
     *<FONT COLOR="blue">
     *Dibuja una linea, cuyas coordenadas se pasan como parmetro,
     *mediante el algorirmo de Bressenham<BR>.
     *@param x0 Coordenada X del punto inicial
     *@param y0 Coordenada Y del punto inicial
		 *@param x1 Coordenada X del punto final
		 *@param y1 Coordenada Y del punto final
		 *@param icol Columna	
     *@param irow Fila
     *@param pix Array de Pixels
     */ 
     

public static void drawline(int x0, int y0, int x1, int y1,int icol,int irow,int pix[]) {
    int xmin,xmax; /* Coordenadas de la Lnea */
    int ymin,ymax;
    int dir;       /* Direccin de bsqueda */
    int dx;       
    int dy;       

    /* Incrementos Este, Nort-Este, Sur, Sur-Este, Norte */
    int incrE,
        incrNE,
        incrS,
        incrSE,
        incrN;     
    int d;         
    int x,y;       
    int mpCase; 
    int done;   
    int color=65535;
    xmin=x0;
    xmax=x1;
    ymin=y0;
    ymax=y1;
    
    dx=xmax-xmin;
    dy=ymax-ymin;

    if (dx*dx>dy*dy) /* busqueda horizontal */ {
      dir=0;
      if (xmax<xmin) {
        {xmin^=xmax; xmax^=xmin; xmin^=xmax;}
        {ymin^=ymax; ymax^=ymin; ymin^=ymax;}
      }
      dx=xmax-xmin;
      dy=ymax-ymin;

      if (dy>=0) {
        mpCase=1;
        d=2*dy-dx;      
      } else {
        mpCase=2;
        d=2*dy+dx;      
      }

      incrNE=2*(dy-dx);
      incrE=2*dy;
      incrSE=2*(dy+dx);
    } else {/* bsqueda vertical */
      dir=1;
      if (ymax<ymin) {
        {xmin^=xmax; xmax^=xmin; xmin^=xmax;}
        {ymin^=ymax; ymax^=ymin; ymin^=ymax;}
      }
      dx=xmax-xmin;
      dy=ymax-ymin;    

      if (dx>=0) {
        mpCase=1;
        d=2*dx-dy;      
      } else {
        mpCase=2;
        d=2*dx+dy;      
      }

      incrNE=2*(dx-dy);
      incrE=2*dx;
      incrSE=2*(dx+dy);
    }
    
    /* Comenzamos la bsqueda */
    x=xmin;
    y=ymin;
    done=0;

   
    while(done==0) {
      if(x>0 && x<icol && y>0 && y<irow)
       pix[y*icol+x]=color;
    
      /* Movemos al siguiente punto */
      switch(dir) {
        case 0: /* horizontal */
        {
          if (x<xmax)
          {
          switch(mpCase) {
              case 1:
              if (d<=0)
              {
                d+=incrE;  x++;
              }
                else
                {
                  d+=incrNE; x++; y++;
                }
              break;
    
              case 2:
                if (d<=0)
                {
                  d+=incrSE; x++; y--;
                }
                else
                {
                  d+=incrE;  x++;
                }
              break;
          } 
          } 
          else
          done=1;
        }  
        break;

        case 1: /* vertical */
        {
          if (y<ymax)
          {
            switch(mpCase)
            {
              case 1:
                if (d<=0) {
                  d+=incrE;  y++;
                } else {
                  d+=incrNE; y++; x++;
                }
              break;
    
              case 2:
                if (d<=0) {
                  d+=incrSE; y++; x--;
                } else {
                  d+=incrE;  y++;
                }
              break;
          } 
          } 
          else
          done=1;
        }
        break;    
      }
    }
  }

}