package jipfunc;

import jip.*;
import java.util.Vector;

/**
* <P><FONT COLOR="red">
*<B>FCanny:</B> Deteccion de bordes por el metodo de Canny<BR>
*<FONT COLOR="blue">
*DESCRIPCION: En el mtodo de Canny para la deteccin de bordes
*se busca el filtro ptimo que, convolucionado con la imagen 
*produce un mximo local donde existe un borde, eliminando el efecto
*del ruido. Para obtener el filtro ptimo se definen dos criterios
*que deben ser maximizados:<BR>
*<B> Buena deteccin: </B>
*Debe existir una probabilidad baja de que 
*se dejen de marcar pxels que forman parte de una arista, as como 
*de marcar como aristas pxels que realmente no lo son.<BR>
*<B> Buena localizacin: </B>
*Los puntos marcados como puntos de borde
*por el operador deberan estar lo ms cerca posible del verdadero
*centro de la arista.<BR>
*Aplicable a tipos: BYTE, WORD, COLOR y REAL<BR>
*Usa: FEqualize y FBrightness <BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>sigma: Nivel de suavizado gaussiano<BR>
*<li>brillo: Ajuste de brillo en imagen resultado<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen Procesada del mismo tipo que la imagen de entrada<BR><BR>
*</ul>
*Notas Adicionales: Se efectua una supresion de valores no-maximos, 
*por lo que los bordes tienen una anchura de un pixel, lo cual dificulta
*su visualizacion, por ello, es recomendable aumentar el brillo.<BR>
*Para normalizar el resultado, se llama a FEqualize
*/


public class FCanny extends JIPFunction {

/**
*<P><FONT COLOR="red">
*Constructor de la Clase.<BR>
*<P><FONT COLOR="blue">		
*Obtiene los parmetros de entrada para aplicar la funcin.
*<ul><B>Parametros de Entrada:</B><BR>
*<li>NINGUNO<BR><BR>
*</ul>
<ul><B>Parametros de Salida:</B><BR>
*<li>NINGUNO<BR><BR>
*</ul>
*/
public FCanny() 
  {
		//Nombre y descripcion de la Funcion
	 name="FCanny";
   description="Deteccion de bordes por el metodo de Canny";
   JIPParameter p1=new JIPParameter("sigma",JIP.pREAL,false);
   p1.setDefault((float)1.0);
   p1.setDescription("Nivel de suavizado gaussiano");
   JIPParameter p2=new JIPParameter("brillo",JIP.pINT,false);
   p2.setDefault(100);
   p2.setDescription("Ajuste de brillo");

		//Asignacion de Parametros
 	 params=new Vector();
   params.addElement(p1);  
   params.addElement(p2);	
  }

/**
*<P><FONT COLOR="red">
*Mtodo Principal<BR>
*<P><FONT COLOR="blue">		
*Se le llama para procesar la imgen con el algoritmo de deteccin
*de bordes de Canny.<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen a procesar<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen Procesada del mismo tipo que la imagen de entrada<BR><BR>
*</ul>
*/

 public JIPImage processImg(JIPImage img) 
   {
		JIPImage res=null;
		int t=img.getType();
		int cont=0;
		if(img!=null) 
      {
			 int w=img.getWidth();
			 int h=img.getHeight();
			 int b=img.getNumBands();
			 int max,x,y;
       switch(t) 
        {
         case JIP.tBYTE:  max=255;   break;
         case JIP.tWORD:  max=65535; break;
         case JIP.tREAL:  max=1;     break;
         case JIP.tCOLOR: 
            max=1;
            int[] red=img.getAllPixelRed();
            int[] green=img.getAllPixelGreen();
            int[] blue=img.getAllPixelBlue();
            int[] gray=new int[w*h];
            for(int i=0;i<w*h;i++)
               gray[i]=(int)Math.round(0.299*red[i]+0.587*green[i]+0.114*blue[i]);
            img=new JIPImage(w,h,JIP.getAllRealFromByte(gray));
            b=img.getNumBands();
            t=img.getType();
            red=green=blue=gray=null;
            break;
         default: return null;
        }
      
      res=new JIPImage(b,w,h,t);
      float p1=((JIPParameter)params.elementAt(0)).getValueReal();
      int p2=((JIPParameter)params.elementAt(1)).getValueInt();

      if (p1!=0)
        {
         JIPFunction suavizado=new FSmoothGaussian();
         suavizado.setParamValue("sigma",p1);
         suavizado.setParamValue("borde",1);
			   img=suavizado.processImg(img); // Llamamos a FSmoothGaussian
        }
			
			for (int i=0;i<b;i++) { // contador de bandas
        
        float Dx,Dy;
        float[] bmp=new float[w*h];
        float[] gradX=new float[w*h];
        float[] gradY=new float[w*h];
        float[] mag  =new float[w*h];
        if (t==JIP.tREAL) bmp=img.getAllPixelFlo(i);
        else
           {
            int[] bmpAux=img.getAllPixel(i);
            for (int a=0;a<w*h;a++) bmp[a]=(float)(bmpAux[a]/(float)max);
            bmpAux=null;
           }
        for (x=1;x<w-1;x++) 
              {
               for (y=1;y<h-1;y++) 
                 { 
                  Dx = (bmp[(y + 1)*w+x + 1] + 2 * bmp[y*w+x + 1]
                      + bmp[(y - 1)*w+x + 1] - bmp[(y + 1)*w+x - 1]
                      - 2 * bmp[y*w+x - 1] - bmp[(y - 1)*w+x - 1])/ 8;

                  Dy = (bmp[(y + 1)*w+x + 1] + 2 * bmp[(y + 1)*w+x]
                      + bmp[(y + 1)*w+x - 1] - bmp[(y - 1)*w+x + 1]
                      - 2 * bmp[(y - 1)*w+x] - bmp[(y - 1)*w+x - 1])/ 8;
                  gradX[y*w+x]=Dx;
                  gradY[y*w+x]=Dy;
                  mag[y*w+x] = (float)(Math.sqrt(Dx * Dx + Dy * Dy));
                 }
              }
      bmp=null;
      float[] bin=new float[w*h];
      float g;              // Magnitud de gradiente del pixel central
      float g1, g2;         // Magnitudes de gradiente interpoladas
      float ga, gb, gc, gd; // Magnitudes de gradiebte de los pixels mas cercanos
      float ux, uy;        
      float dx, dy;
      
      // Para cada pixel, se comprueba si es maximo local en una
      // vecindad de 3x3 en la orientacion del gradiente
      for (x = 1; x < w - 1; x++) 
       {
        for (y = 1; y < h - 1; y++) 
         {
          dx = gradX[y*w+x];
          dy = gradY[y*w+x];
          if (dx == 0.0f && dy == 0.0f)  continue;
          if (Math.abs(dy)>Math.abs(dx)) 
           {
            ux = dx/dy;
            uy = 1.0f;
            gb = mag[(y-1)*w+x];
            gd = mag[(y+1)*w+x];
            if (dx*dy<0) 
             { ga = mag[(y-1)*w+x-1]; gc = mag[(y+1)*w+x+1]; } 
            else 
             { ga = mag[(y-1)*w+x+1]; gc = mag[(y+1)*w+x-1]; }
           } 
          else 
           {
            ux = dy/dx;
            uy = 1.0f;
            gb = mag[y*w+x+1];
            gd = mag[y*w+x-1];
            if (dx*dy<0)  
             { ga = mag[(y+1)*w+x+1]; gc = mag[(y-1)*w+x-1]; } 
            else 
             { ga = mag[(y-1)*w+x+1]; gc = mag[(y+1)*w+x-1]; }
           }
          g1 = (ux * ga) + (uy - ux) * gb;
          g2 = (ux * gc) + (uy - ux) * gd;
          g = (float)Math.sqrt (dx * dx + dy * dy);
          if (g > g1 && g >= g2)bin[y*w+x]=mag[y*w+x];
          else bin[y*w+x]=0;
         }
       }
      gradX=null;
      gradY=null;
      mag=null;

    
      if (t!=JIP.tREAL)
       {
        int[] binAux=new int[w*h];
        for (int a=0;a<w*h;a++) binAux[a]=(int)((bin[a])*(float)max+0.5);
        res.setAllPixel(i,binAux);
       }
      else 
        res.setAllPixel(bin);
       
      }
     
     JIPFunction equ=new FEqualize();
     res=equ.processImg(res); // Equalizamos
     
     JIPFunction brillo=new FBrightness();
     brillo.setParamValue("porcentaje",p2);
     res=brillo.processImg(res); // Ajustamos brillo
 }	
 return res;
	}
}