package jipfunc;

import jip.*;
import java.util.Vector;

/**
* <P><FONT COLOR="red">
*<B>FColorToGray:</B> Convierte una imagen de tipo COLOR a escala de grises<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Convierte una imagen de tipo COLOR, a una imagen de 
*tipo BIT, BYTE, WORD o REAL<BR>
*Aplicable a tipos: COLOR<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>gray : Tipo de imagen destino (BIT, BYTE, WORD, REAL)<BR><BR>
*</ul>
<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen equivalente a la imagen de tipo Color de entrada pero en 
*escala de grises<BR><BR>
*</ul>
*/



public class FColorToGray extends JIPFunction {
		/**
		*<P><FONT COLOR="red">
		*Constructor de la Clase.<BR>
		*<P><FONT COLOR="blue">		
		*Obtiene los parmetros de entrada para aplicar la funcin.
		*<ul><B>Parametros de Entrada:</B><BR>
		*<li>NINGUNO<BR><BR>
		*</ul>
		<ul><B>Parametros de Salida:</B><BR>
		*<li>NINGUNO<BR><BR>
		*</ul>
		*/

    public FColorToGray() {
		
		//Nombre y descripcion de la Funcion
        name="FColorToGray";
        description="Convierte imagenes tipo COLOR a escala de grises.";

        //Definicion de Parametros
		JIPParameter p1=new JIPParameter("gray",JIP.pSTRING,false);
		p1.setDefault("BYTE");
		p1.setDescription("Tipo de gris destino: BIT, BYTE, WORD o REAL");

        //Asignacion de Parametros
        params=new Vector();
		params.addElement(p1);
    }
    
    /**
		*<P><FONT COLOR="red">
		*Mtodo Principal<BR>
		*<P><FONT COLOR="blue">		
		*Se le llama para cambiar el tipo de la imagen de entrada, 
		*desde COLOR a escala de grises.
		*<ul><B>Parametros de Entrada:</B><BR>
		*<li>img: Imagen a procesar<BR><BR>
		*</ul>
		<ul><B>Parametros de Salida:</B><BR>
		*<li>Imagen del tipo especificado por el usuario equivalente
		*a la imagen de entrada de tipo COLOR<BR><BR>
		*</ul>
		*/

    public JIPImage processImg(JIPImage img) {
    	
    	//Obtenemos el tipo de gris al cual convertir		
		String p1=getParamValueStr("gray").toUpperCase();
    	int tipo=JIP.tBYTE;
		if(p1.equals("BIT")) tipo=JIP.tBIT;
		else if(p1.equals("BYTE")) tipo=JIP.tBYTE;
		else if(p1.equals("WORD")) tipo=JIP.tWORD;
		else if(p1.equals("REAL")) tipo=JIP.tREAL;
    	
		JIPImage res=null;
		if(img!=null&&img.getType()==JIP.tCOLOR) {

			//Obtenemos en gray la version convertida a gris tBYTE
			int w=img.getWidth();
			int h=img.getHeight();
			int[] red=img.getAllPixelRed();
			int[] green=img.getAllPixelGreen();
			int[] blue=img.getAllPixelBlue();
			int[] gray=new int[w*h];
			for(int i=0;i<w*h;i++)
				gray[i]=(int)Math.round(0.299*red[i]+0.587*green[i]+0.114*blue[i]);

			//Creamos la imagen resultado convirtiendo al gris destino
			switch(tipo) {
			case JIP.tBIT:
				res=new JIPImage(w,h,JIP.tBIT,JIP.getAllBitFromByte(gray));
				break;
			case JIP.tBYTE:
				res=new JIPImage(w,h,JIP.tBYTE,gray);
				break;
			case JIP.tWORD:
				res=new JIPImage(w,h,JIP.tWORD,JIP.getAllWordFromByte(gray));
				break;
			case JIP.tREAL:
				res=new JIPImage(w,h,JIP.getAllRealFromByte(gray));
				break;
           }
		}
		
		return res;
	}
}