package jipfunc;

import jip.*;
import java.util.Vector;
import java.util.Random;
import java.io.*;

/**
* <P><FONT COLOR="red">
*<B>FConvolveAscii:</B> Convolucion de un imagen con una matriz<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Convoluciona una imagen con una matriz que lee
*desde un archivo de texto en formato Ascii. El archivo en la primera
*linea especifica el ancho y el alto, y en las restantes y separados por 
*espacios o tabulaciones, los datos.<BR>
*Aplicable a tipos: BYTE, WORD, COLOR y REAL<BR>
*Usa: FConvolveImage<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>matriz: Archivo donde se encuentra la Matriz con la que convolucionar<BR>
*<li>mult: Multiplicador<BR>
*<li>div: Divisor <BR>
*<li>metodo: ZERO-> los pixels que quedan fuera se ponen a 0.
*PAD->cuando el pixel est fuera, se toma el valor del pixel de la imagen
*ms cercano. P.e. (-1,-1) se tomara (0,0), (-2,2) se tomara (0,2), etc.
*WRAP-> cuando el pxel est fuera, se toma el valor del pixel opuesto en
*los ejes que corresponda. P.e. (-1,-1) se tomara (w,h), (0,-2) se tomara
*(0,h-1) etc.<BR><BR>
*</ul>
<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen convolucionada con la matriz de entrada, del mismo tipo que la imagen de entrada<BR><BR>
*</ul>
*Notas Adicionales: La matriz se convierte a una imagen y se llama al
*metodo que realiza la convolucin de dos imagenes.
*/


public class FConvolveAscii extends JIPFunction {
		/**
		*<P><FONT COLOR="red">
		*Constructor de la Clase.<BR>
		*<P><FONT COLOR="blue">		
		*Obtiene los parmetros de entrada para aplicar la funcin.
		*<ul><B>Parametros de Entrada:</B><BR>
		*<li>NINGUNO<BR><BR>
		*</ul>
		<ul><B>Parametros de Salida:</B><BR>
		*<li>NINGUNO<BR><BR>
		*</ul>
		*/

    public FConvolveAscii() {
		//Nombre y descripcion de la Funcion
	    name="FConvolveAscii";
	    description=
	         "Convolucion de un imagen con una matriz";

	    //Definicion de Parametros
		JIPParameter p1=new JIPParameter("matriz",JIP.pSTRING,true);
    p1.setDescription("Matriz con la que convolucionar");
    JIPParameter p2=new JIPParameter("mult",JIP.pREAL,false);
    p2.setDefault((float)1.0);
    p2.setDescription("Multiplicador");
    JIPParameter p3=new JIPParameter("div",JIP.pREAL,false);
    p3.setDefault((float)1.0);
    p3.setDescription("Divisor");
    JIPParameter p4=new JIPParameter("metodo",JIP.pSTRING,false);
    p4.setDefault("ZERO");
    p4.setDescription("ZERO, WRAP, PAD");
            
            

		//Asignacion de Parametros
    params=new Vector();
		params.addElement(p1);
    params.addElement(p2);
    params.addElement(p3);
    params.addElement(p4);

    }

		/**
		*<P><FONT COLOR="red">
		*Mtodo Principal<BR>
		*<P><FONT COLOR="blue">		
		*Se le llama para convolucionar la imagen de entrada con
		*una matriz introducida mediante un archivo.<BR>
		*<ul><B>Parametros de Entrada:</B><BR>
		*<li>img: Imagen a procesar<BR><BR>
		*</ul>
		<ul><B>Parametros de Salida:</B><BR>
		*<li>Imagen convolucionada con la matiz que el usuario especifica
		*mediante un archivo.<BR><BR>
		*</ul>
		*/

    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		
    String convoMat=getParamValueStr("matriz");
    float p2=((JIPParameter)params.elementAt(1)).getValueReal();
    float p3=((JIPParameter)params.elementAt(2)).getValueReal();
    String p4=getParamValueStr("metodo").toUpperCase();

    try 
     {
       
      FileInputStream convoF=new FileInputStream(convoMat);
        
      Reader r = new BufferedReader(new InputStreamReader(convoF));
      StreamTokenizer st = new StreamTokenizer(r);
   
      int cw,ch;
      st.nextToken(); cw=(int)st.nval;
      st.nextToken(); ch =(int)st.nval;
    
      float[] mat=new float[cw*ch];
      for (int cont=0;cont<cw*ch;cont++)     
        {
         st.nextToken();
         if (st.ttype == StreamTokenizer.TT_EOF)
          {
           System.out.println("Error en la lectura del fichero");
           return null;
          } 
         float f=(float)st.nval;
         mat[cont]=f;
        }

      JIPFunction convolucion=new FConvolveImage();
       
      JIPImage convo=new JIPImage(cw,ch,mat);
    
      convolucion.setParamValue("imagen",convo);
      convolucion.setParamValue("div",p3);
      convolucion.setParamValue("mult",p2);
      convolucion.setParamValue("metodo",p4);
    
      // LLamamos a FConvolve
      res=convolucion.processImg(img);

    
		
	}catch(FileNotFoundException e) 
              { System.out.println("File Not Found:"); 
                return null;
              } 
   catch(IOException e) 
              { System.out.println("IO Exception"); }
  return res;
 }
}