package jipfunc;

import jip.*;
import java.util.Vector;
import java.util.Random;

/**
* <P><FONT COLOR="red">
*<B>FConvolveImage:</B> Convolucion de un imagen con otra imagen<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Convoluciona una imagen con otra imagen especificada por
*el usuario.<BR>
*Aplicable a tipos: BYTE, WORD, COLOR y REAL<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>imagen: Imagen con la que convolucionar<BR>
*<li>mult: Multiplicador<BR>
*<li>div: Divisor <BR>
*<li>metodo: ZERO-> los pixels que quedan fuera se ponen a 0.
*PAD->cuando el pixel est fuera, se toma el valor del pixel de la imagen
*ms cercano. P.e. (-1,-1) se tomara (0,0), (-2,2) se tomara (0,2), etc.
*WRAP-> cuando el pxel est fuera, se toma el valor del pixel opuesto en
*los ejes que corresponda. P.e. (-1,-1) se tomara (w,h), (0,-2) se tomara
*(0,h-1) etc.<BR><BR>
*</ul>
<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen convolucionada con la imagen de entrada, del mismo tipo que la imagen de entrada<BR><BR>
*</ul>
*/


public class FConvolveImage extends JIPFunction {
		/**
		*<P><FONT COLOR="red">
		*Constructor de la Clase.<BR>
		*<P><FONT COLOR="blue">		
		*Obtiene los parmetros de entrada para aplicar la funcin.
		*<ul><B>Parametros de Entrada:</B><BR>
		*<li>NINGUNO<BR><BR>
		*</ul>
		<ul><B>Parametros de Salida:</B><BR>
		*<li>NINGUNO<BR><BR>
		*</ul>
		*/

    public FConvolveImage() {
		//Nombre y descripcion de la Funcion
	    name="FConvolveImage";
	    description=
	         "Convolucion de un imagen con otra";

	    //Definicion de Parametros
		JIPParameter p1=new JIPParameter("imagen",JIP.pIMAGE,true);
    p1.setDescription("Imagen con la que convolucionar");
    JIPParameter p2=new JIPParameter("mult",JIP.pREAL,false);
    p2.setDefault((float)1.0);
    p2.setDescription("Multiplicador");
    JIPParameter p3=new JIPParameter("div",JIP.pREAL,false);
    p3.setDefault((float)1.0);
    p3.setDescription("Divisor");
    JIPParameter p4=new JIPParameter("metodo",JIP.pSTRING,false);
    p4.setDefault("ZERO");
    p4.setDescription("ZERO, WRAP, PAD");
            
            

		//Asignacion de Parametros
    params=new Vector();
    params.addElement(p1);
    params.addElement(p2);
    params.addElement(p3);
    params.addElement(p4);

    }

		/**
		*<P><FONT COLOR="red">
		*Mtodo Principal<BR>
		*<P><FONT COLOR="blue">		
		*Se le llama para convolucionar la imagen de entrada con
		*otra imagen introducida como parametro por el usuario.<BR>
		*<ul><B>Parametros de Entrada:</B><BR>
		*<li>img: Imagen a procesar<BR><BR>
		*</ul>
		<ul><B>Parametros de Salida:</B><BR>
		*<li>Imagen convolucionada con la imagen que el usuario especifica.<BR><BR>
		*</ul>
		*/

    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		
    JIPImage convo=getParamValueImg("imagen");
    
    float p2=((JIPParameter)params.elementAt(1)).getValueReal();
    float p3=((JIPParameter)params.elementAt(2)).getValueReal();
    String p4=getParamValueStr("metodo").toUpperCase();

    int t=img.getType();		
    int w=img.getWidth();
		int h=img.getHeight();
		int b=img.getNumBands();
    int cw=convo.getWidth();
    int ch=convo.getHeight();
    int ct=convo.getType();
    int radiox,radioy,diametro,inix,iniy,finx,finy;
    int max;
    switch(t) 
        {
         case JIP.tBYTE:  max=255;   break;
         case JIP.tCOLOR: max=255;   break;
         case JIP.tWORD:  max=65535; break;
         case JIP.tREAL:  max=1;     break;
         default: return null;
        }

    float sum=(float)0.0;

    res=new JIPImage(b,w,h,t);
    radiox=cw/2;
    radioy=ch/2;
    
    int x,y,cx,cy,cx2=0,cy2=0,imparX=0,imparY=0;
    if (radiox*2!=cw) imparX=1;
    if (radioy*2!=ch) imparY=1;
      
   
    
      switch(ct) 
       {
        case JIP.tBYTE:  
             int coAux1[]=convo.getAllPixel(0);
             convo=new JIPImage(cw,ch,JIP.getAllRealFromByte(coAux1));
             break;
        case JIP.tWORD:  
             int coAux2[]=convo.getAllPixel(0);
             convo=new JIPImage(cw,ch,JIP.getAllRealFromByte(coAux2));
             break;
        case JIP.tCOLOR: 
             int[] red=convo.getAllPixelRed();
             int[] green=convo.getAllPixelGreen();
             int[] blue=convo.getAllPixelBlue();

             int[] gray=new int[cw*ch];
             for(int i=0;i<cw*ch;i++)
               gray[i]=(int)Math.round(0.299*red[i]+0.587*green[i]+0.114*blue[i]);
             convo=new JIPImage(cw,ch,JIP.getAllRealFromByte(gray));
             break;
        case JIP.tREAL: 
             break;
        default: return null;
       }
    
    if (!p4.equals("PAD") && !p4.equals("WRAP"))
      {
       inix=radiox;    iniy=radioy;
       finx=w-radiox;  finy=h-radioy;
      }
    else 
      {
       inix=iniy=0;
       finx=w; finy=h;
      }
    float bmp[]=new float[w*h];
    float bin[]=new float[w*h];
    for(int nb=0;nb<b;nb++) 
      {
       if (t==JIP.tREAL) bmp=img.getAllPixelFlo(nb);
         else
          {
           int bmpAux[]=img.getAllPixel(nb);
           for (int a=0;a<w*h;a++) bmp[a]=(float)(bmpAux[a]/(float)max);
          }

           float[] con=convo.getAllPixelFlo(0);
           
           for (x=inix;x<finx;x++)
            for (y=iniy;y<finy;y++)

              {
               sum=(float)0.0;
               
               //for (cx=x-radiox;cx<=x+radiox;cx++)
                //for (cy=y-radioy;cy<=y+radioy;cy++)
               for (cx=x-radiox;cx<x+radiox+imparX;cx++)
                for (cy=y-radioy;cy<y+radioy+imparY;cy++)
               
                  {
                   if (cx<0 || cy<0 || cx>=w || cy>=h)
                    {
                     if (p4.equals("WRAP"))
                      {
                       if (cx<0)  cx2=w+cx;
                       if (cx>=w) cx2=cx-w;
                       if (cy<0)  cy2=h+cy;
                       if (cy>=h) cy2=cy-h;
                       sum+=bmp[cx2+cy2*w]*con[(radiox+(cx-x))+(radioy+(cy-y))*cw];
                      }
                     if (p4.equals("PAD"))
                      {
                       if (cx<0)  cx2=0;
                       if (cx>=w) cx2=w-1;
                       if (cy<0)  cy2=0;
                       if (cy>=h) cy2=h-1;
                       sum+=bmp[cx2+cy2*w]*con[(radiox+(cx-x))+(radioy+(cy-y))*cw];
                      }
                    }
                   else sum+=bmp[cx+cy*w]*con[(radiox+(cx-x))+(radioy+(cy-y))*cw];
                  }
                sum=(sum*p2)/p3;
                if (sum<0) sum=0;
                if (t==JIP.tREAL && sum>1) sum=1;
                bin[x+y*w]=sum;
              }
          if (t==JIP.tREAL) 
            { res.setAllPixel(nb,bin);}
          else
            {
             int binAux[]=new int[w*h];
             for (int a=0;a<w*h;a++) binAux[a]=(int)(bin[a]*(float)max+0.5);
             res.setAllPixel(nb,binAux);
            }
    }
     return res;
	}
}