package jipfunc;

import jip.*;
import java.util.Vector;
import java.util.Random;

/**
* <P><FONT COLOR="red">
*<B>FGeoToGray:</B> Convierte una imagen de tipo geomtrico a escala de grises<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Convierte los elementos geomtricos de una imagen de tipo geomtrico
*a un bitmap en escala de grises<BR>
*Aplicable a tipos: POINT, SEGMENT, POLY, EDGES<BR>
*Usa: FGrayToGray
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>gray: Tipo de gris destino: BIT, BYTE, WORD o REAL<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen deltipo especificado equivalente a la
*imagen de tipo geomtrico de entrada.<BR><BR>
*</ul>
*/


public class FGeoToGray extends JIPFunction {

      /**
      *<P><FONT COLOR="red">
      *Constructor de la Clase.<BR>
      *<P><FONT COLOR="blue">    
      *Obtiene los parmetros de entrada para aplicar la funcin.
      *<ul><B>Parametros de Entrada:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      *<ul><B>Parametros de Salida:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      */


    public FGeoToGray() {
		//Nombre y descripcion de la Funcion
	    name="FGeoToGray";
	    description=
	         "Convierte una imagen de tipo geomtrico a escala de grises";     
    
    
   	JIPParameter p1=new JIPParameter("gray",JIP.pSTRING,false);
		p1.setDefault("BYTE");
		p1.setDescription("Tipo de gris destino: BIT, BYTE, WORD o REAL");

    //Asignacion de Parametros
    params=new Vector();
		params.addElement(p1);

    }
    /**
    *<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *Este metodo es el encargado de recibir la imagen de entrada y 
    *procesar el vector de geometria que contiene.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen del tipo elegido equivalente a la
    *imagen de tipo geomtrico de entrada.<BR><BR>
    *</ul>
    */
   
    public JIPImage processImg(JIPImage img) {
    JIPImage res=null;
    JIPImage res2;
    if (img==null) return null;
    String p1=getParamValueStr("gray").toUpperCase();
    int tipo=JIP.tBYTE;
		if(p1.equals("BIT")) tipo=JIP.tBIT;
		else if(p1.equals("BYTE")) tipo=JIP.tBYTE;
		else if(p1.equals("WORD")) tipo=JIP.tWORD;
		else if(p1.equals("REAL")) tipo=JIP.tREAL;
		else return null;
   	int w=img.getWidth();
		int h=img.getHeight();
		int b=img.getNumBands();
		int t=img.getType();
    if (t==JIP.tSEGMENT) {
				res=new JIPImage(w,h,JIP.tREAL);
    		int x,y,i,x0,y0,x1,y1;
    		int pix[]=new int[w*h];
    		Vector aux=new Vector();
    		aux=(Vector) img.getVector();
    		for (i=0;i<w*h;i++) 
    				pix[i]=0;
    		int tam=aux.size()/4;
    		for (i=0;i<tam;i++) {
    				x0=((Integer)aux.elementAt(4*i)).intValue();
    				y0=((Integer)aux.elementAt(4*i+1)).intValue();
    				x1=((Integer)aux.elementAt(4*i+2)).intValue();
    				y1=((Integer)aux.elementAt(4*i+3)).intValue();
    				JIPToolkit.drawline(x0,y0,x1,y1,w,h,pix);	
    		}
     		float bmp[]=new float[w*h];      
    		for (i=0;i<w*h;i++){
       			bmp[i]=pix[i]/65535;
      	}
    		res.setAllPixel(bmp); 
    }	
    else if (t==JIP.tPOINT) {
				res=new JIPImage(w,h,JIP.tREAL);
    		int x,y,i,x0,y0;
    		int pix[]=new int[w*h];
    		Vector aux=new Vector();
    		aux=(Vector) img.getVector();
    		for (i=0;i<w*h;i++) 
    				pix[i]=0;
    		int tam=aux.size()/2;
    		for (i=0;i<tam;i++) {
    				x0=((Integer)aux.elementAt(2*i)).intValue();
    				y0=((Integer)aux.elementAt(2*i+1)).intValue();
    				JIPToolkit.drawline(x0,y0,x0,y0,w,h,pix);	
    		}
     		float bmp[]=new float[w*h];      
    		for (i=0;i<w*h;i++){
       			bmp[i]=pix[i]/65535;
      	}
    		res.setAllPixel(bmp); 

    }
    
    else if (t==JIP.tPOLY) {
				res=new JIPImage(w,h,JIP.tREAL);
    		int x,y,i,x0,y0,x1,y1,tam,xini=0,yini=0;
    		int pix[]=new int[w*h];
    		Vector aux=new Vector();
    		Vector aux2=new Vector();
    		aux=(Vector) img.getVector();
    		for (i=0;i<w*h;i++) 
    				pix[i]=0;
    		for (int kp=0;kp<aux.size();kp++) {
    			aux2=(Vector)((Vector)aux.elementAt(kp)).clone();
    			tam=aux2.size()/2;
    			for (i=0;i<tam-1;i++) {
    			  x0=((Integer)aux2.elementAt(2*i)).intValue();
    				y0=((Integer)aux2.elementAt(2*i+1)).intValue();
    				if (i==0) {
    					xini=x0;
    					yini=y0;	
    				}
    				x1=((Integer)aux2.elementAt(2*i+2)).intValue();
    				y1=((Integer)aux2.elementAt(2*i+3)).intValue();
    				JIPToolkit.drawline(x0,y0,x1,y1,w,h,pix);	
    		 }
    			
  	  		x1=((Integer)aux2.elementAt(2*tam-2)).intValue();
 					y1=((Integer)aux2.elementAt(2*tam-1)).intValue();
			  	JIPToolkit.drawline(xini,yini,x1,y1,w,h,pix);	
    		}
     		float bmp[]=new float[w*h];      
    		for (i=0;i<w*h;i++){
       			bmp[i]=pix[i]/65535;
      	}
    		res.setAllPixel(bmp); 
    }	
    
    else if (t==JIP.tEDGES) {
				res=new JIPImage(w,h,JIP.tREAL);
    		int x,y,i,x0,y0,x1,y1,tam,xini=0,yini=0;
    		int pix[]=new int[w*h];
    		Vector aux=new Vector();
    		Vector aux2=new Vector();
    		aux=(Vector) img.getVector();
    		for (i=0;i<w*h;i++) 
    				pix[i]=0;
    		for (int kp=0;kp<aux.size();kp++) {
    			aux2=(Vector)((Vector)aux.elementAt(kp)).clone();
    			tam=aux2.size()/2;
    			for (i=0;i<tam;i++) {
    			  x0=((Integer)aux2.elementAt(2*i)).intValue();
    				y0=((Integer)aux2.elementAt(2*i+1)).intValue();
    				JIPToolkit.drawline(x0,y0,x0,y0,w,h,pix);	
    		 }
    		}
     		float bmp[]=new float[w*h];      
    		for (i=0;i<w*h;i++){
       			bmp[i]=pix[i]/65535;
      	}
    		res.setAllPixel(bmp); 
    }	

    
    else return null;
       	
    JIPFunction conversion=new FGrayToGray();
    conversion.setParamValue("gray",p1);
    res2=new JIPImage(conversion.processImg(res));
		return res2;
   }
}