package jipfunc;

import jip.*;
import java.util.Vector;

    /**
    * <P><FONT COLOR="red">
    *<B>FGrad:</B> Calculo del Gradiente de una imgen<BR>
    *<FONT COLOR="blue">
    *DESCRIPCION: Calcula el gradiente de una imgen mediante la
    *resta pixel a pixel de sus filas y columnas. La resta
    *no se hace de forma consecutiva, sino que tomando como pivote
    *el pixel X, se resta x-1 con x+1. La entrada es
    *una imagen y la salida es una imagen del mismo tipo 
    *que la entrada, y con el doble de bandas (gradiente en x 
    *y gradiente en y.<BR>.
    *Aplicable a tipos: BIT, BYTE, WORD, y REAL<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen de Entrada<BR><BR>
    *</ul>
    <ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen con el doble de bandas que la original. En las bandas impares
    *se encuentra el gradiente en x de las imgenes y en las bandas pares el
    *gradiente en y.<BR><BR>
    *Notas Adicionales: No se aplica a frames geomtricos.<BR>
    */

public class FGrad extends JIPFunction {
  
  
    /**
    *<P><FONT COLOR="red">
    *Constructor de la Clase.<BR>
    *<P><FONT COLOR="blue">    
    *Obtiene los parmetros de entrada para aplicar la funcin.
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    <ul><B>Parametros de Salida:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    */
  
    public FGrad() {
		//Nombre y descripcion de la Funcion
	    name="FGrad";
	    description=
	         "Calcula el gradiente de una imagen";

		//Asignacion de Parametros
 	       params=new Vector();
	
    }
    
    /**
    *<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *Se calcula el gradiente de la imagen de entrada<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen con el doble de bandas que la original. En las bandas impares
    *se encuentra el gradiente en x de las imgenes y en las bandas pares el
    *gradiente en y.<BR><BR>
    *</ul>
    */

    
    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		if(img!=null && (img.getType()==JIP.tBIT || img.getType()==JIP.tBYTE
			                || img.getType()==JIP.tWORD || img.getType()==JIP.tREAL)) {
			int t=img.getType();			                	
			int cont=0;
			int w=img.getWidth();
			int h=img.getHeight();
			int b=img.getNumBands();
			res=new JIPImage(2*b,w,h,t);
			int[] bin=new int[w*h];
			int[] bin2=new int[w*h];
			float[] binflo=new float[w*h];
			float[] binflo2=new float[w*h];
			int[] bmp;
			float[] bmpflo;
			
			for (int z=0;z<b;z++) { // contador de bandas
  			if(t!=JIP.tREAL) {
  					bmp=img.getAllPixel(z);
  					for (int x=1;x<w-1;x++) { //eje x
		  			for (int y=1;y<h-1;y++) { //eje y
          		 bin[x+y*w]=bmp[x+1+y*w]-bmp[x-1+y*w];
			     	 	 bin2[x+y*w]=bmp[x+(y-1)*w]-bmp[x+(y+1)*w];
			  	  }
  				}  
					res.setAllPixel(z+cont,bin);
					res.setAllPixel(z+cont+1,bin2);
					cont++;
  			}
  			else {
  					bmpflo=img.getAllPixelFlo(z);
  					for (int x=1;x<w-1;x++) { //eje x
		  			for (int y=1;y<h-1;y++) { //eje y
          		 binflo[x+y*w]=bmpflo[x+1+y*w]-bmpflo[x-1+y*w];
			     	 	 binflo2[x+y*w]=bmpflo[x+(y-1)*w]-bmpflo[x+(y+1)*w];
			  	  }
  				}  
					res.setAllPixel(z+cont,binflo);
					res.setAllPixel(z+cont+1,binflo2);
					cont++;
  			}	
  				
  		}	
  			
  			
	 	}	
	 return res;
	}
}