package jipfunc;

import jip.*;
import java.util.Vector;

/**
* <P><FONT COLOR="red">
*<B>FGrayToGray:</B> Convierte una imagen de un tipo de escala de grises
*a otro tipo de escala de grises<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Convierte una imagen en escala de grises a otro tipo 
*distinto (BIT, BYTE, WORD o REAL)<BR>
*Aplicable a tipos: BIT, BYTE, WORD o REAL<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>gray : Tipo de imagen destino (BIT, BYTE, WORD, REAL)<BR><BR>
*</ul>
<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen equivalente a la imagen de entrada en otro tipo 
*de escala de grises<BR><BR>
*</ul>
*/


public class FGrayToGray extends JIPFunction {

    /**
    *<P><FONT COLOR="red">
    *Constructor de la Clase.<BR>
    *<P><FONT COLOR="blue">    
    *Obtiene los parmetros de entrada para aplicar la funcin.
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    <ul><B>Parametros de Salida:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    */


    public FGrayToGray() {
		
		//Nombre y descripcion de la Funcion
        name="FGrayToGray";
        description="Convierte imagenes de un tipo de gris a otro tipo de gris.";

        //Definicion de Parametros
		JIPParameter p1=new JIPParameter("gray",JIP.pSTRING,false);
		p1.setDefault("BYTE");
		p1.setDescription("Tipo de gris destino: BIT, BYTE, WORD o REAL");

        //Asignacion de Parametros
        params=new Vector();
		params.addElement(p1);
    }
    
    /**
    *<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *Se le llama para cambiar el tipo de la imagen de entrada, 
    *desde COLOR a escala de grises.
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar<BR><BR>
    *</ul>
    <ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen del tipo especificado por el usuario equivalente
    *a la imagen de entrada de tipo COLOR<BR><BR>
    *</ul>
    */

    public JIPImage processImg(JIPImage img) {
    	
    	//Obtenemos el tipo de gris al cual convertir		
		String p1=getParamValueStr("gray").toUpperCase();
    	int tipo=JIP.tBYTE;
		if(p1.equals("BIT")) tipo=JIP.tBIT;
		else if(p1.equals("BYTE")) tipo=JIP.tBYTE;
		else if(p1.equals("WORD")) tipo=JIP.tWORD;
		else if(p1.equals("REAL")) tipo=JIP.tREAL;
    			
		JIPImage res=null;
		if(img!=null && img.getType()!=JIP.tCOLOR && img.getType()!=JIP.tSEGMENT
       && img.getType()!=JIP.tPOINT&& img.getType()!=JIP.tPOLY&& img.getType()!=JIP.tEDGES) {
			
			int[] bmp=null;
			float[] bmpflo=null;
			
			// Obtenemos informacion de la imagen
			int nbands=img.getNumBands();
			int tipoorig=img.getType();
			int w=img.getWidth();
			int h=img.getHeight();			
			
			// Creamos la nueva imagen
			res=new JIPImage(nbands,w,h,tipo);
			
			// Para cada banda
			for(int b=0;b<nbands;b++) {

				//Obtenemos los pixels originales de la banda
				if(tipoorig!=JIP.tREAL) bmp=img.getAllPixel(b);
				else bmpflo=img.getAllPixelFlo(b);

				//Convertimos los pixels al gris destino y asignamos la banda
				switch(tipo) {
				case JIP.tBIT:
					switch(tipoorig) {
						case JIP.tBIT:  res.setAllPixel(b,bmp); break;
						case JIP.tBYTE: res.setAllPixel(b,JIP.getAllBitFromByte(bmp)); break;
						case JIP.tWORD: res.setAllPixel(b,JIP.getAllBitFromWord(bmp)); break;
						case JIP.tREAL: res.setAllPixel(b,JIP.getAllBitFromReal(bmpflo)); break;
					}
					break;
				case JIP.tBYTE:
					switch(tipoorig) {
						case JIP.tBIT:  res.setAllPixel(b,JIP.getAllByteFromBit(bmp)); break;
						case JIP.tBYTE: res.setAllPixel(b,bmp); break;
						case JIP.tWORD: res.setAllPixel(b,JIP.getAllByteFromWord(bmp)); break;
						case JIP.tREAL: res.setAllPixel(b,JIP.getAllByteFromReal(bmpflo)); break;
					}
					break;
				case JIP.tWORD:
					switch(tipoorig) {
						case JIP.tBIT:  res.setAllPixel(b,JIP.getAllWordFromBit(bmp)); break;
						case JIP.tBYTE: res.setAllPixel(b,JIP.getAllWordFromByte(bmp)); break;
						case JIP.tWORD: res.setAllPixel(b,bmp); break;
						case JIP.tREAL: res.setAllPixel(b,JIP.getAllWordFromReal(bmpflo)); break;
					}
					break;
				case JIP.tREAL:
					switch(tipoorig) {
						case JIP.tBIT:  res.setAllPixel(b,JIP.getAllRealFromBit(bmp)); break;
						case JIP.tBYTE: res.setAllPixel(b,JIP.getAllRealFromByte(bmp)); break;
						case JIP.tWORD: res.setAllPixel(b,JIP.getAllRealFromWord(bmp)); break;
						case JIP.tREAL: res.setAllPixel(b,bmpflo); break;
					}
					break;
	           }
           }
		}
		
		return res;
	}
}