package jipfunc;

import jip.*;
import java.util.Vector;
import java.util.Random;

/**
* <P><FONT COLOR="red">
*<B>FInterSegment:</B> Calcula las intersecciones de los segmentos.<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Calcula el punto de interseccin de los segmentos de la imagen.
*La imagen debe ser de tipo Segmento.<BR>
*Aplicable a tipos: SEGMENT<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen de tipo Punto con los puntos de interseccin.<BR><BR>
*</ul>
*/
public class FInterSegment extends JIPFunction {

	  /** Vector que contiene los segmentos de la imagen */
		Vector Segmentos;
		/** Vector donde almacenaremos los puntos ya creados */
		Vector Puntos;
    /** Variable donde devolvemos la coordenada X de la interseccin */	
		int devx;
		/** Variable donde devolvemos la coordenada Y de la interseccin */	
		int devy;

    /**
    *<P><FONT COLOR="red">
    *Constructor de la Clase.<BR>
    *<P><FONT COLOR="blue">    
    *Obtiene los parmetros de entrada para aplicar la funcin.
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    */

    public FInterSegment() {
		//Nombre y descripcion de la Funcion
	    name="FInterSegment";
	    description="Calculo de Interseccin de Segmentos";
      params=new Vector();
	          
    }

    /**
    *<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *Se llama a este mtodo para calcular los puntos de interseccion de 
    *los segmentos de la imagen de entrada (que es de tipo Segmento).<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen de tipo Punto con los puntos de interseccin.<BR><BR>
    *</ul>
    */
   
    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		if (img==null) return null;
		int ax0,ay0,ax1,ay1,sx0,sy0,sx1,sy1;
    int w=img.getWidth();
		int h=img.getHeight();
		int t=img.getType();
    if (t!=JIP.tSEGMENT) return null;
    Segmentos=new Vector();
    Segmentos=img.getVector();
    Puntos=new Vector();
    Integer aux;
    int tam=Segmentos.size();
		for (int i=0;i<tam/4;i++) {
    		ax0=((Integer)(Segmentos.elementAt(4*i))).intValue();
    	  ay0=((Integer)(Segmentos.elementAt(4*i+1))).intValue();
    		ax1=((Integer)(Segmentos.elementAt(4*i+2))).intValue();
    		ay1=((Integer)(Segmentos.elementAt(4*i+3))).intValue();
    		for (int j=i+1;j<tam/4;j++) {
						sx0=((Integer)(Segmentos.elementAt(4*j))).intValue();
    			  sy0=((Integer)(Segmentos.elementAt(4*j+1))).intValue();
    				sx1=((Integer)(Segmentos.elementAt(4*j+2))).intValue();
    				sy1=((Integer)(Segmentos.elementAt(4*j+3))).intValue();
  					if (InterseccionPropia(ax0,ay0,ax1,ay1,sx0,sy0,sx1,sy1)) {
  							PuntoCorte(ax0,ay0,ax1,ay1,sx0,sy0,sx1,sy1);
  							aux=new Integer(devx);
  							Puntos.addElement(aux);
  							aux=new Integer(devy);
  							Puntos.addElement(aux);
  					}
    		}
    }
    
    res=new JIPImage(w,h,JIP.tPOINT,Puntos);
		return res;
	}
	
	/**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo para calcular el punto de corte de dos segmentos. Antes de llamar
    *a ste mtodo hay que comprobar que realmente existe ese punto de corte.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>ax1: Coordenada X del Punto Inicial del Primer Segmento<BR>
    *<li>ay1: Coordenada Y del Punto Inicial del Primer Segmento<BR>
		*<li>bx1: Coordenada X del Punto Final del Primer Segmento<BR>
		*<li>by1: Coordenada Y del Punto Final del Primer Segmento<BR>
		*<li>cx1: Coordenada X del Punto Inicial del Segundo Segmento<BR>
		*<li>cy1: Coordenada Y del Punto Inicial del Segundo Segmento<BR>
		*<li>dx1: Coordenada X del Punto Final del Segundo Segmento<BR>
		*<li>dy1: Coordenada Y del Punto Final del Segundo Segmento<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>devx: Coloca en la variable de clase la coordenada x del punto
    * perteneciente a la interseccin<BR>
    *<li>devy: Coloca en la variable de clase la coordenada y del punto
    * perteneciente a la interseccin<BR><BR>
    *</ul>
    */

	
	void  PuntoCorte (int ax1, int ay1,int bx1,int by1,int cx1,int cy1, int dx1,int dy1){
  
  double m1,n1,m2,n2,xx,yy;
  double ax,ay,bx,by,cx,cy,dx,dy;

  ax = (double) ax1;
  ay = (double) ay1;
  bx = (double) bx1;
  by = (double) by1;
  cx = (double) cx1;
  cy = (double) cy1;
  dx = (double) dx1;
  dy = (double) dy1;

  if ((ax-bx)==0)
  {
   xx = ax;
   m2=(cy-dy)/(cx-dx);
   n2=cy-((cy-dy)/(cx-dx))*cx;
   yy = m2*ax+n2;
  }
  else if ((cx-dx)==0)
  {
   xx = cx;
   m1=(ay-by)/(ax-bx);
   n1=ay-((ay-by)/(ax-bx))*ax;
   yy = m1*dx+n1;
  }
  else
  {
   m1=(ay-by)/(ax-bx);
   n1=ay-((ay-by)/(ax-bx))*ax;
   m2=(cy-dy)/(cx-dx);
   n2=cy-((cy-dy)/(cx-dx))*cx;

   xx = (n2-n1)/(m1-m2);
   yy = m1*xx+n1; 
  }

  devx= (int) xx;
  devy= (int) yy;
  
  
 }


    /**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo que nos dice si un punto esta a la izquierda de otros dos.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>ax: Coordenada X del Punto 1<BR>
    *<li>ay: Coordenada Y del Punto 1<BR>
		*<li>bx: Coordenada X del Punto 2<BR>
		*<li>by: Coordenada Y del Punto 2<BR>
		*<li>cx: Coordenada X del Punto 3<BR>
		*<li>cy: Coordenada Y del Punto 3<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>return: Devuelve True si el punto est a la izquierda<BR><BR>
    *</ul>
    */
	
	private boolean left(int ax,int ay,int bx,int by, int cx, int cy){
		return area2(ax,ay,bx,by,cx,cy) > 0;
	}

	

  	/**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo para calcular si dos segmentos se intersectan entre si.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>ax0: Coordenada X del Punto Inicial del Primer Segmento<BR>
    *<li>ay0: Coordenada Y del Punto Inicial del Primer Segmento<BR>
		*<li>ax1: Coordenada X del Punto Final del Primer Segmento<BR>
		*<li>ay1: Coordenada Y del Punto Final del Primer Segmento<BR>
		*<li>sx0: Coordenada X del Punto Inicial del Segundo Segmento<BR>
		*<li>sy0: Coordenada Y del Punto Inicial del Segundo Segmento<BR>
		*<li>sx1: Coordenada X del Punto Final del Segundo Segmento<BR>
		*<li>sy1: Coordenada Y del Punto Final del Segundo Segmento<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>return: Devuelve true si los segmentos se intersectan.<BR><BR>
    *</ul>
    */



private boolean InterseccionPropia(int ax0,int ay0,int ax1,int ay1,int sx0,int sy0,int sx1,int sy1) {

		// Quito los casos dejenerados
		if(colinear(ax0,ay0,ax1,ay1,sx0,sy0) || colinear(ax0,ay0,ax1,ay1,sx1,sy1) ||
		    colinear(sx0,sy0,sx1,sy1,ax0,ay0) || colinear(sx0,sy0,sx1,sy1,ax1,ay1))
			  return false;
		
		return xor(left(ax0,ay0,ax1,ay1,sx0,sy0), left(ax0,ay0,ax1,ay1,sx1,sy1)) &&
		       xor(left(sx0,sy0,sx1,sy1,ax0,ay0), left(sx0,sy0,sx1,sy1,ax1,ay1));
	}
	
    /**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo que realiza la funcin XOR entre dos booleanos<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>a: Booleano 1<BR>
    *<li>b: Booleano 2<BR><BR>
		*</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>return: Devuelve True o False dependiendo del resultado de la funcin XOR<BR><BR>
    *</ul>
    */

	private boolean xor(boolean a, boolean b){
		return (!a&&b) || (a&&!b);
	}
	
    /**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo que nos indica si tres puntos son colineales<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>p1x: Coordenada X del Punto 1<BR>
    *<li>p1y: Coordenada Y del Punto 1<BR>
		*<li>p2x: Coordenada X del Punto 2<BR>
		*<li>p2y: Coordenada Y del Punto 2<BR>
		*<li>p3x: Coordenada X del Punto 3<BR>
		*<li>p3y: Coordenada Y del Punto 3<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>return: Devuelve True si los puntos son colineales<BR><BR>
    *</ul>
    */

	
	private boolean colinear(int p1x,int p1y,int p2x,int p2y,int p3x,int p3y){
		return area2(p1x,p1y,p2x,p2y,p3x,p3y) == 0;
	}
	
	  /**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo que devuelve el rea (con signo) del tringulo formado por los tres puntos<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>ax: Coordenada X del Punto 1<BR>
    *<li>ay: Coordenada Y del Punto 1<BR>
		*<li>bx: Coordenada X del Punto 2<BR>
		*<li>by: Coordenada Y del Punto 2<BR>
		*<li>cx: Coordenada X del Punto 3<BR>
		*<li>cy: Coordenada Y del Punto 3<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>return: Devuelve el rea del tringulo<BR><BR>
    *</ul>
    */

	private int area2(int ax,int ay,int bx,int by,int cx,int cy){
		return
			ax * by - ay * bx +
			ay * cx - ax * cy +
			bx * cy - cx * by;
	}

}