package jipfunc;

import jip.*;
import java.util.Vector;

/**
* <P><FONT COLOR="red">
*<B>FLink:</B> Crea un conjunto de Edges a partir de una imagen<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Crea un conjunto de puntos, los cualse son devueltos en
*una imagen de tipo Edge. Se utiliza principalmente con imagenes resultado
*de aplicar el mtodo de Canny para la deteccin de bordes, con lo cual
*obtendremos los puntos que forman esos bordes.<BR>
*Aplicable a tipos: BYTE, WORD<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>low: Umbral Inferior (En Porcentaje)<BR>
*<li>high: Umbral Superior (En Porcentaje)<BR>
*<li>minlengh: Tamao Mnimo de Puntos del Edge a incluir<BR><BR>
*</ul>
<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen de tipo Edge correspondiente a los puntos de la imagen<BR><BR>
*</ul>
*/

public class FLink extends JIPFunction {
	
      /** Vector de puntos del Edge Actual*/ 
	    Vector PointData;
      /** Vector que incluye todos los Edges resultado */
	    Vector Edges;
      /** Numero de puntos en el Edge Actual */
      int PointCount;	
      /** Filas de la Imagen*/  
		  int ImageRows;
      /** Columnas de la Imagen*/
      int ImageColumns;
      /** Umbral Inferior*/
      int LowUmbral;
      /** Umbral Superior*/
      int HighUmbral;
      /** Longitud Mnima del Edge*/
      int MinLong;
      /** Puntero a Fila utilizado para el prximo punto*/
		  int rp;
      /** Puntero a Columna utilizado para el prximo punto*/
      int cp;
      /** Imagen con la que vamos a operar*/
		  int []bmp; 
      /** Aqui guardamos la imagen original */
		  int []bmpOriginal;
	
  /**
  *<P><FONT COLOR="red">
  *Constructor de la Clase.<BR>
  *<P><FONT COLOR="blue">    
  *Obtiene los parmetros de entrada para aplicar la funcin.
  *<ul><B>Parametros de Entrada:</B><BR>
  *<li>NINGUNO<BR><BR>
  *</ul>
  <ul><B>Parametros de Salida:</B><BR>
  *<li>NINGUNO<BR><BR>
  *</ul>
  */
	  
		  	
public FLink() 
  {
	//Nombre y descripcion de la Funcion
	 name="FLink";
   description="Crea un conjunto de Edges a partir de una imagen. Solo se procesa la banda 0";
   JIPParameter p1=new JIPParameter("low",JIP.pINT,false);
   p1.setDefault(30);
   p1.setDescription("Umbral Inferior (En Porcentaje)");
   JIPParameter p2=new JIPParameter("high",JIP.pINT,false);
   p2.setDefault(45);
   p2.setDescription("Umbral Superior (En Porcentaje)");
   JIPParameter p3=new JIPParameter("minlengh",JIP.pINT,false);
   p3.setDefault(2);
   p3.setDescription("Tamao Mnimo de Puntos del Edge a incluir");

   //Asignacion de Parametros
   params=new Vector();
   params.addElement(p1);  
   params.addElement(p2);	
   params.addElement(p3);
  }
   
 /**
  *<P><FONT COLOR="red">
  *Mtodo Principal<BR>
  *<P><FONT COLOR="blue">    
  *Se le llama para calcular los Edges a partir de la imagen de
  *entrada.<BR>
  *<ul><B>Parametros de Entrada:</B><BR>
  *<li>img: Imagen a procesar<BR><BR>
  *</ul>
  *<ul><B>Parametros de Salida:</B><BR>
  *<li>Imagen de tipo Edge correspondiente a los Edges encontrados
  * en la imagen de entrada.<BR><BR>
  *</ul>
  */
   
 public JIPImage processImg(JIPImage img) 
   {
	JIPImage res=null;
	if(img!=null && (img.getType()==JIP.tBYTE || img.getType()==JIP.tWORD || img.getType()==JIP.tREAL)) {
		int max,x,y,auxint=0;
		int r,c;
    Edges=new Vector();
    PointData=new Vector();
    PointCount = 0;
    
 		int t=img.getType();
		int cont=0;
		int w=img.getWidth();
		int h=img.getHeight();
		int b=img.getNumBands();
		LowUmbral=((JIPParameter)params.elementAt(0)).getValueInt();
    HighUmbral=((JIPParameter)params.elementAt(1)).getValueInt();
    MinLong=((JIPParameter)params.elementAt(2)).getValueInt();
    ImageRows=h;
    ImageColumns=w;
    
    switch(t) {
        case JIP.tBYTE:  
        			max=255;
        			LowUmbral=LowUmbral*255/100;
        			HighUmbral=HighUmbral*255/100;
        		  break;
        case JIP.tWORD:  
        			max=65535; 
        			LowUmbral=LowUmbral*65535/100;
        			HighUmbral=HighUmbral*65535/100;
        			break;
        case JIP.tREAL:
        			JIPFunction gray=new FGrayToGray();
        			gray.setParamValue("gray","BYTE");
      				img=gray.processImg(img); // Convertimos a BYTE
    }
    

	  bmp=img.getAllPixel(0);
    bmpOriginal=img.getAllPixel(0);     
    for (r = 0; r < ImageRows; r++)
				for (c = 0; c < ImageColumns; c++) {
    
	    		if (bmp[c+r*ImageColumns] >= HighUmbral) {
	  			  	this.TrackEdge(r, c);
	    		}
         
				}
  		PointData.removeAllElements();
    
    	/* La imagen resultado ser una imagen de tipo EDGES con los edges 
    	   que se hayan encontrado */
    	
    	res=new JIPImage(w,h,JIP.tEDGES,Edges);
	}
	return res;
}


  /**
  *<P><FONT COLOR="red">
  *DESCRIPCIN<BR>
  *<P><FONT COLOR="blue">    
  *Tracea un Edge a partir de la posicin que se le pasa
  *como argumento.Por cada punto que se aade al Edge,
  *se coloca un cero en la imagen original. Despus se le da la vuelta
  *al array y se comienza a tracear en la siguiente direccin.<BR>
  *<ul><B>Parametros de Entrada:</B><BR>
  *<li>row: Fila en la que comienza<BR>
  *<li>col: Columna en la que comienza<BR><BR>
  *</ul>
  *<ul><B>Parametros de Salida:</B><BR>
  *<li>Introduce el Edge en la lista de Edges (Variable de la Clase)<BR><BR>
  *</ul>
  */
 

 public void TrackEdge(int row, int col) {
 	
 	rp=row;
  cp=col;
  
  this.StorePoint(rp,cp);

  
  	while ( NextPoint() ) {
  		this.StorePoint(rp,cp);
  	}
  
	if (PointData.size()/2>=MinLong) {
		  Vector Aux=new Vector();
		  Aux=(Vector)PointData.clone();
			Edges.addElement(Aux);
		
	}
	PointData.removeAllElements();
 }	

  /**
  *<P><FONT COLOR="red">
  *DESCRIPCIN<BR>
  *<P><FONT COLOR="blue">    
  *Almacena el punto del Edge en la variable de la clase que 
  *contiene la lista de Edges temporal.<BR>
  *<ul><B>Parametros de Entrada:</B><BR>
  *<li>r: Fila a almacenar<BR>
  *<li>c: Columna a almacenar<BR><BR>
  *</ul>
  *<ul><B>Parametros de Salida:</B><BR>
  *<li>Introduce el punto en la lista temporal de Edges<BR><BR>
  *</ul>
  */
 


  public void StorePoint(int r, int c) {
  		
  	 	Integer entero;
      entero=new Integer(c);
  		PointData.addElement(entero);
  		entero=new Integer(r);
  		PointData.addElement(entero);
 
   		bmp[c+r*ImageColumns]=0;
	}
 	
 /**
  *<P><FONT COLOR="red">
  *DESCRIPCIN<BR>
  *<P><FONT COLOR="blue">    
  *Calcula el sigiguiente punto para seguir con el traceo.<BR>
  *<ul><B>Parametros de Entrada:</B><BR>
  *<li>rp: Fila por la que empieza<BR>
  *<li>cp: Columna por la que empieza<BR><BR>
  *</ul>
  *<ul><B>Parametros de Salida:</B><BR>
  *<li>Introduce el punto en la lista temporal de Edges<BR><BR>
  *</ul>
  *Notas Adicionales: rp y cp son las variables de la clase, utilizadas
  *desde TraceEdge para pasar los valores por referencia.
  */

  public boolean NextPoint() {
  	{ 
  		int i,r,c;
      int [] roff=new int [8];
      int [] coff=new int [8];
      roff[0]=1;
      roff[1]=0;
      roff[2]=-1;
      roff[3]=0;
      roff[4]=1;
      roff[5]=1;
      roff[6]=-1;
      roff[7]=-1;
      coff[0]=0;
      coff[1]=1;
      coff[2]=0;
      coff[3]=-1;
      coff[4]=1;
      coff[5]=-1;
      coff[6]=-1;
      coff[7]=1;
  
 			for (i = 0; i < 8; i++) {
					r = rp + roff[i];
					c = cp + coff[i];
					if (r >= 0  &&  c >= 0  &&  r < ImageRows  &&  c < ImageColumns)
	    			if ((int) bmp[c+r*ImageColumns] >= LowUmbral) {
								rp += roff[i];
								cp += coff[i];
   						return true;
	    			}
    	}	
   
			return false;

  	}
  } // DE LA FUNCION NEXPOINT()

}