package jipfunc;

import jip.*;
import java.util.Vector;
import java.util.Random;

/**
* <P><FONT COLOR="red">
*<B>FMinimum:</B> Reemplaza el valor de cada pixels por un mnimo local<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Reemplaza el valor de la intensidad por el menor valor en una 
*ventana de vecindad del tamao especificado.<BR>
*Aplicable a tipos: BIT, BYTE, WORD, COLOR y REAL<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>radio: Radio de la Ventana de Vecindad<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen Procesada del mismo tipo que la imagen de entrada<BR><BR>
*</ul>
*Notas Adicionales: No se aplica a frames geomtricos.<BR>
**/


public class FMinimum extends JIPFunction {

    /**
    *<P><FONT COLOR="red">
    *Constructor de la Clase.<BR>
    *<P><FONT COLOR="blue">    
    *Obtiene los parmetros de entrada para aplicar la funcin.
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    */


    public FMinimum() {
		//Nombre y descripcion de la Funcion
	    name="FMinimum";
	    description=
	         "Reemplaza el valor de la intensidad por el menor valor en una vecindad del tamao especificado";

	    //Definicion de Parametros
		JIPParameter p1=new JIPParameter("radio",JIP.pINT,false);
		p1.setDefault(2);
		p1.setDescription("Radio ");

		//Asignacion de Parametros
        params=new Vector();
		params.addElement(p1);

    }

    /**<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *Se le llama para sustituir el valor de cada pixel de la imagen por
    *el mnimo local en una ventana de vecindad.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen Procesada del mismo tipo que la imagen de entrada<BR><BR>
    *</ul>
    */


    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		int p1=((JIPParameter)params.elementAt(0)).getValueInt();
		Random rnd=new Random();
                int t=img.getType();		
                int w=img.getWidth();
		int h=img.getHeight();
		int b=img.getNumBands();
		int max;
    /** FILTRAMOS LA GEOMETRIA */   
    if (t!=JIP.tBIT && t!=JIP.tBYTE && t!=JIP.tWORD
        && t!=JIP.tREAL && t!=JIP.tCOLOR) 
          
          return res;

    res=new JIPImage(b,w,h,t);
		for(int nb=0;nb<b;nb++) 
                   {
		    if (t!=JIP.tREAL)
		      {
		 	int[] bmp=img.getAllPixel(nb);
	    	    	int[] bin=new int[w*h];
		    	for(int i=0; i<h; i++)
		    	  for(int j=0; j<w; j++)
			   {
			    int inicioX=j-p1, finX=j+p1;
			    int inicioY=i-p1, finY=i+p1;
			    if(inicioX<0) inicioX=0 ;  if(finX>=w) finX=w-1 ;
			    if(inicioY<0) inicioY=0 ;  if(finY>=h) finY=h-1 ;
			    int aux=65535;
			    int cont=0;
			    for (int y=inicioY; y<=finY; y++)
			        for (int x=inicioX; x<=finX; cont++, x++)
				    if (aux>bmp[x+y*w]) aux=bmp[x+y*w];
 			    bin[j+i*w]=aux;
			   }
		    	res.setAllPixel(nb,bin);
		      }
		   else
		      {
		 	float[] bmp=img.getAllPixelFlo(nb);
	    	    	float[] bin=new float[w*h];
		    	for(int i=0; i<h; i++)
		    	  for(int j=0; j<w; j++)
			   {
			    int inicioX=j-p1, finX=j+p1;
			    int inicioY=i-p1, finY=i+p1;
			    if(inicioX<0) inicioX=0 ;  if(finX>=w) finX=w-1 ;
			    if(inicioY<0) inicioY=0 ;  if(finY>=h) finY=h-1 ;
			    float aux=1;
			    int cont=0;
			    for (int y=inicioY; y<=finY; y++)
			        for (int x=inicioX; x<=finX; cont++, x++)
				    if (aux>bmp[x+y*w]) aux=bmp[x+y*w];				   
 			    bin[j+i*w]=aux;
			   }
		    	res.setAllPixel(nb,bin);
		      }
                   }
	
		return res;
	}
}