package jipfunc;

import jip.*;
import java.util.Vector;

/**
* <P><FONT COLOR="red">
*<B>FMirror:</B> Inversin de Pixels con respecto al eje Vertical.<BR>
*<FONT COLOR="blue">
*DESCRIPCION:Realiza la inversion de los pixels de la imagen y/o la inversion de coordenadas
*geometricas respecto al eje vertical.<BR>
*Aplicable a tipos: BIT, BYTE, WORD, COLOR, REAL, POINT, SEGMENT, POLY, EDGE <BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen con la inversin de pixels, con respecto al eje vertical.<BR><BR>
*</ul>
*/

public class FMirror extends JIPFunction {
  
    
    /**
    *<P><FONT COLOR="red">
    *Constructor de la Clase.<BR>
    *<P><FONT COLOR="blue">    
    *Obtiene los parmetros de entrada para aplicar la funcin.
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    <ul><B>Parametros de Salida:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    */

    public FMirror() {
		//Nombre y descripcion de la Funcion
	    name="FMirror";
	    description="Realiza el reflejo de una imagen";

	    //Definicion de Parametros
	    //Sin Parametros

		//Asignacion de Parametros
        params=new Vector();
    }
    
     /**
    *<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *En este metodo se realizan los calculos de inversion.
    *Segun el tipo de imagen a tratar
    *se genera una imagen resultado u otra. Esta funcion gestiona datos geometricos si estos se
    *presentan.
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen procesada del mismo tipo que la imagen de entrada<BR><BR>
    *</ul>
    */

    
    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		if(img!=null) {
			int w=img.getWidth();
			int h=img.getHeight();
			int b=img.getNumBands();
			int t=img.getType();
			int ip;
			float fp;
      
      if (img.getType()!=JIP.tPOINT && img.getType()!=JIP.tSEGMENT &&
          img.getType()!=JIP.tPOLY && img.getType()!=JIP.tEDGES){  
      
        res=new JIPImage(b,w,h,t);
        for(int nb=0;nb<b;nb++)
          for(int y=0;y<h;y++)
            for(int x=0;x<w;x++) {
              if(t!=JIP.tREAL) {
                ip=(int)img.getPixel(nb,x,y);
                res.setPixel(nb,x,y,img.getPixel(nb,w-x-1,y));
                res.setPixel(nb,w-x-1,y,ip);
              }
              else {
                fp=(float)img.getPixelFlo(nb,x,y);
                res.setPixel(nb,x,y,img.getPixelFlo(nb,w-x-1,y));
                res.setPixel(nb,w-x-1,y,fp);
              }
            }
          }
      else{
        if (img.getType()==JIP.tPOINT || img.getType()==JIP.tSEGMENT){
          // puntos o segmentos
          Vector geodata=(Vector)(img.getVector()).clone();
          Integer aux;
          for(int i=0;i<geodata.size();i+=2){  // solo cojo las coordenadas en X
            aux=new Integer(w-1-((Integer)geodata.elementAt(i)).intValue());            
            geodata.setElementAt(aux,i);
          }
          res=new JIPImage(w,h,t,geodata);
        }
        else{
          if (img.getType()==JIP.tPOLY || img.getType()==JIP.tEDGES){
            // poligonos o edges            
              Vector geodata=new Vector();
              for (int k=0; k < img.getVector().size();k++){
                geodata.addElement(((Vector)(((Vector)(((Vector)((Vector)(img.getVector())).elementAt(k))))).clone()));
              }
              for (int j=0; j<geodata.size();j++){
                for (int i=0;i<((Vector)(geodata.elementAt(j))).size();i+=2){ // solo las X
                  ((Vector)(geodata.elementAt(j))).setElementAt(
                    new Integer((int)(w-1-(double)(((Integer)(((Vector)(geodata.elementAt(j))).elementAt(i))).intValue())))
                    ,i);    
                }
              }
              res=new JIPImage(w,h,t,geodata);
          }
        }
      }
		}
		return res;
	}
}