package jipfunc;

import jip.*;
import java.util.Vector;
import java.util.Random;

/**
* <P><FONT COLOR="red">
*<B>FNoise:</B> Introduccion de ruido aleatorio<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Introduce ruido aleatorio dependiendo del
*nivel de ruido que el usuario especifique.<BR>
*<B> Buena deteccin: </B>
*Aplicable a tipos: BIT, BYTE, WORD, COLOR y REAL<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>nivel: Nivel de ruido (0 no altera la imagen)<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen original con el ruido aadido.<BR><BR>
*</ul>
*/


public class FNoise extends JIPFunction {

    /**
    *<P><FONT COLOR="red">
    *Constructor de la Clase.<BR>
    *<P><FONT COLOR="blue">    
    *Obtiene los parmetros de entrada para aplicar la funcin.
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    */


    public FNoise() {
		//Nombre y descripcion de la Funcion
	    name="FNoise";
	    description=
	         "Introduccion de ruido aleatorio";

	    //Definicion de Parametros
		JIPParameter p1=new JIPParameter("nivel",JIP.pINT,false);
		p1.setDefault(20);
		p1.setDescription("Nivel de ruido (0 no altera la imagen)");

		//Asignacion de Parametros
        params=new Vector();
		params.addElement(p1);

    }

    /**
    *<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *Se le llama para aadir ruido aleatorio a la imagen.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen original con el ruido aadido.<BR><BR>
    *</ul>
    */

    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		int p1=((JIPParameter)params.elementAt(0)).getValueInt();
		Random rnd=new Random();
                int t=img.getType();		
                int w=img.getWidth();
		int h=img.getHeight();
		int b=img.getNumBands();
		int max;
   	res=new JIPImage(b,w,h,t);
    switch(t) {
				case JIP.tBYTE:  max=255;   break;
				case JIP.tWORD:  max=65535; break;
				case JIP.tCOLOR: max=255;   break;
				case JIP.tREAL:  max=1;   break;
				default: return null;
			}
		float []aleatorio=new float[w*h];
				// Guardamos los valores para no alterar el color
                for (int i=0;i<w*h;i++)
		aleatorio[i]=(float)(rnd.nextGaussian()*p1*max)/1000;	
		for(int nb=0;nb<b;nb++) 
                   {
		    if (t!=JIP.tREAL)
		       {
		    	int[] bmp=img.getAllPixel(nb);
		    	int[] bin=new int[w*h];
		    	for(int k=0;k<w*h;k++)
                        	bin[k]=(int)Math.min(bmp[k]+aleatorio[k],max);
  		        res.setAllPixel(nb,bin);
		       }
		    else
                       {
		    	float[] bmp=img.getAllPixelFlo(nb);
		    	float[] bin=new float[w*h];
		    	for(int k=0;k<w*h;k++)
                        	bin[k]=(float)Math.min(bmp[k]+aleatorio[k],max);
			res.setAllPixel(nb,bin);
		       }
                
		   }
		
		return res;
	}
}