package jipfunc;

import jip.*;
import java.util.Vector;
import java.math.*;

/**
* <P><FONT COLOR="red">
*<B>FOp:</B> Operaciones aritmticas<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Realiza operaciones aritmeticas entre los pixels
*de entrada y un valor u otra imagen<BR>
*Aplicable a tipos: BIT, BYTE, WORD, COLOR y REAL<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>op: Operador a utilizar.<BR>
*<li>value: Constante Operando, en caso de utilizar un valor fijo como 
*segundo operando.<BR>
*<li>max: Cota mxima de representacin <BR>
*<li>min: Cota mnima de representacin <BR>
*<li>idem: Si se marca esta opcin, se utilizar la imagen de entrada
*como segundo operando<BR>
*<li>imgOp: Imagen que se utilizar como segundo operando 
*si no es un valor fijo.<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen Procesada segn el operando escogido<BR><BR>
*</ul>
*/


public class FOp extends JIPFunction {
		/**
		*<P><FONT COLOR="red">
		*Constructor de la Clase.<BR>
		*<P><FONT COLOR="blue">		
		*Obtiene los parmetros de entrada para aplicar la funcin.
		*<ul><B>Parametros de Entrada:</B><BR>
		*<li>NINGUNO<BR><BR>
		*</ul>
		<ul><B>Parametros de Salida:</B><BR>
		*<li>NINGUNO<BR><BR>
		*</ul>
		*/

    public FOp() {
		//Nombre y descripcion de la Funcion
	    name="FOp";
	    description="Realiza operaciones aritmeticas entre los pixels de entrada y un valor u otra imagen.";

	    //Definicion de Parametros
	    JIPParameter p1=new JIPParameter("op",JIP.pINT,false);
			p1.setDescription("Operador: 1 exp, 2 log, 3 sqrt, 4 square, 5 add, 6 and, 7 dist, 8 div, 9 max, 10 min, 11 mult, 12 or, 13 sub, 14 xor");
			p1.setDefault(1);
			
			JIPParameter p2=new JIPParameter("value",JIP.pREAL,false);
			p2.setDescription("Constante operando. Si opta por este operando no utilice una segunda imagen operando. Este es un valor decimal.");
			p2.setDefault(0);
			
			JIPParameter p3=new JIPParameter("max",JIP.pINT,false);
			p3.setDescription("Cota maxima de representacion. Este valor es dependiente del tipo de imagen.");
			p3.setDefault(255);

			JIPParameter p4=new JIPParameter("min",JIP.pINT,false);
			p4.setDescription("Cota minima de representacion. Este valor es dependiente del tipo de imagen.");
			p4.setDefault(0);
			
			JIPParameter p5=new JIPParameter("idem",JIP.pINT,false);
			p5.setDescription("Utilizar la misma imagen como 2 operando.");
			p5.setDefault(0);
			
			JIPParameter p6=new JIPParameter("imgOp",JIP.pSTRING,false);
			p6.setDescription("Segunda imagen operando. Ha de ser un fichero JIP.");
			p6.setDefault("");
			
		//Asignacion de Parametros
      params=new Vector();
   		params.addElement(p1);
			params.addElement(p2);
			params.addElement(p3);
			params.addElement(p4);
			params.addElement(p5);
			params.addElement(p6);

    }
    
    /**
		*<P><FONT COLOR="red">
		*Mtodo Principal<BR>
		*<P><FONT COLOR="blue">		
		*Se le llama para procesar la imgen segn el operador escogido<BR>
		*<ul><B>Parametros de Entrada:</B><BR>
		*<li>img: Imagen a procesar<BR><BR>
		*</ul>
		*<ul><B>Parametros de Salida:</B><BR>
		*<li>Imagen Procesada segn el operando escogido<BR><BR>
		*</ul>
		*/

    
    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		JIPImage aux=null;
    int p1=((JIPParameter)params.elementAt(0)).getValueInt();
    float p2=((JIPParameter)params.elementAt(1)).getValueReal();
    int p3=((JIPParameter)params.elementAt(2)).getValueInt();
    int p4=((JIPParameter)params.elementAt(3)).getValueInt();
    int p5=((JIPParameter)params.elementAt(4)).getValueInt();
    String p6=((JIPParameter)params.elementAt(5)).getValueStr();

		if(img!=null) {
			int w=img.getWidth();
			int h=img.getHeight();
			int b=img.getNumBands();
			int t=img.getType();
			int maximo=1;
			int ip;
			float fp;
      
      if (t==JIP.tBIT){
        if (p3>1 || p3<0){
          System.out.println("Error, maximo incorrecto.");
          return(res);
        }
        if (p4>1 || p4<0){
          System.out.println("Error, minimo incorrecto.");
          return(res);
        }
        if (p2!=0 && p2!=1){
        	System.out.println("Error, la constante operando ha de ser binaria.");
          return(res);	
        }
      }
      
      if (t==JIP.tCOLOR || t==JIP.tBYTE){
        if (p3>255 || p3<0){
          System.out.println("Error, maximo incorrecto.");
          return(res);
        }
        if (p4>255 || p4<0){
          System.out.println("Error, minimo incorrecto.");
          return(res);
        }
       }
      
      if (t==JIP.tWORD){
        if (p3>65536 || p3<0){
          System.out.println("Error, maximo incorrecto.");
          return(res);
        }
        if (p4>65536 || p4<0){
          System.out.println("Error, minimo incorrecto.");
          return(res);
        }
        if (p2<0){
        	System.out.println("Error, la constante operando ha de ser positiva.");
          return(res);	
        }

      }
      
      if (t==JIP.tPOINT || t==JIP.tPOLY || t==JIP.tSEGMENT || t==JIP.tEDGES){
      	System.out.println("Error, Fop no maneja datos geometricos.");
      	return(null);      	
      }		
      
      if (p5==1){
      	 aux=new JIPImage(img);
      	 if (p2!=0){
      	 	System.out.println("Error, esta selccionando como segundo operando una imagen. Sobra la constante operando.");
	      	return(null);      	
      	 }
      }
      else 
      if ((p6.length())!=0){  // si existe una imagen como parametro se comprueba
      	      	
      	String ext=p6.substring(p6.lastIndexOf(".")+1,p6.length());
      	if (!ext.equals(new String("jip"))){
      		System.out.println("Error, la imagen operando no es de tipo JIP.");
	      	return(null);      	
      	}

      	if (p6.length()!=0 && p2!=0){
      		System.out.println("Error, esta selccionando como segundo operando una imagen. Sobra la constante operando.");
	      	return(null);      	
      	}
      	
      	JIPSequence sec=JIPToolkit.getSeqFromFile(p6);
      	if (sec==null){
	      	System.out.println("Error, imposible abrir el fichero: "+p6);	      	
	      	return(null);      		
      	}
      	
      	aux=sec.getFrame(0);
      	
      	if (aux==null){
	      	System.out.println("Error, el fichero: "+p6+" no contiene informacion valida.");
	      	return(null);      		
      	}
      	
      	int waux=aux.getWidth();
				int haux=aux.getHeight();
				int baux=aux.getNumBands();
				int taux=aux.getType();
				
				if (t!=taux){
					System.out.println("Error, el tipo de la imagen "+p6+" ha de ser identico al de la imagen a operar.");
	      	return(null);      	
				}
				
				if (w!=waux || h!=haux){
					System.out.println("Error, el tamao de la imagen "+p6+" ha de ser identico al de la imagen a operar.");
	      	return(null);      	
				}
				
				if (b!=baux){
					System.out.println("Error, el numero de bandas de la imagen "+p6+" ha de ser identico al de la imagen a operar.");
	      	return(null);      	
				}
      }
      	
      System.out.println("Procesando Algoritmo.");
      
			res=new JIPImage(b,w,h,t);
			
      switch (p1){
        case 1: 
    
			    for(int nb=0;nb<b;nb++){          //exp
			      if(t!=JIP.tREAL){
			        int[] bmp=img.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val=bmp[x+y*w];
			              
			              val=(int)Math.exp((double)val);
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        float[] bmp=img.getAllPixelFlo(nb);
			        float[] bin=new float[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              float val=bmp[x+y*w];
			              
			              val=(float)Math.exp((double)val);
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			
			    }
    
          break;
        case 2:
          
			    for(int nb=0;nb<b;nb++){         // log
			      if(t!=JIP.tREAL){
			        int[] bmp=img.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val=bmp[x+y*w];
			              
			              val=(int)Math.log((double)val);
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        float[] bmp=img.getAllPixelFlo(nb);
			        float[] bin=new float[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              float val=bmp[x+y*w];
			              
			              val=(float)Math.log((double)val);
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			    }
    
          break;
        
        case 3:
          
			    for(int nb=0;nb<b;nb++){            //sqrt
			      if(t!=JIP.tREAL){
			        int[] bmp=img.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val=bmp[x+y*w];
			              
			              val=(int)Math.sqrt((double)val);
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        float[] bmp=img.getAllPixelFlo(nb);
			        float[] bin=new float[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              float val=bmp[x+y*w];
			              
			              val=(float)Math.sqrt((double)val);
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			    }
    
          break;
        case 4:
          
			    for(int nb=0;nb<b;nb++){            // square
			      if(t!=JIP.tREAL){
			        int[] bmp=img.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val=bmp[x+y*w];
			              
			              val=(int)Math.pow((double)val,(double)2);
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        float[] bmp=img.getAllPixelFlo(nb);
			        float[] bin=new float[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              float val=bmp[x+y*w];
			              
			              val=(float)Math.pow((double)val,(double)2);
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			    }
    
          break;
          
        case 5:

			    for(int nb=0;nb<b;nb++){            // add
			      if(t!=JIP.tREAL){
			        int[] bmp1=img.getAllPixel(nb);
			        int[] bmp2=null;
			        if (aux!=null) bmp2=aux.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val;
			              
			              if (p2==0.0 && aux!=null) 
			                val=bmp1[x+y*w]+bmp2[x+y*w];
			              else
			                val=bmp1[x+y*w]+(int)p2;
			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        float[] bmp1=img.getAllPixelFlo(nb);
			        float[] bmp2=null;
			        float[] bin=new float[w*h];
			        int ind=0;
			        if (aux!=null) bmp2=aux.getAllPixelFlo(nb);
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              float val;
			              
			              if (p2==0.0 && aux!=null) 
			                val=bmp1[x+y*w]+bmp2[x+y*w];
			              else
			                val=bmp1[x+y*w]+p2;
			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			    }

          break;
        case 6:
          
			    for(int nb=0;nb<b;nb++){            // and
			      if(t!=JIP.tREAL){
			        int[] bmp1=img.getAllPixel(nb);
			        int[] bmp2=null;
			        if (aux!=null) bmp2=aux.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        BigInteger val1,val2;
			        val2=new BigInteger(""+(int)p2);
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val;
			              
			              val1=new BigInteger(""+bmp1[x+y*w]);
			              
			              if (p2==0.0 && aux!=null) 
			                val2=new BigInteger(""+bmp2[x+y*w]);
			              
			              val=(val1.and(val2)).intValue();    
			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        System.out.println("Error, las operaciones logicas solo se realizan sobre valores enteros.");
			        return(null);
			      }
			    }

          break;
        case 7:

			    for(int nb=0;nb<b;nb++){            // dist
			      if(t!=JIP.tREAL){
			        int[] bmp1=img.getAllPixel(nb);
			        int[] bmp2=null;
			        if (aux!=null) bmp2=aux.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val;
			              
			              if (p2==0.0 && aux!=null) 
			                val=Math.abs(bmp1[x+y*w]-bmp2[x+y*w]);
			              else
			                val=Math.abs(bmp1[x+y*w]-(int)p2);
			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        float[] bmp1=img.getAllPixelFlo(nb);
			        float[] bmp2=null;
			        float[] bin=new float[w*h];
			        int ind=0;
			        if (aux!=null) bmp2=aux.getAllPixelFlo(nb);
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              float val;
			              
			              if (p2==0.0 && aux!=null) 
			                val=Math.abs(bmp1[x+y*w]-bmp2[x+y*w]);
			              else
			                val=Math.abs(bmp1[x+y*w]-p2);
			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			    }
          break;
        case 8:
          
          if (p2==0.0 && aux==null){
	        	System.out.println("Error, no se puedo dividir por cero.");
	          return(null);	
          }

			    for(int nb=0;nb<b;nb++){            // div
			      if(t!=JIP.tREAL){
			        int[] bmp1=img.getAllPixel(nb);
			        int[] bmp2=null;
			        if (aux!=null) bmp2=aux.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val=bmp1[x+y*w];
			              
			              if (p2==0.0){
            						if (aux!=null){
             							if (bmp2[x+y*w]!=0) {
               								val=bmp1[x+y*w]/bmp2[x+y*w];
              						}
              					}
           					}
            				else
          						val=(int)(bmp1[x+y*w]/p2);

			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        float[] bmp1=img.getAllPixelFlo(nb);
			        float[] bmp2=null;
			        float[] bin=new float[w*h];
			        int ind=0;
			        if (aux!=null) bmp2=aux.getAllPixelFlo(nb);
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              float val=bmp1[x+y*w];
			              
			              if (p2==0.0)
			              	if (aux!=null)
			              		if (bmp2[x+y*w]!=0) 
			               			val=bmp1[x+y*w]/bmp2[x+y*w];
			              else
			              	val=bmp1[x+y*w]/p2;
			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			    }

          
          break;
        case 9:
          
		    for(int nb=0;nb<b;nb++){            // max
		      if(t!=JIP.tREAL){
		        int[] bmp1=img.getAllPixel(nb);
		        int[] bmp2=null;
			      if (aux!=null) bmp2=aux.getAllPixel(nb);
		        int[] bin=new int[w*h];
		        int ind=0;
		        for (int y=0; y<h; y++)  {
		            for (int x=0; x<w; x++) {
		              int val1,val2,val;
		              
		              if (p2==0.0 && aux!=null){
		                val1=bmp1[x+y*w];
		                val2=bmp2[x+y*w];
		                if (val1>val2) 
		                  val=val1;
		                else
		                  val=val2;
		              }
		              else{
		                val1=bmp1[x+y*w];
		                val2=(int)p2;
		                if (val1>val2) 
		                  val=val1;
		                else
		                  val=val2;
		              }
		
		              if (val > p3) val=p3;
		              if (val < p4) val=p4;
		              
		              bin[ind]=val;    
		              ind++;
		            
		            } 
		        } 
		        res.setAllPixel(nb,bin);
		      }
		      else{
		        float[] bmp1=img.getAllPixelFlo(nb);
		        float[] bmp2=null;
		        float[] bin=new float[w*h];
		        int ind=0;
		        if (aux!=null) bmp2=aux.getAllPixelFlo(nb);
		        for (int y=0; y<h; y++)  {
		            for (int x=0; x<w; x++) {
		              float val1,val2,val;
		              
		              if (p2==0.0 && aux!=null){
		                val1=bmp1[x+y*w];
		                val2=bmp2[x+y*w];
		                if (val1>val2) 
		                  val=val1;
		                else
		                  val=val2;
		              }
		              else{
		                val1=bmp1[x+y*w];
		                val2=p2;
		                if (val1>val2) 
		                  val=val1;
		                else
		                  val=val2;
		              }
		
		              if (val > p3) val=p3;
		              if (val < p4) val=p4;
		              
		              bin[ind]=val;    
		              ind++;
		            
		            } 
		        } 
		        res.setAllPixel(nb,bin);
		      }
		    }

          break;
        case 10:
          
			    for(int nb=0;nb<b;nb++){            // min
			      if(t!=JIP.tREAL){
			        int[] bmp1=img.getAllPixel(nb);
			        int[] bmp2=null;
			        if (aux!=null) bmp2=aux.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val1,val2,val;
			              
			              if (p2==0.0 && aux!=null){
			                val1=bmp1[x+y*w];
			                val2=bmp2[x+y*w];
			                if (val1<val2) 
			                  val=val1;
			                else
			                  val=val2;
			              }
			              else{
			                val1=bmp1[x+y*w];
			                val2=(int)p2;
			                if (val1<val2) 
			                  val=val1;
			                else
			                  val=val2;
			              }
			
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        float[] bmp1=img.getAllPixelFlo(nb);
			        float[] bmp2=null;
			        float[] bin=new float[w*h];
			        int ind=0;
			        if (aux!=null) bmp2=aux.getAllPixelFlo(nb);
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              float val1,val2,val;
			              
			              if (p2==0.0 && aux!=null){
			                val1=bmp1[x+y*w];
			                val2=bmp2[x+y*w];
			                if (val1<val2) 
			                  val=val1;
			                else
			                  val=val2;
			              }
			              else{
			                val1=bmp1[x+y*w];
			                val2=p2;
			                if (val1<val2) 
			                  val=val1;
			                else
			                  val=val2;
			              }
			
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			
			    }

          break;
        
        case 11:
    
			    for(int nb=0;nb<b;nb++){            // mult
			      if(t!=JIP.tREAL){
			        int[] bmp1=img.getAllPixel(nb);
			        int[] bmp2=null;
			        if (aux!=null) bmp2=aux.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val;
			              
			              if (p2==0.0 && aux!=null) 
			                val=bmp1[x+y*w]*bmp2[x+y*w];
			              else
			                val=(int)(bmp1[x+y*w]*p2);
			
			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        float[] bmp1=img.getAllPixelFlo(nb);
			        float[] bmp2=null;
			        float[] bin=new float[w*h];
			        int ind=0;
			        if (aux!=null) bmp2=aux.getAllPixelFlo(nb);
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              float val;
			              
			              if (p2==0.0 && aux!=null) 
			                val=bmp1[x+y*w]*bmp2[x+y*w];
			              else
			                val=bmp1[x+y*w]*p2;
			
			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			    }

          break;

        case 12:
         
			   for(int nb=0;nb<b;nb++){            // or
			      if(t!=JIP.tREAL){
			        int[] bmp1=img.getAllPixel(nb);
			        int[] bmp2=null;
			        if (aux!=null) bmp2=aux.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        BigInteger val1,val2;
			        val2=new BigInteger(""+(int)p2);
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val;
			              
			              val1=new BigInteger(""+bmp1[x+y*w]);
			              
			              if (p2==0.0 && aux!=null) 
			                val2=new BigInteger(""+bmp2[x+y*w]);
			              
			              val=(val1.or(val2)).intValue();    
			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        System.out.println("Error, las operaciones logicas solo se realizan sobre valores enteros.");
			        return(null);
			      }
			
			    }

          break;
          
        case 13:
          
			    for(int nb=0;nb<b;nb++){            // sub
			      if(t!=JIP.tREAL){
			        int[] bmp1=img.getAllPixel(nb);
			        int[] bmp2=null;
			        if (aux!=null) bmp2=aux.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val;
			              
			              if (p2==0.0 && aux!=null) 
			                val=bmp1[x+y*w]-bmp2[x+y*w];
			              else
			                val=(int)(bmp1[x+y*w]-p2);
			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        float[] bmp1=img.getAllPixelFlo(nb);
			        float[] bmp2=null;
			        float[] bin=new float[w*h];
			        int ind=0;
			        if (aux!=null) bmp2=aux.getAllPixelFlo(nb);
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              float val;
			              
			              if (p2==0.0 && aux!=null) 
			                val=bmp1[x+y*w]-bmp2[x+y*w];
			              else
			                val=bmp1[x+y*w]-p2;
			                            
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			    }

          break;
          
        case 14:
          
                    
			    for(int nb=0;nb<b;nb++){            // xor
			      if(t!=JIP.tREAL){
			        int[] bmp1=img.getAllPixel(nb);
			        int[] bmp2=null;
			        if (aux!=null) bmp2=aux.getAllPixel(nb);
			        int[] bin=new int[w*h];
			        int ind=0;
			        BigInteger val1,val2;
			        val2=new BigInteger(""+(int)p2);
			        for (int y=0; y<h; y++)  {
			            for (int x=0; x<w; x++) {
			              int val;			              
			              
		              	val1=new BigInteger(""+bmp1[x+y*w]);		              	
		              			              			              
			              if (p2==0.0 && aux!=null)
				              val2=new BigInteger(""+(int)bmp2[x+y*w]);
			                            
			              val=(val1.xor(val2)).intValue();
			              
			              if (val > p3) val=p3;
			              if (val < p4) val=p4;
			              
			              bin[ind]=val;    
			              ind++;
			            
			            } 
			        } 
			        res.setAllPixel(nb,bin);
			      }
			      else{
			        System.out.println("Error, las operaciones logicas solo se realizan sobre valores enteros.");
			        return(null);
			      }			
			    }

          break;
          
        default:
          System.out.println("Error, operador "+p1+" no definido.");
          return(null);
      }
		}
  
		return res;
	}
}