package jipfunc;

import jip.*;
import java.util.Vector;
import java.lang.*;

/**
* <P><FONT COLOR="red">
*<B>FPhase:</B> Calcula la fase de una imagen compleja.<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Calcula la fase la imagen de entrada. Para ello la imagen
*debe ser compleja, es decir, cada imagen tiene dos bandas (aunque podemos entrar
*como parmetro una imagen con n bandas que contenga n/2 de imagenes).
*Por ejemplo lo podemos utilizar con la salida de VGrad. Para el clculo
*de la fase se utiliza la funcin atan2.<BR>
*Aplicable a tipos: BIT, BYTE, WORD y REAL<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen Procesada de tipo Real, con la mitad de bandas que la imagen
*de entrada.<BR><BR>
*</ul>
*Notas Adicionales: La imagen de entrada debe tener un nmero par de bandas.
*No se aplica a frames geomtricos.<BR>
*/


public class FPhase extends JIPFunction {
  
    /**
    *<P><FONT COLOR="red">
    *Constructor de la Clase.<BR>
    *<P><FONT COLOR="blue">    
    *Obtiene los parmetros de entrada para aplicar la funcin.
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    */

  
    public FPhase() {
       	
		//Nombre y descripcion de la Funcion
	    name="FPhase";
	    description=
	         "Calcula la fase de una imagen compleja.";

		//Asignacion de Parametros
 	       params=new Vector();
	
    }
    
    /**
    *<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *Se calcula la fase de la imagen de entrada<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar con numero par de bandas<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen con la mitad de bandas que la original. En cada banda 
    *contendr la Fase de la imagen.<BR><BR>
    *</ul>
    */

        
    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		if(img!=null) {
			int w=img.getWidth();
			int h=img.getHeight();
			int b=img.getNumBands();
			int t=img.getType();
			int cont=0;
			res=new JIPImage(b/2,w,h,JIP.tREAL);
			if (t==JIP.tBIT || t==JIP.tBYTE || t==JIP.tWORD) { // En primer lugar, en el caso de que no sea Real
				int pp,pp2;
				int[] bmp;
				int[] bmp2;
				float[] bin=new float[w*h];
			
			for (int z=0;z<b;z=z+2) { // Bucle de Bandas
				bmp=img.getAllPixel(z);
				bmp2=img.getAllPixel(z+1);
				for (int x=0;x<w;x++) // EJE X
		  		for (int y=0;y<h;y++) { // EJE Y
		  			pp=bmp[x+y*w];
		  		 	pp2=bmp2[x+y*w];
		  		 	if (pp==0 && pp2==0) bin[x+y*w]=0;
		  		 	else bin[x+y*w]=(float)Math.atan2(pp,pp2);
		  		}  
				res.setAllPixel(cont,bin);
				cont++;
			}
		}
		else if (t==JIP.tREAL) { // En el caso de imagenes Reales
			float[] bmp;
			float[] bmp2;
			float[] bin=new float[w*h];
			float pp,pp2;
			for (int z=0;z<b;z=z+2) {
				bmp=img.getAllPixelFlo(z);
				bmp2=img.getAllPixelFlo(z+1);
				for (int x=0;x<w;x++)
		  		for (int y=0;y<h;y++) {
		  			pp=bmp[x+y*w];
		  		 	pp2=bmp2[x+y*w];
		  		 	if (pp==0 && pp2==0) bin[x+y*w]=0;
		  		 	else bin[x+y*w]=(float)Math.atan2(pp,pp2);
		  		}  
				res.setAllPixel(cont,bin);
				cont++;
			}
		}	// Del Else
		else return null;
	 }
		return res;
	} // de la Funcin
}