package jipfunc;

import jip.*;
import java.util.Vector;
import java.lang.*;

/**
* <P><FONT COLOR="red">
*<B>FRotate:</B> Realiza la Rotacin de la imagen de entrada<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Se realiza la rotacin de la imagen segn el ngulo 
*introducido por el usuario.<BR>
*Aplicable a tipos: BIT, BYTE, WORD, COLOR, REAL, POINT, SEGMENT, POLY y EDGES<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>angulo: ngulo de Rotacin en grados. Si el angulo es positivo la rotacion 
*se hace en sentido horario. Si es negativo se hace en sentido antihorario.<BR>
*<li>clipping: 1 Realiza el recorte de la imagen resultado al tamao
*de la imagen original. NO ES RECOMENDADO UTILIZARLO CON FRAMES 
*GEOMETRICOS<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen de entrada rotada.<BR><BR>
*</ul>
*Notas Adicionales: Se gestionan tambin frames geometricos<BR>
*/

public class FRotate extends JIPFunction {

    /**
    *<P><FONT COLOR="red">
    *Constructor de la Clase.<BR>
    *<P><FONT COLOR="blue">    
    *Obtiene los parmetros de entrada para aplicar la funcin.
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    */

	public FRotate() {
	  //Nombre y descripcion de la Funcion
	  name="FRotate";
	  description="Rota la secuencia o el frame sel. segun el ngulo introducido. (>0 sentido horario, <0 sentido antihorario)";

	  //Definicion de Parametros
		JIPParameter p1=new JIPParameter("angulo",JIP.pINT,false);
		p1.setDefault(30);
		p1.setDescription("Angulo de rotacion en grados");
		
		JIPParameter p2=new JIPParameter("clipping",JIP.pINT,false);
		p2.setDefault(0);
		p2.setDescription("Realizar clipping, (0/1). Advertencia: Pueden corromperse las seq. geomtricas");
		
		//Asignacion de Parametros
    params=new Vector();
		params.addElement(p1);
		params.addElement(p2);

  }
    
  /**
  *<P><FONT COLOR="red">
  *Mtodo Principal<BR>
  *<P><FONT COLOR="blue">    
  *Se le llama para procesar la imgen con el algoritmo de deteccin
  *de bordes de Canny.<BR>
  *<ul><B>Parametros de Entrada:</B><BR>
  *<li>img: Imagen a procesar<BR><BR>
  *</ul>
  *<ul><B>Parametros de Salida:</B><BR>
  *<li>Imagen rotada segn el angulo introducido por el usuario.<BR><BR>
  *</ul>
  */
    
  public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		int ang_g=(int)((JIPParameter)params.elementAt(0)).getValueInt();
		int clip =(int)((JIPParameter)params.elementAt(1)).getValueInt();
		int w=img.getWidth();
		int h=img.getHeight();
		int b=img.getNumBands();
		int t=img.getType();		
  	int cx = w/2;
  	int cy = h/2;
    double sin =Math.sin(-ang_g/(180.0/Math.PI));
    double cos =Math.cos(-ang_g/(180.0/Math.PI));
    int ind;
    int xs,ys;
        
    if (ang_g%360 == 0) 
      ang_g = 0; 
    if (ang_g == 0) 
      return(res);
    
    int wid,hei;
		
    if (clip==0){
			wid=(int)( Math.abs(w*cos)+Math.abs(h*sin) );
			hei=(int)( Math.abs(w*sin)+Math.abs(h*cos) );
			cx=(wid/2);
			cy=(hei/2);
    }
		else{  //  hacemos clipping
      wid=w;
      hei=h;      
		}
    
    if (img.getType()!=JIP.tPOINT && img.getType()!=JIP.tSEGMENT && img.getType()!=JIP.tPOLY && img.getType()!=JIP.tEDGES){        
      res=new JIPImage(b,wid,hei,img.getType());
  
      double dtemp11 = (double)(cy)*sin - (double)(cx)*cos;
      double dtemp21 = -(double)(cx)*sin - (double)(cy)*cos;          

      for(int nb=0;nb<b;nb++){
        if(t!=JIP.tREAL){
          int[] bmp=img.getAllPixel(nb);
          int[] bin=new int[wid*hei];
          for (int y=0; y<hei; y++)  {
              ind = y*wid;
              double dtemp31 = (dtemp11 - (double)(y)*sin) + (double)(w/2.0);
              double dtemp41 = (dtemp21 + (double)(y)*cos) + (double)(h/2.0);
              for (int x=0; x<wid; x++) {
                  xs = (int)(((double)(x)*cos) + dtemp31);
                  ys = (int)(((double)(x)*sin) + dtemp41);
                  if ((xs>=0) && (xs<w) && (ys>=0) && (ys<h))
                     bin[ind++] = bmp[w*ys+xs];
                  else
                     bin[ind++] = 0;
              } 
          } 
          res.setAllPixel(nb,bin);
        }
        else{
          float[] bmp=img.getAllPixelFlo(nb);
          float[] bin=new float [wid*hei];
          for (int y=0; y<hei; y++)  {
              ind = y*wid;
              double dtemp31 = (dtemp11 - (double)(y)*sin) + (double)(w/2.0);
              double dtemp41 = (dtemp21 + (double)(y)*cos) + (double)(h/2.0);
              for (int x=0; x<wid; x++) {
                  xs = (int)(((double)(x)*cos) + dtemp31);
                  ys = (int)(((double)(x)*sin) + dtemp41);
                  if ((xs>=0) && (xs<w) && (ys>=0) && (ys<h))
                     bin[ind++] = bmp[w*ys+xs];
                  else
                     bin[ind++] = 0;
              } 
          } 
           res.setAllPixel(nb,bin);
        }
      }
    }
    else{
        if (img.getType()==JIP.tPOINT || img.getType()==JIP.tSEGMENT){
          // puntos o segmentos
          double dtemp1 = (double)(cy)*sin - (double)(cx)*cos;
          double dtemp2 = -(double)(cx)*sin - (double)(cy)*cos;          

          Vector geodata=(Vector)(img.getVector()).clone();
          Integer auxX,auxY;
          for(int i=0;i<geodata.size();i+=2){  // cojo las coordenadas de dos en dos                       
              int xAct=((Integer)(geodata.elementAt(i))).intValue();
              int yAct=((Integer)(geodata.elementAt(i+1))).intValue();
              for (int y=0; y<hei; y++)  {
                double dtemp3 = (dtemp1 - (double)(y)*sin) + (double)(w/2.0);
                double dtemp4 = (dtemp2 + (double)(y)*cos) + (double)(h/2.0);                    
                for (int x=0;x<wid;x++){
                   xs =(int)(((double)(x)*cos) + dtemp3);
                   ys =(int)(((double)(x)*sin) + dtemp4);                   
                   if (xs>=(xAct-1) && xs<=(xAct+1) && ys>=(yAct-1) && ys<=(yAct+1)){                       
                     auxX=new Integer(x);
                     auxY=new Integer(y);
                     geodata.setElementAt(auxX,i);                       
                     geodata.setElementAt(auxY,i+1);
                   }                  
                }
              }     
          }                        
          res=new JIPImage(wid,hei,t,geodata);
        }
        else{
          if (img.getType()==JIP.tPOLY || img.getType()==JIP.tEDGES){
            // poligonos o edges  

          double dtemp1 = (double)(cy)*sin - (double)(cx)*cos;
          double dtemp2 = -(double)(cx)*sin - (double)(cy)*cos;          

            Vector geodata=new Vector();
            for (int k=0; k < img.getVector().size();k++){
              geodata.addElement(((Vector)(((Vector)(((Vector)((Vector)(img.getVector())).elementAt(k))))).clone()));
            }
            for (int j=0; j<geodata.size();j++){

              for (int i=0;i<((Vector)(geodata.elementAt(j))).size();i+=2){ // cojo de 2 en 2
                Integer auxX,auxY;
                int xAct=(((Integer)(((Vector)(geodata.elementAt(j))).elementAt(i))).intValue());
                int yAct=(((Integer)(((Vector)(geodata.elementAt(j))).elementAt(i+1))).intValue());
                for (int y=0; y<hei; y++)  {
                  double dtemp3 = (dtemp1 - (double)(y)*sin) + (double)(w/2.0);
                  double dtemp4 = (dtemp2 + (double)(y)*cos) + (double)(h/2.0);                      
                  for (int x=0;x<wid;x++){
                    xs =(int)(((double)(x)*cos) + dtemp3);
                    ys =(int)(((double)(x)*sin) + dtemp4);                   
                    if (xs>=(xAct-1) && xs<=(xAct+1) && ys>=(yAct-1) && ys<=(yAct+1)){                       
                      auxX=new Integer(x);                         
                      auxY=new Integer(y);                         
                      ((Vector)(geodata.elementAt(j))).setElementAt(auxX,i);    
                      ((Vector)(geodata.elementAt(j))).setElementAt(auxY,i+1);
                    }
                  }
                }
              }
            }
            res=new JIPImage(wid,hei,t,geodata);          
          }
        }
    }
		return(res);
  }
}