package jipfunc;

import jip.*;
import java.util.Vector;

/**
* <P><FONT COLOR="red">
*<B>FScale:</B> Realiza un escalado de la imagen de entrada.<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Realiza un escalado del frame introducido. El escalado 
*se basa en cuatro metodos sobredos estrategias, Ampliar o Reducir.
*Los metodos para Ampliar son; Ampliar por scale y Ampliar por Expand. El primero genera para un 
*pixel de la imagen un area en funcion del factor de escala con la misma intensidad de pixel. El segundo
*genera para un pixel un area en funcion del factor de escala con intensidades de pixel interpoladas.<BR>
*Los metodos para reducir son: Reducir por Sample y Reducir por Reduce: El primero genera a partir de
*un area en funcion del factor de escala unh pixel con valor de intensidad de pixel igual al primer pixel
*del area. El segundo a partir de un area en funcion del factor de escala genera un pixel con intensidad
*de pixel igual a la media de intensidades de los pixels del area.
*Segun el metodo elegido se obtendra un escalado con resultados graficos mejores o peores.
*Los metodos basados en la estrategia de Sample y Expand, son los que generan resultados graficos peores.
*Los metodos basados en Reduce y Scale son los que generan resultados graficos mejores.
*Este metodo gestiona frames geometricos si se presentan. Los resultados de escalar frames geometricos
*son independientes del metodo escogido.<BR>
*Aplicable a tipos: BIT, BYTE, WORD, COLOR, REAL, POINT, SEGMENT, POLY y EDGES<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>FE: o factor de escala en %.Para reducir el factor de escala oscila entre [1..99], otros 
*factores son rechazados. Para ampliar el factor de escala oscila entre [1..infinito], evidentemente
*el valor maximo del factor de escala para ampliar ira en funcion de los recursos hardware disponibles.
*Notar que por la complejidad del algoritmo cuanto mayor sea el factor de escala para ampliar, mas tiempo
*se consume asi como mas memoria se necesita.<BR>
*<li>RR: reducir por reduce, sus posibles valore son 0 / 1, 1 activa el mtodo. 0 lo desactiva.<BR>
*<li>RS: reducir por sample, sus posibles valore son 0 / 1, 1 activa el mtodo. 0 lo desactiva.<BR>
*<li>AE: apliar por expand, sus posibles valore son 0 / 1, 1 activa el mtodo. 0 lo desactiva.<BR>
*<li>AS: apliar por scale, sus posibles valore son 0 / 1, 1 activa el mtodo. 0 lo desactiva.<BR>
*<li>EX: factor de escala % sobre eje X, El mtodo solo es aplicado al eje X, solo se activa si FE=1<BR>
*<li>EY: factor de escala % sobre eje Y, El mtodo solo es aplicado al eje Y, solo se activa si FE=1<BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen a la que se le ha realizado el escalado<BR><BR>
*</ul>
*Notas Adicionales: Solo uno de los cuatro mtodos es aplicado, el mtodo que se aplica es el primero seleccionado 
*en el dialogo.
*/

public class FScale extends JIPFunction {
  
    /**
    *<P><FONT COLOR="red">
    *Constructor de la Clase.<BR>
    *<P><FONT COLOR="blue">    
    *Obtiene los parmetros de entrada para aplicar la funcin.
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>NINGUNO<BR><BR>
    *</ul>
    */

    public FScale() {
		//Nombre y descripcion de la Funcion
	    name="FScale";
	    description="Modifica el tamao una imagen. Solo se permite un mtodo a la vez y hay que selecionar uno";
  //Definicion de Parametros
  
	  JIPParameter p1=new JIPParameter("FE",JIP.pINT,false);
		p1.setDescription("Factor de Escala en %.");
		p1.setDefault(1);

		JIPParameter p2=new JIPParameter("RR",JIP.pINT,false);
		p2.setDescription("Reducir Por Reduce, 0 no, 1 si");
		p2.setDefault(0);
		
		JIPParameter p3=new JIPParameter("RS",JIP.pINT,false);
		p3.setDescription("Reducir Por Sample, 0 no, 1 si");
		p3.setDefault(0);
		
		JIPParameter p4=new JIPParameter("AE",JIP.pINT,false);
		p4.setDescription("Ampliar Por Expand, 0 no, 1 si");
		p4.setDefault(0);
		
		JIPParameter p5=new JIPParameter("AS",JIP.pINT,false);
		p5.setDescription("Ampliar Por Scale, 0 no, 1 si");
		p5.setDefault(1);
		
		JIPParameter p6=new JIPParameter("EX",JIP.pINT,false);		
		p6.setDescription("Factor de escala para el eje X. Se aplica solo si el factor de escala = 1");
		p6.setDefault(0);
		
		JIPParameter p7=new JIPParameter("EY",JIP.pINT,false);		
		p7.setDescription("Factor de escala para el eje Y. Se aplica solo si el factor de escala = 1");
		p7.setDefault(0);		

		//Asignacion de Parametros
    params=new Vector();
		params.addElement(p1);
		params.addElement(p2);
		params.addElement(p3);
		params.addElement(p4);
		params.addElement(p5);
		params.addElement(p6);
		params.addElement(p7);

    }
    
    /**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *mtodo para reducir frames que no sean de tipo REAL..<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>mtodo: indica el mtodo de reduccion escogido, (RS,RR)<BR>
    *<li>img: frame a tratar.<BR>
    *<li>b: numero de bandas del frame.<BR>
    *<li>p6: ancho de la imagen.<BR>
    *<li>p7: largo de la imagen.<BR>
    *<li>Xstep: tamao del area en funcion del FE para las X<BR>
    *<li>Ystep: tamao del area en funcion del FE para las Y<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>res: Imagen resultado del algoritmo<BR><BR>
    *</ul>
    */
    
    public JIPImage reducir(int metodo,JIPImage img,int b,int p6,int p7,double Xstep,double Ystep){
    	
    	JIPImage res=new JIPImage(b,p6,p7,img.getType());
    	int media=0,x,y,denom=0,pix=0,xs,ys,nb,X,Y;

    	for (nb=0; nb<b; nb++)
				for (Y=0;Y<p7;Y++)
   				for (X=0;X<p6;X++){
    				xs=(int)(X*Xstep);
    				ys=(int)(Y*Ystep);
    				if (metodo==1){
    					media=img.getPixel(nb,xs,ys);
    					denom=0;
    					pix=0;
    					for (x=xs;x<xs+Xstep-1;x++)
    						for (y=ys;y<ys+Ystep-1;y++){
    							pix=pix+img.getPixel(nb,x,y);
    							media=pix;
    							denom++;
    						}
    					if (denom==0) denom=1;
    					res.setPixel(nb,X,Y,(media/denom));    
    				}
    				else
    					res.setPixel(nb,X,Y,img.getPixel(nb,xs,ys));
   				}
   		return(res);
    }
    
    /**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo para reducir frames de tipo REAL..<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>metodo: indica el metodo de reduccion escogido, (RS,RR)<BR>
    *<li>img: frame a tratar.<BR>
    *<li>b: numero de bandas del frame.<BR>
    *<li>p6: ancho de la imagen.<BR>
    *<li>p7: largo de la imagen.<BR>
    *<li>Xstep: tamao del area en funcion del FE para las X<BR>
    *<li>Ystep: tamao del area en funcion del FE para las Y<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>res: Imagen resultado del algoritmo<BR><BR>
    *</ul>
    */

    public JIPImage reducirFlo(int metodo,JIPImage img,int b,int p6,int p7,double Xstep,double Ystep){
    	
    	JIPImage res=new JIPImage(b,p6,p7,img.getType());
    	int x,y,denom=0,xs,ys,nb,X,Y;
    	float media=0,pix=0;

    	for (nb=0; nb<b; nb++)
				for (Y=0;Y<p7;Y++)
   				for (X=0;X<p6;X++){
    				xs=(int)(X*Xstep);
    				ys=(int)(Y*Ystep);
    				if (metodo==1){
    					media=img.getPixel(nb,xs,ys);
    					denom=0;
    					pix=0;
    					for (x=xs;x<xs+Xstep-1;x++)
    						for (y=ys;y<ys+Ystep-1;y++){
    							pix=pix+img.getPixelFlo(nb,x,y);
    							media=pix;
    							denom++;
    						}
    					if (denom==0) denom=1;
    					res.setPixel(nb,X,Y,(media/denom));    
    				}
    				else
    					res.setPixel(nb,X,Y,img.getPixelFlo(nb,xs,ys));
   				}
   		return(res);
    }

    /**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo para ampliar frames que no sean de tipo REAL..<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>metodo: indica el metodo de reduccion escogido, (RS,RR)<BR>
    *<li>img: frame a tratar.<BR>
    *<li>b: numero de bandas del frame.<BR>
    *<li>p6: ancho de la imagen.<BR>
    *<li>p7: largo de la imagen.<BR>
    *<li>Xstep: tamao del area en funcion del FE para las X<BR>
    *<li>Ystep: tamao del area en funcion del FE para las Y<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>res: Imagen resultado del algoritmo<BR><BR>
    *</ul>
    */

    
   public JIPImage ampliar(int metodo,JIPImage img,int b,int p6,int p7,double Xstep,double Ystep){
    	
    	JIPImage res=null;
    	JIPImage ih=null;
    	JIPImage iv=null;
    	int media=0,x,y,denom=0,pix=0,xs,ys,nb,X,Y,rangoX,rangoY,i,j,d,e;
    	int w=img.getWidth();
    	int h=img.getHeight();
    	int intO,intOX,intOY,intFX,intFY,intDifX,intDifY,incX,incY,m,n;
    	rangoX=(int)(1/Xstep);
			rangoY=(int)(1/Ystep);
			double Xinv=1/Xstep,Yinv=1/Ystep;
			res=new JIPImage(b,p6,p7,img.getType());
    	
    	if (metodo==1){
    		int[] imgaux=new int [p6*p7];
	    	for (nb=0; nb<b; nb++){
	    		int[] bmp=img.getAllPixel(nb);
					for (Y=0;Y<p7;Y++)
   					for (X=0;X<p6;X++){
    					xs=(int)(X*Xstep);
    					ys=(int)(Y*Ystep);
    					imgaux[X+Y*p6]=bmp[xs+ys*w];
   					}
 					res.setAllPixel(nb,imgaux);
	    	}
    	}
   		else{
   			
			  int[][] pixels=new int [p6][p7];
			    		  
   		  for (nb=0; nb<b; nb++){
   		  	for (Y=0;Y<p7;Y++)
   					for (X=0;X<p6;X++){
    						pixels[X][Y]=-1;
    				}
   									
					for (Y=0;Y<h;Y++)       //  todas las filas y columnas menos la ultima
  	 				for (X=0;X<w;X++){
    					xs=(int)(X*(Xinv));
    					ys=(int)(Y*(Yinv));
    					pixels[xs][ys]=img.getPixel(nb,X,Y);
  	 				}
   					
   				for(X=0;X<w;X++){      // ultima columna
 						xs=(int)(X*(Xinv));
   					pixels[xs][p7-1]=img.getPixel(nb,X,h-1);
   				}

   				for(Y=0;Y<h;Y++){      // ultima fila
 						ys=(int)(Y*(Yinv));
   					pixels[p6-1][ys]=img.getPixel(nb,w-1,Y);
   				}
   				
   				pixels[p6-1][p7-1]=img.getPixel(nb,w-1,h-1); //	ultimo pxel
   			
   				int rx=0,ry=0,fin=0;

  				// calculo de malla de degradado hasta la penultima fila y columna
  			
  				for (X=0;X<p6;){
  					for (Y=0;Y<p7;){
  						rx=0;
  						fin=0;
  						while(fin==0){
  							if (rx+X+1<p6)
  							 if (pixels[rx+X+1][Y]==-1) 
  							 	rx++;
  							 else 
  							 	fin=1;
  							else
  							fin=1;
  						}
  						ry=0;
  						fin=0;
  						while(fin==0){
  							if (ry+Y+1<p7)
  								if (pixels[X][ry+Y+1]==-1) 
  									ry++;
  								else
  									fin=1;
  							else
  								fin=1;
  						}
  					
  						intO=pixels[X][Y];  						
  					
  						if (rx>0){
  							intFX=pixels[X+rx+1][Y];
   							intDifX=java.lang.Math.abs(intO-intFX);
  	 						incX=intDifX/(rx+1);
				   			if (intO>intFX) incX=-incX;
				   			for(m=X+1;m<=X+rx;m++)
  	 							pixels[m][Y]=intO+(incX*(m-X));
	  	 				}
  						
  						if (ry>0){  						
   							intFY=pixels[X][Y+ry+1];
   							intDifY=java.lang.Math.abs(intO-intFY);
   							incY=intDifY/(ry+1);   					
	 	 						if (intO>intFY) incY=-incY;
	 							for(m=Y+1;m<=Y+ry;m++)
   								pixels[X][m]=intO+(incY*(m-Y));
  	 					}
  	  				
  	  				Y+=ry+1;
  	  				if (Y==p7-1) Y=p7;
  					}
  					X+=rx+1;
  					if (X==p6-1) X=p6;
  				}
  			
  				// ultima fila de la malla	
  			
  				for (X=0;X<p6;){
  					rx=0;
  					fin=0;
	  				while(fin==0){
  						if (rx+X+1<p6)
  							if (pixels[rx+X+1][p7-1]==-1) 
  							 	rx++;
  							else 
  						 		fin=1;
  						else
  							fin=1;
	  				}
  						
  					if (rx>0){
  						intO=pixels[X][p7-1];
	  					intFX=pixels[X+rx+1][p7-1];   						
  	 					intDifX=java.lang.Math.abs(intO-intFX);   						
   						incX=intDifX/(rx+1);
   								
			  	 		if (intO>intFX) incX=-incX;
			   			 				
	  		 			for(m=X+1;m<=X+rx;m++)
  	 							pixels[m][p7-1]=intO+(incX*(m-X));
  	 				}
  	 						
	  	  		X+=rx+1;
  					if (X==p6-1) X=p6;
  				}
  			
  				// ultima columna de la malla
  					
  				for (Y=0;Y<p7;){
	  				ry=0;
  					fin=0;
  					while(fin==0){
  						if (ry+Y+1<p7)
  							if (pixels[p6-1][ry+Y+1]==-1) 
  								ry++;
  							else
  								fin=1;
	  					else
  							fin=1;
  					}
  						
	  				if (ry>0){
  						intO=pixels[p6-1][Y];
  						intFY=pixels[p6-1][Y+ry+1];
   						intDifY=java.lang.Math.abs(intO-intFY);
   						incY=intDifY/(ry+1);   					
		
			   			if (intO>intFY) incY=-incY;
		   		
		  	 			for(m=Y+1;m<=Y+ry;m++)
  	 								pixels[p6-1][m]=intO+(incY*(m-Y));
  	 				}
  	  				
	  	  		Y+=ry+1;
  		  		if (Y==p7-1) Y=p7;  					
  				}
  				
  				// Primera interpolacion sobre la malla en eje X
  				
  				for (Y=1;Y<p7-1;Y++){
	  				for (X=0;X<p6;){
  						fin=0;
  						while(fin==0){
  							if (X+1<p6)
  								if(pixels[X+1][Y]!=-1)
  									X++;
  								else
  									fin=1;
	  						else{
  								fin=1;
  								X=p6;
  							}
  						}
  						
	  					if (X<p6){
  							rx=0;
	  						fin=0;
  							while(fin==0){
  								if (rx+X+1<p6)
  								 if (pixels[rx+X+1][Y]==-1) 
  								 	rx++;
  								 else 
  								 	fin=1;
	  							else
  									fin=1;
	  						}
  							
  							if (rx>0){
		  						intO=pixels[X][Y];
  								intFX=pixels[X+rx+1][Y];   						
   								intDifX=java.lang.Math.abs(intO-intFX);   						
   								incX=intDifX/(rx+1);
   							
			  					if (intO>intFX) incX=-incX;
			   					if (rx==0) rx=1;
	  	 				
	  		 					for(m=X+1;m<=X+rx;m++)
  		 							pixels[m][Y]=intO+(incX*(m-X));  		 						
								}
  	 					
  		  				X+=rx+1;
  							if (X==p6-1) X=p6;
  						}
  					}
  				}
  				
	  			// Segunda interpolacion sobre la malla en eje Y

  				for (X=1;X<p6-1;X++){
  					for (Y=0;Y<p7;){
  						fin=0;
  						while(fin==0){
  							if (Y+1<p7)
  								if(pixels[X][Y+1]!=-1)
  									Y++;
  								else
	  								fin=1;
  							else{
  								fin=1;
  								Y=p7;
  							}
  						}
  							
  						if (Y<p7){
								ry=0;
		  					fin=0;
  							while(fin==0){
  								if (ry+Y+1<p7)
  									if (pixels[X][ry+Y+1]==-1) 
  										ry++;
  									else
  										fin=1;
  								else
  									fin=1;
		  					}
	  				
	  						if (ry!=0){
  								intO=pixels[X][Y];
	  							intFY=pixels[X][Y+ry+1];
	  	 						intDifY=java.lang.Math.abs(intO-intFY);
  	 							incY=intDifY/(ry+1);   					
	
	  		 					if (intO>intFY) incY=-incY;
		   		
	 	 							for(m=Y+1;m<=Y+ry;m++)  // la segunda interpolacion se promedia con la anterior.
  	 	  		 				pixels[X][m]=(pixels[X][m]+intO+(incY*(m-Y)))/2; 
								}
  	  				
  	 						Y+=ry+1;
  	 						if (Y==p7-1) Y=p7;  					
  						}
  					}
  				}/**/
  				 				
  				for (Y=0;Y<p7;Y++)
   					for (X=0;X<p6;X++)
    					res.setPixel(nb,X,Y,pixels[X][Y]); // copia de la interpolacion
   		  }
   		}
   		return(res);
   }
   
    /**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo para ampliar frames que sean de tipo REAL..<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>metodo: indica el metodo de reduccion escogido, (RS,RR)<BR>
    *<li>img: frame a tratar.<BR>
    *<li>b: numero de bandas del frame.<BR>
    *<li>p6: ancho de la imagen.<BR>
    *<li>p7: largo de la imagen.<BR>
    *<li>Xstep: tamao del area en funcion del FE para las X<BR>
    *<li>Ystep: tamao del area en funcion del FE para las Y<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>res: Imagen resultado del algoritmo<BR><BR>
    *</ul>
    */

    public JIPImage ampliarFlo(int metodo,JIPImage img,int b,int p6,int p7,double Xstep,double Ystep){
    	
    	JIPImage res=null;
    	JIPImage ih=null;
    	JIPImage iv=null;
    	int media=0,x,y,denom=0,pix=0,xs,ys,nb,X,Y,rangoX,rangoY,i,j,d,e;
    	int w=img.getWidth();
    	int h=img.getHeight();
    	int m,n;
    	float intO,intOX,intOY,intFX,intFY,intDifX,intDifY,incX,incY;
    	rangoX=(int)(1/Xstep);
			rangoY=(int)(1/Ystep);
			double Xinv=1/Xstep,Yinv=1/Ystep;
			res=new JIPImage(b,p6,p7,img.getType());
    	
    	if (metodo==1){
    		float[] imgaux=new float [p6*p7];
	    	for (nb=0; nb<b; nb++){
	    		float[] bmp=img.getAllPixelFlo(nb);
					for (Y=0;Y<p7;Y++)
   					for (X=0;X<p6;X++){
    					xs=(int)(X*Xstep);
    					ys=(int)(Y*Ystep);
    					imgaux[X+Y*p6]=bmp[xs+ys*w];
   					}
 					res.setAllPixel(nb,imgaux);
	    	}
    	}
   		else{
   			
			  float[][] pixels=new float [p6][p7];
			    		  
   		  for (nb=0; nb<b; nb++){
   		  	for (Y=0;Y<p7;Y++)
   					for (X=0;X<p6;X++){
    						pixels[X][Y]=-1;
    				}
   									
					for (Y=0;Y<h;Y++)       //  todas las filas y columnas menos la ultima
  	 				for (X=0;X<w;X++){
    					xs=(int)(X*(Xinv));
    					ys=(int)(Y*(Yinv));
    					pixels[xs][ys]=img.getPixelFlo(nb,X,Y);
  	 				}
   					
   				for(X=0;X<w;X++){      // ultima columna
 						xs=(int)(X*(Xinv));
   					pixels[xs][p7-1]=img.getPixelFlo(nb,X,h-1);
   				}

   				for(Y=0;Y<h;Y++){      // ultima fila
 						ys=(int)(Y*(Yinv));
   					pixels[p6-1][ys]=img.getPixelFlo(nb,w-1,Y);
   				}
   				
   				pixels[p6-1][p7-1]=img.getPixelFlo(nb,w-1,h-1); //	ultimo pxel
   			
   				int rx=0,ry=0,fin=0;

  				// calculo de malla de degradado hasta la penultima fila y columna
  			
  				for (X=0;X<p6;){
  					for (Y=0;Y<p7;){
  						rx=0;
  						fin=0;
  						while(fin==0){
  							if (rx+X+1<p6)
  							 if (pixels[rx+X+1][Y]==-1) 
  							 	rx++;
  							 else 
  							 	fin=1;
  							else
  							fin=1;
  						}
  						ry=0;
  						fin=0;
  						while(fin==0){
  							if (ry+Y+1<p7)
  								if (pixels[X][ry+Y+1]==-1) 
  									ry++;
  								else
  									fin=1;
  							else
  								fin=1;
  						}
  					
  						intO=pixels[X][Y];  						
  					
  						if (rx>0){
  							intFX=pixels[X+rx+1][Y];
   							intDifX=java.lang.Math.abs(intO-intFX);
  	 						incX=intDifX/(rx+1);
				   			if (intO>intFX) incX=-incX;
				   			for(m=X+1;m<=X+rx;m++)
  	 							pixels[m][Y]=intO+(incX*(m-X));
	  	 				}
  						
  						if (ry>0){  						
   							intFY=pixels[X][Y+ry+1];
   							intDifY=java.lang.Math.abs(intO-intFY);
   							incY=intDifY/(ry+1);   					
	 	 						if (intO>intFY) incY=-incY;
	 							for(m=Y+1;m<=Y+ry;m++)
   								pixels[X][m]=intO+(incY*(m-Y));
  	 					}
  	  				
  	  				Y+=ry+1;
  	  				if (Y==p7-1) Y=p7;
  					}
  					X+=rx+1;
  					if (X==p6-1) X=p6;
  				}
  			
  				// ultima fila de la malla	
  			
  				for (X=0;X<p6;){
  					rx=0;
  					fin=0;
	  				while(fin==0){
  						if (rx+X+1<p6)
  							if (pixels[rx+X+1][p7-1]==-1) 
  							 	rx++;
  							else 
  						 		fin=1;
  						else
  							fin=1;
	  				}
  						
  					if (rx>0){
  						intO=pixels[X][p7-1];
	  					intFX=pixels[X+rx+1][p7-1];   						
  	 					intDifX=java.lang.Math.abs(intO-intFX);   						
   						incX=intDifX/(rx+1);
   								
			  	 		if (intO>intFX) incX=-incX;
			   			 				
	  		 			for(m=X+1;m<=X+rx;m++)
  	 							pixels[m][p7-1]=intO+(incX*(m-X));
  	 				}
  	 						
	  	  		X+=rx+1;
  					if (X==p6-1) X=p6;
  				}
  			
  				// ultima columna de la malla
  					
  				for (Y=0;Y<p7;){
	  				ry=0;
  					fin=0;
  					while(fin==0){
  						if (ry+Y+1<p7)
  							if (pixels[p6-1][ry+Y+1]==-1) 
  								ry++;
  							else
  								fin=1;
	  					else
  							fin=1;
  					}
  						
	  				if (ry>0){
  						intO=pixels[p6-1][Y];
  						intFY=pixels[p6-1][Y+ry+1];
   						intDifY=java.lang.Math.abs(intO-intFY);
   						incY=intDifY/(ry+1);   					
		
			   			if (intO>intFY) incY=-incY;
		   		
		  	 			for(m=Y+1;m<=Y+ry;m++)
  	 								pixels[p6-1][m]=intO+(incY*(m-Y));
  	 				}
  	  				
	  	  		Y+=ry+1;
  		  		if (Y==p7-1) Y=p7;  					
  				}
  				
  				// Primera interpolacion sobre la malla en eje X
  				
  				for (Y=1;Y<p7-1;Y++){
	  				for (X=0;X<p6;){
  						fin=0;
  						while(fin==0){
  							if (X+1<p6)
  								if(pixels[X+1][Y]!=-1)
  									X++;
  								else
  									fin=1;
	  						else{
  								fin=1;
  								X=p6;
  							}
  						}
  						
	  					if (X<p6){
  							rx=0;
	  						fin=0;
  							while(fin==0){
  								if (rx+X+1<p6)
  								 if (pixels[rx+X+1][Y]==-1) 
  								 	rx++;
  								 else 
  								 	fin=1;
	  							else
  									fin=1;
	  						}
  							
  							if (rx>0){
		  						intO=pixels[X][Y];
  								intFX=pixels[X+rx+1][Y];   						
   								intDifX=java.lang.Math.abs(intO-intFX);   						
   								incX=intDifX/(rx+1);
   							
			  					if (intO>intFX) incX=-incX;
			   					if (rx==0) rx=1;
	  	 				
	  		 					for(m=X+1;m<=X+rx;m++)
  		 							pixels[m][Y]=intO+(incX*(m-X));  		 						
								}
  	 					
  		  				X+=rx+1;
  							if (X==p6-1) X=p6;
  						}
  					}
  				}
  				
	  			// Segunda interpolacion sobre la malla en eje Y

  				for (X=1;X<p6-1;X++){
  					for (Y=0;Y<p7;){
  						fin=0;
  						while(fin==0){
  							if (Y+1<p7)
  								if(pixels[X][Y+1]!=-1)
  									Y++;
  								else
	  								fin=1;
  							else{
  								fin=1;
  								Y=p7;
  							}
  						}
  							
  						if (Y<p7){
								ry=0;
		  					fin=0;
  							while(fin==0){
  								if (ry+Y+1<p7)
  									if (pixels[X][ry+Y+1]==-1) 
  										ry++;
  									else
  										fin=1;
  								else
  									fin=1;
		  					}
	  				
	  						if (ry!=0){
  								intO=pixels[X][Y];
	  							intFY=pixels[X][Y+ry+1];
	  	 						intDifY=java.lang.Math.abs(intO-intFY);
  	 							incY=intDifY/(ry+1);   					
	
	  		 					if (intO>intFY) incY=-incY;
		   		
	 	 							for(m=Y+1;m<=Y+ry;m++)  // la segunda interpolacion se promedia con la anterior.
  	 	  		 				pixels[X][m]=(pixels[X][m]+intO+(incY*(m-Y)))/2; 
								}
  	  				
  	 						Y+=ry+1;
  	 						if (Y==p7-1) Y=p7;  					
  						}
  					}
  				}/**/
  				 				
  				for (Y=0;Y<p7;Y++)
   					for (X=0;X<p6;X++)
    					res.setPixel(nb,X,Y,pixels[X][Y]); // copia de la interpolacion
   		  }
   		}
   		return(res);
   }
   
  /**
  *<P><FONT COLOR="red">
  *Mtodo Principal<BR>
  *<P><FONT COLOR="blue">    
  *Este metodo es el encargado de precalcular datos y realizar las llamadas a los metodos
  *de calculo en funcion de la naturaleza del frame a tratar.
  *Se distinguen tres partes en el metodo, la primera es la encargada de realizar la asignacion
  *de parametros, filtrarlos y hacer los precalculos en funcion de los datos introducidos.
  *La segunda y la tercera se encagan de realizar las llamadas a los metodos correspondientes.
  *La segunda gestiona las llamadas para los frames no geometricos mientras que la tercera se 
  *encarga de gestionar los frames geometricos.<BR>
  *<ul><B>Parametros de Entrada:</B><BR>
  *<li>img: Imagen a procesar<BR><BR>
  *</ul>
  *<ul><B>Parametros de Salida:</B><BR>
  *<li>Imagen de entrada a la que se le ha realizado el escalado.<BR><BR>
  *</ul>
  */
    public JIPImage processImg(JIPImage img) {
		int p1=((JIPParameter)params.elementAt(0)).getValueInt();
		int p2=((JIPParameter)params.elementAt(1)).getValueInt();
		int p3=((JIPParameter)params.elementAt(2)).getValueInt();
		int p4=((JIPParameter)params.elementAt(3)).getValueInt();
		int p5=((JIPParameter)params.elementAt(4)).getValueInt();
		int p6=((JIPParameter)params.elementAt(5)).getValueInt();		
		int p7=((JIPParameter)params.elementAt(6)).getValueInt();		

		JIPImage res=null;
		
		if(img!=null) {
			int w=img.getWidth();
			int h=img.getHeight();
			int b=img.getNumBands();
			int t=img.getType();
						
			System.out.println("Procesando Algoritmo...");			
			
			if (p2==0 && p3==0 && p4==0 && p5==0) {
				System.out.println("Error, no hay metodo selecionado");
				return(res);
			}
			if (p1<0){
				System.out.println("Error, el factor de escala ha de ser >= 0");
				return(res);
			}
			
			if (p1>1){
				if (p2==1 || p3==1){ //estoy reduciendo por factor de escala
					p6=w-(w*p1/100);
					p7=h-(h*p1/100);
				}
				else{
					if (p4==1 || p5==1){ // estoy ampliando por factor de escala
						p6=w+(w*p1/100);
						p7=h+(h*p1/100);					
					}
				}
			}
			else{
				if (p2==1 || p3==1){ //estoy reduciendo por factor de escala en x y en y
					p6=w-(w*p6/100);
					p7=h-(h*p7/100);
				}
				else{
					if (p4==1 || p5==1){ // estoy ampliando por factor de escala en x y en y
							p6=w+(w*p6/100);
							p7=h+(h*p7/100);					
					}
				}
			}
 		 	
 		 	double Xstep=(double)w/(double)p6;
  		double Ystep=(double)h/(double)p7;
			
			if (t==JIP.tBYTE || t==JIP.tBIT || t==JIP.tCOLOR){        
        res=new JIPImage(b,p6,p7,t);
        
    		if (p2==1)
    			res=reducir(1,img,b,p6,p7,Xstep,Ystep);
    		else
    			if (p3==1)
    				res=reducir(2,img,b,p6,p7,Xstep,Ystep);
    			else
    				if (p4==1)
    					res=ampliar(1,img,b,p6,p7,Xstep,Ystep);
    				else
    					if (p5==1)
    						res=ampliar(2,img,b,p6,p7,Xstep,Ystep);
			}
			else
        if (t==JIP.tREAL){
          res=new JIPImage(b,p6,p7,t);

          if (p2==1)
            res=reducirFlo(1,img,b,p6,p7,Xstep,Ystep);
          else
            if (p3==1)
              res=reducirFlo(2,img,b,p6,p7,Xstep,Ystep);
            else
              if (p4==1)
                res=ampliarFlo(1,img,b,p6,p7,Xstep,Ystep);
              else
                if (p5==1)
                  res=ampliarFlo(2,img,b,p6,p7,Xstep,Ystep);
			}
      else{  // datos geometricos
        int wAct=p6;  // w y h de la nueva resolucion
        int hAct=p7;
        double dmul=((double)wAct/(double)w);
        
        if (t==JIP.tPOINT){
          Vector puntos=(Vector)((img.getVector()).clone()); // hago un clone para no machacar
          for (int i=0;i<puntos.size();i++){
            puntos.setElementAt(
              new Integer((int)(dmul*(double)(((Integer)(puntos.elementAt(i))).intValue())))
              ,i);    
          }
          res=new JIPImage(p6,p7,t,puntos);   
        }
        else
          if (t==JIP.tSEGMENT){
            Vector segmentos=(Vector)((img.getVector()).clone());
            for (int i=0;i<segmentos.size();i++){
              segmentos.setElementAt(
                new Integer((int)(dmul*(double)(((Integer)(segmentos.elementAt(i))).intValue())))
                ,i);    
            }
            res=new JIPImage(p6,p7,t,segmentos);
          }
          else
            if (t==JIP.tPOLY){
              Vector poligonos=new Vector();
              for (int k=0; k < img.getVector().size();k++){
                poligonos.addElement(((Vector)(((Vector)(((Vector)((Vector)(img.getVector())).elementAt(k))))).clone()));
              }
              for (int j=0; j<poligonos.size();j++){
                for (int i=0;i<((Vector)(poligonos.elementAt(j))).size();i++){
                  ((Vector)(poligonos.elementAt(j))).setElementAt(
                    new Integer((int)(dmul*(double)(((Integer)(((Vector)(poligonos.elementAt(j))).elementAt(i))).intValue())))
                    ,i);    
                }
              }
              res=new JIPImage(p6,p7,t,poligonos);
            }
            else
              if (t==JIP.tEDGES){
                Vector edges=new Vector();
                for (int k=0; k < img.getVector().size();k++){
                  edges.addElement(((Vector)(((Vector)(((Vector)((Vector)(img.getVector())).elementAt(k))))).clone()));
                }
                for (int j=0; j<edges.size();j++){
                  for (int i=0;i<((Vector)(edges.elementAt(j))).size();i++){
                    ((Vector)(edges.elementAt(j))).setElementAt(
                      new Integer((int)(dmul*(double)(((Integer)(((Vector)(edges.elementAt(j))).elementAt(i))).intValue())))
                      ,i);    
                  }
                }
                res=new JIPImage(p6,p7,t,edges);
              }
      }    	
		}
		System.out.println("Algoritmo Terminado OK...");
		return res;
	}
}