package jipfunc;

import jip.*;
import java.util.Vector;

/**
* <P><FONT COLOR="red">
*<B>FSegEdges:</B> Crea un conjunto de Segmentos a partir de un conjunto de Edges<BR>
*<FONT COLOR="blue">
*DESCRIPCION: A partir de cada Edge creamos un conjunto de Segmentos correspondiente a la
*superposicin de esos segmentos sobre la lista de puntos.</B>
*Aplicable a tipos: EDGES<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>accuracy: Mnima desviacin debido a la medida de los pixels<BR>
*<li>granularity: Mnima longitud del segmento durante la subdivisin
*recursiva<BR>
*<li>magnitude:Magnitud mnima del segmento en curso<BR>
*<li>product:<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen de tipo Segmento correspondiente a la conversin aproximada de
*tipo Edge a tipo Segmento.<BR><BR>
*</ul>
*/



public class FSegEdges extends JIPFunction {
	
			/** Minima desviacin debido a la medida de los pixels*/
      float accuracy;
      /** Magnitud mnima del segmento en curso */
      float magnitude;
      /**Mnima longitud del segmento durante la subdivisin recursiva*/
			int granularity;
      /** Vector donde almaceno los segmentos ya creados */
			Vector Segmentos;
      /** Vector que contiene todos los Edges */
			Vector Edges;
      /** Vector que contiene el Edge que se est procesando */
			Vector Edge; 
      /** Filas de la imagen */
      int ImageRows;
      /** Columnas de la imagen */
      int ImageColumns;
      /** Variable para pasar por referencia datos de SplitSegment a Maxpoint*/
		  float sig;
      /** Lista de Breakpoints para cortar los segmentos */
		  int [] breakpoints;
      /** Nmero de Breakpoints*/
		  int nbreakpoints;
		  
      /**
      *<P><FONT COLOR="red">
      *Constructor de la Clase.<BR>
      *<P><FONT COLOR="blue">    
      *Obtiene los parmetros de entrada para aplicar la funcin.
      *<ul><B>Parametros de Entrada:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      *<ul><B>Parametros de Salida:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      */
      
public FSegEdges() {
	//Nombre y descripcion de la Funcion
	 name="FSegEdges";
   description="Crea un conjunto de Segmentos a partir de un conjunto de Edges";
   JIPParameter p1=new JIPParameter("accuracy",JIP.pREAL,false);
   p1.setDefault((float)1.0);
   p1.setDescription("Minima desviacin debido a la medida de los pixels");
   JIPParameter p2=new JIPParameter("granularity",JIP.pINT,false);
   p2.setDefault(4);
   p2.setDescription("Mnima longitud del segmento durante la subdivisin recursiva");
   JIPParameter p3=new JIPParameter("magnitude",JIP.pREAL,false);
   p3.setDefault((float)1.0);
   p3.setDescription("Magnitud mnima del segmento en curso");
   

   //Asignacion de Parametros
   params=new Vector();
   params.addElement(p1);  
   params.addElement(p2);	
   params.addElement(p3);   
  }
    
  /**
  *<P><FONT COLOR="red">
  *Mtodo Principal<BR>
  *<P><FONT COLOR="blue">    
  *Este metodo es el encargado de recibir la imagen de entrada y llamar para cada
  *edge de la imagen a la funcion que trata cada edge independientemente.<BR>
  *<ul><B>Parametros de Entrada:</B><BR>
  *<li>img: Imagen a procesar<BR><BR>
  *</ul>
  *<ul><B>Parametros de Salida:</B><BR>
  *<li>Imagen de tipo Segmento correspondiente al equivalente de los Edges.<BR><BR>
  *</ul>
  */

    
 public JIPImage processImg(JIPImage img) 
   {
	JIPImage res=null;
  if(img!=null && img.getType()==JIP.tEDGES) {
  	int t=img.getType();
		int cont=0;
		int w=img.getWidth();
		int h=img.getHeight();
		int b=img.getNumBands();
	  Edges=new Vector();
		Edge=new Vector();
		Segmentos=new Vector();
		Edges=img.getVector();
    accuracy=((JIPParameter)params.elementAt(0)).getValueReal();
    granularity=((JIPParameter)params.elementAt(1)).getValueInt();
    magnitude=((JIPParameter)params.elementAt(2)).getValueReal();
    for (int pp=0; pp<Edges.size();pp++) {
  			Vector Aux2=new Vector();
  			Aux2=(Vector)Edges.elementAt(pp);
		}
   
    for (int r = 0; r < Edges.size(); r++)
    			FSegEdgesIntoLines((Vector)Edges.elementAt(r));
  	
  	res=new JIPImage(w,h,JIP.tSEGMENT,Segmentos);

		return res;			
	}
	else return res;
 }	    		
 
    /**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo procesa un Edge en particular.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>aux: Vector Edge a procesar<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Almacena el resultado en las variables de la clase.<BR><BR>
    *</ul>
    */


 
 void FSegEdgesIntoLines(Vector Aux) {
 		
 		float accuracyActual;
 		int max_npoints,i,j,npoints;
 		float t;
 		Integer coor;
 		 		
    Edge=(Vector)Aux.clone();
    		 		
 		accuracyActual=accuracy*accuracy;
		max_npoints=Edge.size()/2;
		breakpoints=new int [max_npoints];
		breakpoints[0]=0;
		nbreakpoints=1;
		SplitSegment(0,(Edge.size()/2)-2);
		breakpoints[nbreakpoints]=(Edge.size()/2)-2;
		
		for (i=0; i < nbreakpoints; i++) {
								
				if (magnitude > 0.0 && (Edge.size()/2)	> 2 ) { 
						t=(float)0.0;
						for (j=breakpoints[i];j <=breakpoints[i+1]; j++) 
								t+=Math.abs(((Integer)Edge.elementAt(i)).floatValue());
						if (t/(j-breakpoints[i]) < magnitude)
								continue;
				}
				
				/* Aadimos los segmentos al resultado */
				
				npoints=breakpoints[i+1]-breakpoints[i]+1;
				coor=new Integer(0);
				
				coor=(Integer)Edge.elementAt(breakpoints[i]*2);
				Segmentos.addElement(coor);
				coor=(Integer)Edge.elementAt(breakpoints[i]*2+1);
				Segmentos.addElement(coor);
				coor=(Integer)Edge.elementAt(breakpoints[i+1]*2);
				Segmentos.addElement(coor);
				coor=(Integer)Edge.elementAt(breakpoints[i+1]*2+1);
				Segmentos.addElement(coor);
		
		}
 	}
	
    /**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo que corta un Edge en Lineas (segmentos), 
    *recursivamente, y dependiendo de la 
    *mejor ratio de desviacin/longitud que tenga el segmento.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>first: Primer punto del Edge a tratar (punto inicial del segmento)<BR>
    *<li>last: litmo punto del Edge a tratar (punto final del segmento) <BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Devuelve la mxima longitud/ratio de desviacin del segmento.<BR><BR>
    *</ul>
    */

	float SplitSegment(int first,int last) {
			int maxp,save_nbreakpoints=nbreakpoints;
			float sig1,sig2,maxsig;
      Vector sig=new Vector();

			/* Si el segmento es muy pequeo, no lo cortamos ms  */
			if (last-first<=granularity) 
					return (float) 0.0;

			/* Buscamos el punto con mxima desviacin */
			
			maxp=MaxPoint(first, last, sig); 
			
			/* Creamos subsegmentos en llamadas recursivas */
			
			sig1=SplitSegment(first,maxp);
			breakpoints[nbreakpoints++]=maxp;
			sig2=SplitSegment(maxp,last);
			maxsig= sig1>sig2 ? sig1:sig2;
			
			if (maxsig > ((Float)sig.elementAt(0)).floatValue()) {
				return maxsig;
			}
			else {
				nbreakpoints=save_nbreakpoints;
				return ((Float)sig.elementAt(0)).floatValue();
			}
	}

    /**
    *<P><FONT COLOR="red">
    *DESCRIPCIN<BR>
    *<P><FONT COLOR="blue">    
    *Metodo que devuelve la posicin del punto mas alejado a la recta que 
    *une el primer punto y el ultimo del edge y que se encuentra 
    *en el edge.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>first: Primer punto del Edge a tratar (punto inicial del segmento)<BR>
    *<li>last: litmo punto del Edge a tratar (punto final del segmento) <BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Devuelve la posicin del punto ms alejado.
    *<li>Devuelve el cuadrado de la longitud
    *mediante la variable de clase sig<BR><BR>
    *</ul>
    */

	

	int MaxPoint(int first, int last, Vector sig) { 
			float x0,y0,dx,dy,dev,px,py,t,maxdev;
			int maxp,i;
			x0=(float)((Integer)Edge.elementAt(first*2)).intValue();
			y0=(float)((Integer)Edge.elementAt(first*2+1)).intValue();
			dx=((float)((Integer)Edge.elementAt(last*2)).intValue())-x0;
			dy=((float)((Integer)Edge.elementAt(last*2+1)).intValue())-y0;
						
			   
			for (i=maxp=first+1, maxdev=0f; i < last; i++) {
					px=(float)((Integer)Edge.elementAt(i*2)).intValue()-x0;
					py=(float)((Integer)Edge.elementAt(i*2+1)).intValue()-y0;
					
					/* Calculamos ls distancia a la linea  */
					
					t= (dx*px+dy*py)/(dx*dx+dy*dy);
					px-=dx*t;
					py-=dy*t;
					dev=px*px+py*py;
					
					if (dev>maxdev) {
							maxdev=dev;
							maxp=i;
					}
          
			}
			
			
			
			if (maxdev < accuracy) 
					maxdev=accuracy;
					
				
			sig.addElement(new Float( (dx*dx+dy*dy) / maxdev));
			return maxp;
			
	}
		
		
		
}// DE LA CLASE