package jipfunc;

import jip.*;
import java.util.Vector;
import java.util.Random;
import java.io.*;

/**
* <P><FONT COLOR="red">
*<B>FSharpen:</B> Realce de una imagen<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Convoluciona una imagen con una matriz de valores dependientes
*del nivel de realce especificado como parmetro. La matriz es de 3x3 y los valores
*que varan el pixel (x,y) a tomar son (x-1,y) (x+1,y) (x,y-1) (x,y+1) los pxels
*de las esquinas quedan a 0.
*Aplicable a tipos: BYTE, WORD, COLOR y REAL<BR>
*Usa: FConvolveImage<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>Nivel: Desde 1 (-0.1 en vecinos) hasta 100 (-10.0 en vecinos)<BR>
*<li>metodo: ZERO-> los pixels que quedan fuera se ponen a 0.
*PAD->cuando el pixel est fuera, se toma el valor del pixel de la imagen
*ms cercano. P.e. (-1,-1) se tomara (0,0), (-2,2) se tomara (0,2), etc.
*WRAP-> cuando el pxel est fuera, se toma el valor del pixel opuesto en
*los ejes que corresponda. P.e. (-1,-1) se tomara (w,h), (0,-2) se tomara
*(0,h-1) etc.<BR><BR>
*</ul>
<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen realzada, del mismo tipo que la imagen de entrada<BR><BR>
*</ul>
*Notas Adicionales: La matriz que se genera internamente se convierte a una imagen 
*y se llama al metodo que realiza la convolucin de dos imagenes.
*/


public class FSharpen extends JIPFunction {
		/**
		*<P><FONT COLOR="red">
		*Constructor de la Clase.<BR>
		*<P><FONT COLOR="blue">		
		*Obtiene los parmetros de entrada para aplicar la funcin.
		*<ul><B>Parametros de Entrada:</B><BR>
		*<li>NINGUNO<BR><BR>
		*</ul>
		<ul><B>Parametros de Salida:</B><BR>
		*<li>NINGUNO<BR><BR>
		*</ul>
		*/

    public FSharpen() {
		//Nombre y descripcion de la Funcion
	    name="FSharpen";
	    description=
	         "Realiza el realce de un imagen (4-vecinos)";

	    //Definicion de Parametros
        JIPParameter p1=new JIPParameter("nivel",JIP.pREAL,false);
        p1.setDescription("Nivel de realce (1..100)");
        p1.setDefault(5f);
        JIPParameter p2=new JIPParameter("metodo",JIP.pSTRING,false);
        p2.setDefault("ZERO");
        p2.setDescription("ZERO, WRAP, PAD");
            
            

		//Asignacion de Parametros
        params=new Vector();
	    params.addElement(p1);
        params.addElement(p2);
        
    }

		/**
		*<P><FONT COLOR="red">
		*Mtodo Principal<BR>
		*<P><FONT COLOR="blue">		
		*Se le llama para convolucionar la imagen de entrada con
		*la matriz 3x3 generada.<BR>
		*<ul><B>Parametros de Entrada:</B><BR>
		*<li>img: Imagen a procesar<BR><BR>
		*</ul>
		<ul><B>Parametros de Salida:</B><BR>
		*<li>Imagen realzada <BR><BR>
		*</ul>
		*/

    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		
    String p2=getParamValueStr("metodo").toUpperCase();
    float nivel=getParamValueReal("nivel");
    
    if (nivel<1 || nivel>100) return null;

      float[] mat=new float[3*3];

      float valor=-(nivel/10f);
      for (int i=0;i<3*3;i++) mat[i]=0f;
      mat[1]=mat[3]=mat[5]=mat[7]=valor;
      mat[4]=valor*(-4f)+1;
      JIPFunction convolucion=new FConvolveImage();
       
      JIPImage convo=new JIPImage(3,3,mat);
    
      convolucion.setParamValue("imagen",convo);
      convolucion.setParamValue("div",1);
      convolucion.setParamValue("mult",1);
      convolucion.setParamValue("metodo",p2);
    
      // LLamamos a FConvolve
      res=convolucion.processImg(img);
      return res;
 }
}