package jipfunc;

import jip.*;
import java.util.Vector;

/**
* <P><FONT COLOR="red">
*<B>FSkew:</B> Abate una imagen con respecto a uno de sus lados<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Abate todos los puntos de una imagen segun un
*angulo y respecto a uno de los lados<BR>
*Aplicable a tipos: BYTE, WORD, COLOR y REAL<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>angulo: Angulo de abatimiento, < 0 abate a la izq. > 0 a la der. Maximos (-89, +89)<BR>
*<li>clipping: Permite cortar la imagen, 0 desactivado, 1 activado.<BR>
*<li>bottom: Abatimiento sobre la base.1 activado 0 desactivado.<BR>
*<li>left: Abatimiento sobre la izquierda. 1 activado 0 desactivado.<BR>
*<li>right:Abatimiento sobre la derecha.1 activado 0 desactivado.<BR>
*<li>up: Abatimiento sobre la parte superior.1 activado 0 desactivado.<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen Procesada del mismo tipo que la imagen de entrada,
*pero abatida.<BR><BR>
*</ul>
*/


public class FSkew extends JIPFunction {

      /**
      *<P><FONT COLOR="red">
      *Constructor de la Clase.<BR>
      *<P><FONT COLOR="blue">    
      *Obtiene los parmetros de entrada para aplicar la funcin.
      *<ul><B>Parametros de Entrada:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      *<ul><B>Parametros de Salida:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      */


    public FSkew() {
		//Nombre y descripcion de la Funcion
	    name="FSkew";
	    description=
	         "Abate todos los puntos de una imagen segun un angulo y respecto a uno de los lados";

	    //Definicion de Parametros
		JIPParameter p1=new JIPParameter("angulo",JIP.pINT,false);
		p1.setDefault(15);
		p1.setDescription("Angulo de abatimiento, < 0 abate a la izq. > 0 a la der. Maximos (-89, +89)");
		
		JIPParameter p2=new JIPParameter("clipping",JIP.pINT,false);
		p2.setDefault(0);
		p2.setDescription("Permite cortar la imagen, 0 desactivado, 1 activado.");

		JIPParameter p3=new JIPParameter("bottom",JIP.pINT,false);
		p3.setDefault(1);
		p3.setDescription("Eje origen, 1 activado 0 desactivado.");
		
		JIPParameter p4=new JIPParameter("left",JIP.pINT,false);
		p4.setDefault(0);
		p4.setDescription("Eje origen, 1 activado 0 desactivado.");

		JIPParameter p5=new JIPParameter("right",JIP.pINT,false);
		p5.setDefault(0);
		p5.setDescription("Eje origen, 1 activado 0 desactivado.");
		
		JIPParameter p6=new JIPParameter("up",JIP.pINT,false);
		p6.setDefault(0);
		p6.setDescription("Eje origen, 1 activado 0 desactivado.");


		//Asignacion de Parametros
        params=new Vector();
		params.addElement(p1);
		params.addElement(p2);
		params.addElement(p3);
		params.addElement(p4);
		params.addElement(p5);
		params.addElement(p6);

    }

    /**
    *<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *Este metodo es el encargado de recibir la imagen de entrada y 
    *abatirla segn los parametros introducidos por el usuario.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen del mismo tipo que la imagen de entrada,
    pero abatida.<BR><BR>
    *</ul>
    */

    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		JIPImage res2=null;
		int p1=((JIPParameter)params.elementAt(0)).getValueInt();
		int p2=((JIPParameter)params.elementAt(1)).getValueInt();
		int p3=((JIPParameter)params.elementAt(2)).getValueInt();
		int p4=((JIPParameter)params.elementAt(3)).getValueInt();
    int p5=((JIPParameter)params.elementAt(4)).getValueInt();
		int p6=((JIPParameter)params.elementAt(5)).getValueInt();
    
    int t=img.getType();		
    int w=img.getWidth();
		int h=img.getHeight();
		int b=img.getNumBands();
		int x,y,noIni=0;
		double ang,sa,ca;		

    ang=Math.abs(p1)*0.01745329252; //a radianes
		sa=Math.sin(ang);
				
		int ntam=0;
		
		if (Math.abs(p1)>89) {
			System.out.println("Error, angulo incorrecto.");
			return(res);
		}
		
		if (p3==0 && p4==0 && p5==0 && p6==0){
			System.out.println("Error, no hay ningun lado seleccionado.");
		}		
		else{
			System.out.println("Procesando Algoritmo..."); 

			if (p3==1){	// bottom
				ntam=(int)(w+h*sa);
				if (p2==0) {
					res=new JIPImage(b,ntam,h,t);
				}
				else
				if (p2==1){
					res=new JIPImage(b,w,h,t);
				}
				else{
					System.out.println("Error, opcion de clipping incorrecta.");
					return(res);
				}
			
				for(int nb=0;nb<b;nb++){
					if (t!=JIP.tREAL){		  
  					int[] bmp=img.getAllPixel(nb);
		  			int[] bin=new int[ntam*h];
		    		for (y=0;y<h;y++){
        			for (x=0;x<w;x++){
          			int pos;
	         			if (p1>0){
	         				pos=(int)(x+(h-y)*sa)+(y*ntam);
	         				bin[pos]=bmp[x+y*w];
         	  		}
         	  		else{
         	  		  pos=(int)(x+(h-(h-y))*sa)+(y*ntam);
           				bin[pos]=bmp[x+y*w];
         	  		}
        			}
		    		}
        
        		if (p2==1){	
        			int[] aux=new int [w*h];       	
        			for (x=0;x<w;x++)
        				for (y=0;y<h;y++){
        					if (p1>0)
        						aux[x+y*w]=bin[x+y*ntam];
        					else
        						aux[x+y*w]=bin[(int)(x+h*sa)+(y*ntam)];
	        			}        	
			      	res.setAllPixel(nb,aux);
        		}
      	  	else
        			res.setAllPixel(nb,bin);
					}
					else{
						float[] bmp=img.getAllPixelFlo(nb);
		  			float[] bin=new float[ntam*h];
		    		for (y=0;y<h;y++){
        			for (x=0;x<w;x++){
          			int pos;
	         			if (p1>0){
	         				pos=(int)(x+(h-y)*sa)+(y*ntam);
	         				bin[pos]=bmp[x+y*w];
         	  		}
         	  		else{
         	  		  pos=(int)(x+(h-(h-y))*sa)+(y*ntam);
           				bin[pos]=bmp[x+y*w];
         	  		}
        			}
		    		}
        
        		if (p2==1){	
        			float[] aux=new float [w*h];       	
        			for (x=0;x<w;x++)
        				for (y=0;y<h;y++){
        					if (p1>0)
        						aux[x+y*w]=bin[x+y*ntam];
        					else
        						aux[x+y*w]=bin[(int)(x+h*sa)+(y*ntam)];
	        			}        	
			      	res.setAllPixel(nb,aux);
        		}
      	  	else
        			res.setAllPixel(nb,bin);
					}
				}
			}
			else
			if (p4==1){	// left
				ntam=(int)(h+w*sa);
				
				if (p2==0) {
					res=new JIPImage(b,w,ntam,t);
				}
				else
				if (p2==1){
					res=new JIPImage(b,w,h,t);
				}
				else{
					System.out.println("Error, opcion de clipping incorrecta.");
					return(res);
				}
			
				for(int nb=0;nb<b;nb++){
					if (t!=JIP.tREAL){		  
  				int[] bmp=img.getAllPixel(nb);
			  	int[] bin=new int[ntam*w];
		    	for (y=0;y<h;y++){
        		for (x=0;x<w;x++){
        	  	int pos;
	      	   	if (p1>0){
	      	   		pos=(int)(y+(w-(w-x))*sa)*w+x;
	      	   		bin[pos]=bmp[x+y*w];
        	 	  }
        	 	  else{
       	  	    pos=(int)(y+(w-x)*sa)*w+x;
       	    		bin[pos]=bmp[x+y*w];
       	  	  }
       		 	}
		    	}
		    	
		    	if (p2==1){
       	 		int[] aux=new int [w*h];       	
        		for (x=0;x<w;x++)
        			for (y=0;y<h;y++){
        				if (p1>0)
        					aux[x+y*w]=bin[x+y*w];
        				else
        					aux[x+y*w]=bin[(int)(y+w*sa)*w+x];
        			}        	
		      	res.setAllPixel(nb,aux);
        	}
        	else
        		res.setAllPixel(nb,bin);
				}
				else{
					float[] bmp=img.getAllPixelFlo(nb);
			  	float[] bin=new float[ntam*w];
		    	for (y=0;y<h;y++){
        		for (x=0;x<w;x++){
        	  	int pos;
	      	   	if (p1>0){
	      	   		pos=(int)(y+(w-(w-x))*sa)*w+x;
	      	   		bin[pos]=bmp[x+y*w];
        	 	  }
        	 	  else{
       	  	    pos=(int)(y+(w-x)*sa)*w+x;
       	    		bin[pos]=bmp[x+y*w];
       	  	  }
       		 	}
		    	}
		    	
		    	if (p2==1){
       	 		float[] aux=new float [w*h];       	
        		for (x=0;x<w;x++)
        			for (y=0;y<h;y++){
        				if (p1>0)
        					aux[x+y*w]=bin[x+y*w];
        				else
        					aux[x+y*w]=bin[(int)(y+w*sa)*w+x];
        			}        	
		      	res.setAllPixel(nb,aux);
        	}
        	else
        		res.setAllPixel(nb,bin);
	
				}
			}
		}
		else
		if (p5==1){	// right
			ntam=(int)(h+w*sa);
			if (p2==0) {
				res=new JIPImage(b,w,ntam,t);
			}
			else
			if (p2==1){
				res=new JIPImage(b,w,h,t);
			}
			else{
				System.out.println("Error, opcion de clipping incorrecta.");
				return(res);
			}
			
			for(int nb=0;nb<b;nb++){
				if (t!=JIP.tREAL){		  
  				int[] bmp=img.getAllPixel(nb);
			  	int[] bin=new int[ntam*w];
			    for (y=0;y<h;y++){
      	  	for (x=0;x<w;x++){
        	  	int pos;
	        	 	if (p1>0){
	         			pos=(int)(y+(w-x)*sa)*w+x;
	         			bin[pos]=bmp[x+y*w];
         	  	}
         	  	else{
         	  	  pos=(int)(y+(w-(w-x))*sa)*w+x;
           			bin[pos]=bmp[x+y*w];
         	 	 }
        		}
		    	}
        
        	if (p2==1){
        		int[] aux=new int [w*h];       	
        		for (x=0;x<w;x++)
        			for (y=0;y<h;y++){
        				if (p1>0)
        					aux[x+y*w]=bin[x+y*w];
        				else
									aux[x+y*w]=bin[(int)(y+w*sa)*w+x];
        			}        	
		    	  res.setAllPixel(nb,aux);
        	}
        	else
        		res.setAllPixel(nb,bin);
				}
				else{
					float[] bmp=img.getAllPixelFlo(nb);
			  	float[] bin=new float [ntam*w];
			    for (y=0;y<h;y++){
      	  	for (x=0;x<w;x++){
        	  	int pos;
	        	 	if (p1>0){
	         			pos=(int)(y+(w-x)*sa)*w+x;
	         			bin[pos]=bmp[x+y*w];
         	  	}
         	  	else{
         	  	  pos=(int)(y+(w-(w-x))*sa)*w+x;
           			bin[pos]=bmp[x+y*w];
         	 	 }
        		}
		    	}
        
        	if (p2==1){
        		float[] aux=new float [w*h];       	
        		for (x=0;x<w;x++)
        			for (y=0;y<h;y++){
        				if (p1>0)
        					aux[x+y*w]=bin[x+y*w];
        				else
									aux[x+y*w]=bin[(int)(y+w*sa)*w+x];
        			}        	
		    	  res.setAllPixel(nb,aux);
        	}
        	else
        		res.setAllPixel(nb,bin);

				}
			}
		}
		if (p6==1){	// up
			ntam=(int)(w+h*sa);
			if (p2==0) {
				res=new JIPImage(b,ntam,h,t);
			}
			else
			if (p2==1){
				res=new JIPImage(b,w,h,t);
			}
			else{
				System.out.println("Error, opcion de clipping incorrecta.");
				return(res);
			}
			
			for(int nb=0;nb<b;nb++){
				if (t!=JIP.tREAL){		  
  				int[] bmp=img.getAllPixel(nb);
  				int[] bin=new int[ntam*h];
		  	  for (y=0;y<h;y++){
        		for (x=0;x<w;x++){
          		int pos;          		
          		//System.out.println("X: "+x+" Y: "+y);
          		if (p1>0){
         	    	pos=(int)(x+(h-(h-y))*sa)+(y*ntam);
           			bin[pos]=bmp[x+y*w];
	         		}
         	 	 else{
           			pos=(int)(x+(h-y)*sa)+(y*ntam);
	         			bin[pos]=bmp[x+y*w];
         	 	 }
        		}
		  	  }
        
       	 if (p2==1){
       	 	int[] aux=new int [w*h];       	
        		for (x=0;x<w;x++)
        			for (y=0;y<h;y++){
        				if (p1>0)
        					aux[x+y*w]=bin[x+y*ntam];
        				else
        					aux[x+y*w]=bin[(int)(x+h*sa)+(y*ntam)];
        			}        	
		      	res.setAllPixel(nb,aux);
       	 }
        	else
        		res.setAllPixel(nb,bin);
				}
				else{
  				float[] bmp=img.getAllPixelFlo(nb);
  				float[] bin=new float[ntam*h];
		  	  for (y=0;y<h;y++){
        		for (x=0;x<w;x++){
          		int pos;
	         		if (p1>0){
         	    	pos=(int)(x+(h-(h-y))*sa)+(y*ntam);
           			bin[pos]=bmp[x+y*w];
	         		}
         	 	 else{
           			pos=(int)(x+(h-y)*sa)+(y*ntam);
	         			bin[pos]=bmp[x+y*w];
         	 	 }
        		}
		  	  }
        
       	 if (p2==1){
       	 	float[] aux=new float [w*h];       	
        		for (x=0;x<w;x++)
        			for (y=0;y<h;y++){
        				if (p1>0)
        					aux[x+y*w]=bin[x+y*ntam];
        				else
        					aux[x+y*w]=bin[(int)(x+h*sa)+(y*ntam)];
        			}        	
		      	res.setAllPixel(nb,aux);
       	 }
        	else
        		res.setAllPixel(nb,bin);

				}
			}
		}
		System.out.println("Algoritmo Porcesado OK.");
		}
		return res;
    }
}