package jipfunc;

import jip.*;
import java.util.Vector;
import java.util.Random;

/**
* <P><FONT COLOR="red">
*<B>FSmoothGaussian:</B> Realiza el suavizado gaussiano<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Realiza el suavizado gaussiano de laimagen
*de entrada.<BR>
*Aplicable a tipos: BYTE, WORD, COLOR y REAL<BR>
*Usa: FConvolveImage.<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>sigma: Nivel de suavizado gaussiano<BR>
*<li>ejes: 0- Horizontal, 1- Vertical, 2- Ambos <BR>
*<li>bordes: ZERO-> los pixels que quedan fuera se ponen a 0.
*PAD->cuando el pixel est fuera, se toma el valor del pixel de la imagen
*ms cercano. P.e. (-1,-1) se tomara (0,0), (-2,2) se tomara (0,2), etc.
*WRAP-> cuando el pxel est fuera, se toma el valor del pixel opuesto en
*los ejes que corresponda. P.e. (-1,-1) se tomara (w,h), (0,-2) se tomara
*(0,h-1) etc.<BR><BR>

*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen Procesada del mismo tipo que la imagen de entrada<BR><BR>
*</ul>
*/


public class FSmoothGaussian extends JIPFunction {

      /**
      *<P><FONT COLOR="red">
      *Constructor de la Clase.<BR>
      *<P><FONT COLOR="blue">    
      *Obtiene los parmetros de entrada para aplicar la funcin.
      *<ul><B>Parametros de Entrada:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      *<ul><B>Parametros de Salida:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      */


    public FSmoothGaussian() {
		//Nombre y descripcion de la Funcion
	    name="FSmoothGaussian";
	    description=
	         "Suavizado gaussiano";

	    //Definicion de Parametros
		JIPParameter p1=new JIPParameter("sigma",JIP.pREAL,false);
		p1.setDefault((float)2);
		p1.setDescription("Nivel de suavizado");
    JIPParameter p2=new JIPParameter("ejes",JIP.pINT,false);
    p2.setDefault(2);
    p2.setDescription("0- Horizontal, 1- Vertical, 2- Ambos");
    JIPParameter p3=new JIPParameter("metodo",JIP.pSTRING,false);
    p3.setDefault("PAD");
    p3.setDescription("Metodo: ZERO, WRAP, PAD");
            
            

		//Asignacion de Parametros
    params=new Vector();
		params.addElement(p1);
    params.addElement(p2);
    params.addElement(p3);

    }

    /**
    *<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *Se le llama para realizar el suavizado mediante
    *la media a la imagen de entrada.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen suavizada mediante el suavizado gaussiano.<BR><BR>
    *</ul>
    *Notas Adicionales: Los clculos no se realizan en esta
    *funcin, sino que se crea un objeto de FConvolveImage, y 
    *llama a esta funcin despus de crear una imagen con los
    *datos matemticos de entrada.
    */

    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		float p1=((JIPParameter)params.elementAt(0)).getValueReal();
    int p2=((JIPParameter)params.elementAt(1)).getValueInt();
    String metodo=getParamValueStr("metodo").toUpperCase();
    if (p1==0.0) return(null);
	
		float mult,factor;
    int radio, diametro;
    
    radio   =(int)(p1*3.0+0.5);
    diametro=radio*2+1;
    float[] vector=new float[diametro];

    mult=(float)(1.0/(p1*(Math.sqrt(2.0*Math.PI)))) ;
    factor=(float)(-1.0/(2.0*p1*p1));
  
    float sum=(float)0.0;
    for(int i=0,r=-radio; r<=radio; r++, i++) 
       {
        vector[i] =(float)(mult*Math.exp(factor*r*r));
        sum+= vector[i];
       }
    for(int i=0; i<diametro; i++)  vector[i]/=sum;

    JIPFunction convolucion=new FConvolveImage();
      
    JIPImage convo=new JIPImage(diametro,1,vector);
    JIPImage convo2=new JIPImage(1,diametro,vector);
    convolucion.setParamValue("metodo",metodo);

    convolucion.setParamValue("imagen",convo); // Horizontal
    
    // LLamamos a FConvolve

    if (p2!=1) res=convolucion.processImg(img);
    
    convolucion.setParamValue("imagen",convo2); // Vertical
      
    if (p2==2) res=convolucion.processImg(res);
    else if (p2!=0) res=convolucion.processImg(img);

    
    
    

			return res;
	}
}