package jipfunc;

import jip.*;
import java.util.Vector;
import java.util.Random;

/**
*<P><FONT COLOR="red">
*<B>FSmoothRadial:</B> Realiza el suavizado (media) 
* con radio dependiente de la distancia al centro<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Realiza el suavizado (media) de la imagen,
*con el radio, dependiente de la distancia al centro de
*la imagen de entrada.
*Aplicable a tipos: BYTE, WORD, COLOR y REAL<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>radio: Radio mximo<BR>
*<li>sentido: Sentido 0 - Centro a Borde, 1 - Borde a Centro<BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen Procesada del mismo tipo que la imagen de entrada<BR><BR>
*</ul>
*/


public class FSmoothRadial extends JIPFunction {

      /**
      *<P><FONT COLOR="red">
      *Constructor de la Clase.<BR>
      *<P><FONT COLOR="blue">    
      *Obtiene los parmetros de entrada para aplicar la funcin.
      *<ul><B>Parametros de Entrada:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      *<ul><B>Parametros de Salida:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      */


    public FSmoothRadial() {
		//Nombre y descripcion de la Funcion
	    name="FSmoothRadial";
	    description=
	         "Realiza un suavizado (media) con radio dependiente de la distancia al centro";

	    //Definicion de Parametros
		JIPParameter p1=new JIPParameter("radio",JIP.pINT,false);
		p1.setDefault(5);
		p1.setDescription("Radio maximo");
    JIPParameter p2=new JIPParameter("sentido",JIP.pINT,false);
    p2.setDefault(0);
    p2.setDescription("Sentido 0 - Centro a Borde, 1 - Borde a Centro");

		//Asignacion de Parametros
        params=new Vector();
		params.addElement(p1);
    params.addElement(p2);

    }

    /**
    *<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *Se le llama para realizar el suavizado mediante
    *la media, dependiente de la distancia al centro 
    *de la imagen de entrada.<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen suavizada mediante la media, dependiente
    *de la distancia al centro.<BR><BR>
    *</ul>
    */


    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		int p1=((JIPParameter)params.elementAt(0)).getValueInt();
    int p2=((JIPParameter)params.elementAt(1)).getValueInt();

		Random rnd=new Random();
    int t=img.getType();		
    int w=img.getWidth();
		int h=img.getHeight();
		int b=img.getNumBands();
    /** FILTRAMOS LA GEOMETRIA */   
    if (t!=JIP.tBYTE && t!=JIP.tWORD
        && t!=JIP.tREAL && t!=JIP.tCOLOR) 
          
          return res;

		res=new JIPImage(b,w,h,t);
		int radio;
    float inc;
    int midX=(int)(w/2);
    int midY=(int)(h/2);
    inc=(float)(p1/Math.sqrt(Math.pow(midX,2)+Math.pow(midY,2)));
    for(int nb=0;nb<b;nb++) 
       {
        if (t!=JIP.tREAL)
           {
            int[] bmp=img.getAllPixel(nb);
            int[] bin=new int[w*h];
            for(int i=0; i<h; i++)
              for(int j=0; j<w; j++)
               {
                if (p2==0)
                  radio=(int)(inc*Math.sqrt(Math.pow(j-midX,2)+Math.pow(i-midY,2)));
                else
                  radio=(int)(p1-inc*Math.sqrt(Math.pow(j-midX,2)+Math.pow(i-midY,2)));

                int inicioX=j-radio, finX=j+radio;
                int inicioY=i-radio, finY=i+radio;
                if(inicioX<0) inicioX=0 ;  if(finX>=w) finX=w-1 ;
                if(inicioY<0) inicioY=0 ;  if(finY>=h) finY=h-1 ;
                float aux=0 ;
                int cont=0;
                for (int y=inicioY; y<=finY; y++)
                for (int x=inicioX; x<=finX; cont++, x++)
                aux+=bmp[x+y*w];
                bin[j+i*w]=(int)(aux/cont);
               }
             res.setAllPixel(nb,bin);
           }
        else
           {
            float[] bmp=img.getAllPixelFlo(nb);
            float[] bin=new float[w*h];
            for(int i=0; i<h; i++)
              for(int j=0; j<w; j++)
               {
                if (p2==0)
                  radio=(int)(inc*Math.sqrt(Math.pow(j-midX,2)+Math.pow(i-midY,2)));
                else
                  radio=(int)(p1-inc*Math.sqrt(Math.pow(j-midX,2)+Math.pow(i-midY,2)));
             
                int inicioX=j-radio, finX=j+radio;
                int inicioY=i-radio, finY=i+radio;
                if(inicioX<0) inicioX=0 ;  if(finX>=w) finX=w-1 ;
                if(inicioY<0) inicioY=0 ;  if(finY>=h) finY=h-1 ;
                float aux=0 ;
                int cont=0;
                for (int y=inicioY; y<=finY; y++)
                for (int x=inicioX; x<=finX; cont++, x++)
                aux+=bmp[x+y*w];
                bin[j+i*w]=aux/cont;
               }
             res.setAllPixel(nb,bin);
           }

		   }
	
		return res;
	}
}