package jipfunc;

import jip.*;
import java.util.Vector;

/**
* <P><FONT COLOR="red">
*<B>FWaveVert:</B> Deforma la imagen introduciendo ondas (distorsion vertical)<BR>
*<FONT COLOR="blue">
*DESCRIPCION: Deforma la imagen de entrada, introduciendo ondas
*(distorsin vertical). El numero de ondas es introducido por el
*usuario.<BR>
*Aplicable a tipos: BIT, BYTE, WORD, COLOR y REAL<BR>
*<ul><B>Parametros de Entrada:</B><BR>
*<li>img: Imagen de Entrada<BR>
*<li>nOndas: Nmero de ondas<BR>
*<li>Porcentaje: Porcentaje de distorsin<BR>
*<li>Desplazamiento: Desplazamiento de las ondas <BR><BR>
*</ul>
*<ul><B>Parametros de Salida:</B><BR>
*<li>Imagen procesada del mismo tipo que la imagen de entrada
*con la distorsin de ondas aplicada<BR><BR>
*</ul>
*/


public class FWaveVert extends JIPFunction {

      /**
      *<P><FONT COLOR="red">
      *Constructor de la Clase.<BR>
      *<P><FONT COLOR="blue">    
      *Obtiene los parmetros de entrada para aplicar la funcin.
      *<ul><B>Parametros de Entrada:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      *<ul><B>Parametros de Salida:</B><BR>
      *<li>NINGUNO<BR><BR>
      *</ul>
      */


    public FWaveVert() {
	//Nombre y descripcion de la Funcion
	    name="FWaveVert";
	    description=
	         "Deforma la imagen introduciendo ondas (distorsion vertical)";

	//Definicion de Parametros
		JIPParameter p1=new JIPParameter("nOndas",JIP.pREAL,false);
		p1.setDefault((float)5);
		p1.setDescription("Numero de ondas");
		JIPParameter p2=new JIPParameter("porcentaje",JIP.pREAL,false);
		p2.setDefault((float)5);
		p2.setDescription("Porcentaje de distorsion");
		JIPParameter p3=new JIPParameter("Desplazamiento",JIP.pREAL,false);
		p3.setDefault((float)0.0);
		p3.setDescription("Desplazamiento");

	//Asignacion de Parametros
	  params=new Vector();
		params.addElement(p1); 
		params.addElement(p2); 
		params.addElement(p3);

    }

    /**
    *<P><FONT COLOR="red">
    *Mtodo Principal<BR>
    *<P><FONT COLOR="blue">    
    *Se le llama para realizar la distorsin de ondas
    *(distorsin vertical).<BR>
    *<ul><B>Parametros de Entrada:</B><BR>
    *<li>img: Imagen a procesar<BR><BR>
    *</ul>
    *<ul><B>Parametros de Salida:</B><BR>
    *<li>Imagen distorsionada con ondas
    *(distorsin vertical)<BR><BR>
    *</ul>
    */

    
    public JIPImage processImg(JIPImage img) {
		JIPImage res=null;
		JIPImage res2=null;
		float nOndas=((JIPParameter)params.elementAt(0)).getValueReal();
		float radio1=((JIPParameter)params.elementAt(1)).getValueReal();
		float desplazamiento=((JIPParameter)params.elementAt(2)).getValueReal();
    int t=img.getType();		
    int w=img.getWidth();
    int h=img.getHeight();
    int b=img.getNumBands();
    int max;
    double frecuencia =(nOndas*Math.PI*2.0)/h;
    double desp = (desplazamiento*nOndas*Math.PI*2.0)/100.0;
    double radio = (w*radio1)/100.0;

    if (t!=JIP.tBYTE && t!=JIP.tWORD && t!=JIP.tCOLOR 
    	  && t!=JIP.tREAL && t!=JIP.tBIT) return null;
    	  	
    res=new JIPImage(b,w,h,t);

  for(int nb=0;nb<b;nb++) 
    {
     if (t!=JIP.tREAL)
       {
         int[] bmp=img.getAllPixel(nb);
         int[] bin=new int[w*h];
         for (int x=0; x<w; x++)
           {
             int yDesp = (int)Math.round(Math.sin(x*frecuencia+desp)*radio);
             for (int y=0; y<h; y++)
               {
                 if ( (yDesp>=0) && (yDesp<h) ) bin[x+y*w] = bmp[x+yDesp*w];
                 else bin[x+y*w] = 0;
                 yDesp++;
               } 
           }
         res.setAllPixel(nb,bin);
       }
     else
       {
         float[] bmp=img.getAllPixelFlo(nb);
         float[] bin=new float[w*h];
         for (int x=0; x<w; x++)
           {
             int yDesp = (int)Math.round(Math.sin(x*frecuencia+desp)*radio);
             for (int y=0; y<h; y++)
               {
                 if ( (yDesp>=0) && (yDesp<h) ) bin[x+y*w] = bmp[x+yDesp*w];
                 else bin[x+y*w] = 0;
                 yDesp++;
               } 
           }
         res.setAllPixel(nb,bin);
       }
     
     }
   return res;
   }
}